/***************************************************************************
 *   Copyright (C) 2006 - 2007 by Ian Ventura-Whiting (Fizz)               *
 *   Fizz@Titania.co.uk                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


// Access List Permission
struct accessListPermission
{
	int permission;					// perm_allow or perm_deny
	char protocol[8];
	char source[64];
	int sourceOperator;				// perm_op_lt, perm_op_gt...
	char sourceService[32];
	char sourceServiceEnd[32];
	char dest[64];
	int destOperator;				// perm_op_lt, perm_op_gt...
	char destService[32];
	char destServiceEnd[32];
	int established;				// true or false
	int fragments;					// true or false
	int log;						// true or false
	struct accessListPermission *next;
};

// Access List
struct accessList
{
	int listNumber;
	char remark[128];
	struct accessListPermission *permission;
	struct accessList *next;
};


// Process Access List
void processAccessList(char *line, struct ciscoIOSConfig *config, int lineSize, const int debugMode)
{
	// Variables...
	int listNumber = 0;
	int tempInt = 0;
	char tempString[lineSize];
	char tempString2[lineSize];
	struct accessList *accessPointer;
	struct accessListPermission *permissionPointer;
	struct ciscoCommand command;

	// Debug
	if (debugMode == true)
	{
		printf("Access-List Line: %s\n", line);
	}

	// Init
	command = splitLine(line);
	tempString[0] = 0;
	if (command.parts < 4)
		return;

	// Get access-list number
	listNumber = atoi(command.part[1]);

	// Is this the first access list?
	if (config->access == 0)
	{
		// Create storage for access-list...
		config->access = malloc(sizeof(struct accessList));
		memset(config->access, 0, sizeof(struct accessList));

		// Set pointer
		accessPointer = config->access;

		// Init
		accessPointer->listNumber = listNumber;
		accessPointer->permission = 0;
		accessPointer->next = 0;
		strcpy(accessPointer->remark, "");
	}
	else
	{
		// Search for existing access-list...
		accessPointer = config->access;
		while ((accessPointer->next != 0) && (accessPointer->listNumber != listNumber))
		{
			accessPointer = accessPointer->next;
		}

		// If access list was not found, create a new one
		if (accessPointer->listNumber != listNumber)
		{
			// Create storage for access-list...
			accessPointer->next = malloc(sizeof(struct accessList));
			memset(accessPointer->next, 0, sizeof(struct accessList));

			// Set pointer
			accessPointer = accessPointer->next;

			// Init
			accessPointer->listNumber = listNumber;
			accessPointer->permission = 0;
			accessPointer->next = 0;
			strcpy(accessPointer->remark, "");
		}
	}

	// Is the line a remark?
	if ((strncmp(command.part[2], "remark", 6) == 0) && (command.parts > 3))
	{
		int loop;
		//strncpy(accessPointer->remark, command.part[3], sizeof(accessPointer->remark));
		for (loop = 3; loop < command.parts; loop++)
		{
			sprintf(tempString2, "%s%s ",tempString, command.part[loop]);
			strcpy(tempString, tempString2);
		}
		tempString[strlen(tempString) - 1] = 0;
		strncpy(accessPointer->remark, tempString, sizeof(accessPointer->remark));
	}
	else
	{
		tempInt = 2;

		// Is the line a permit / deny line?
		if ((strncmp(command.part[tempInt], "permit", 6) == 0) || (strncmp(command.part[tempInt], "deny", 4) == 0))
		{
			// Is this the first permission
			if (accessPointer->permission == 0)
			{
				// Create a new permission storage structure
				permissionPointer = malloc(sizeof(struct accessListPermission));

				// sort out pointer
				accessPointer->permission = permissionPointer;
			}
			else
			{
				permissionPointer = accessPointer->permission;
				while (permissionPointer->next != 0)
					permissionPointer = permissionPointer->next;
				
				// Create a new permission storage structure
				permissionPointer->next = malloc(sizeof(struct accessListPermission));

				// Sort out pointer
				permissionPointer = permissionPointer->next;
			}

			// Init...
			memset(permissionPointer, 0, sizeof(struct accessListPermission));
			strcpy(permissionPointer->protocol, "");
			strcpy(permissionPointer->source, "");
			permissionPointer->sourceOperator = perm_op_none;
			strcpy(permissionPointer->sourceService, "");
			strcpy(permissionPointer->sourceServiceEnd, "");
			strcpy(permissionPointer->dest, "");
			permissionPointer->destOperator = perm_op_none;
			strcpy(permissionPointer->destService, "");
			strcpy(permissionPointer->destServiceEnd, "");
			permissionPointer->established = false;
			permissionPointer->fragments = false;
			permissionPointer->log = false;

			// Was it a permit line?
			if (strncmp(command.part[tempInt], "permit", 6) == 0)
				permissionPointer->permission = perm_permit;
			else
				permissionPointer->permission = perm_deny;
			tempInt++;

			// Is it an extended access-list?
			if (listNumber > 99)
			{
				strncpy(permissionPointer->protocol, command.part[tempInt], sizeof(permissionPointer->protocol));
				tempInt++;
			}

			// Get source
			if (strncmp(command.part[tempInt], "host", 4) == 0)
			{
				tempInt++;
				strncpy(permissionPointer->source, command.part[tempInt], sizeof(permissionPointer->source));
				tempInt++;
			}
			else if (strncmp(command.part[tempInt], "any", 3) == 0)
			{
				strncpy(permissionPointer->source, command.part[tempInt], sizeof(permissionPointer->source));
				tempInt++;
			}
			else
			{
				// Standard access-list?
				if (listNumber < 100)
				{
					strncpy(permissionPointer->source, command.part[tempInt], sizeof(permissionPointer->source));
					tempInt++;
					if ((tempInt < command.parts) && (strncmp(command.part[tempInt], "log", 3) != 0))
					{
						sprintf(permissionPointer->source, "%s %s", command.part[tempInt - 1], command.part[tempInt]);
						tempInt++;
					}
				}
				else
				{
					sprintf(permissionPointer->source, "%s %s", command.part[tempInt], command.part[tempInt + 1]);
					tempInt = tempInt + 2;
				}
			}

			// Standard access-list?
			if (listNumber < 100)
			{
				if ((tempInt < command.parts) && (strncmp(command.part[tempInt], "log", 3) == 0))
				{
					permissionPointer->log = true;
				}
				return;
			}

			// Is operator used with source?
			if (strncmp(command.part[tempInt], "lt", 2) == 0)
			{
				tempInt++;
				permissionPointer->sourceOperator = perm_op_lt;
				strncpy(permissionPointer->sourceService, command.part[tempInt], sizeof(permissionPointer->sourceService));
				tempInt++;
			}
			else if (strncmp(command.part[tempInt], "gt", 2) == 0)
			{
				tempInt++;
				permissionPointer->sourceOperator = perm_op_gt;
				strncpy(permissionPointer->sourceService, command.part[tempInt], sizeof(permissionPointer->sourceService));
				tempInt++;
			}
			else if (strncmp(command.part[tempInt], "eq", 2) == 0)
			{
				tempInt++;
				permissionPointer->sourceOperator = perm_op_eq;
				strncpy(permissionPointer->sourceService, command.part[tempInt], sizeof(permissionPointer->sourceService));
				tempInt++;
			}
			else if (strncmp(command.part[tempInt], "neq", 3) == 0)
			{
				tempInt++;
				permissionPointer->sourceOperator = perm_op_neq;
				strncpy(permissionPointer->sourceService, command.part[tempInt], sizeof(permissionPointer->sourceService));
				tempInt++;
			}
			else if (strncmp(command.part[tempInt], "range", 5) == 0)
			{
				tempInt++;
				permissionPointer->sourceOperator = perm_op_range;
				strncpy(permissionPointer->sourceService, command.part[tempInt], sizeof(permissionPointer->sourceService));
				tempInt++;
				strncpy(permissionPointer->sourceServiceEnd, command.part[tempInt], sizeof(permissionPointer->sourceServiceEnd));
				tempInt++;
			}

			// Get destination
			if (strncmp(command.part[tempInt], "host", 4) == 0)
			{
				tempInt++;
				strncpy(permissionPointer->dest, command.part[tempInt], sizeof(permissionPointer->dest));
				tempInt++;
			}
			else if (strncmp(command.part[tempInt], "any", 3) == 0)
			{
				strncpy(permissionPointer->dest, command.part[tempInt], sizeof(permissionPointer->dest));
				tempInt++;
			}
			else
			{
				sprintf(permissionPointer->dest, "%s %s", command.part[tempInt], command.part[tempInt + 1]);
				tempInt = tempInt + 2;
			}

			// If no more, then exit
			if (tempInt == command.parts)
				return;

			// Is operator used with destination?
			if (strncmp(command.part[tempInt], "lt", 2) == 0)
			{
				tempInt++;
				permissionPointer->destOperator = perm_op_lt;
				strncpy(permissionPointer->destService, command.part[tempInt], sizeof(permissionPointer->destService));
				tempInt++;
			}
			else if (strncmp(command.part[tempInt], "gt", 2) == 0)
			{
				tempInt++;
				permissionPointer->destOperator = perm_op_gt;
				strncpy(permissionPointer->destService, command.part[tempInt], sizeof(permissionPointer->destService));
				tempInt++;
			}
			else if (strncmp(command.part[tempInt], "eq", 2) == 0)
			{
				tempInt++;
				permissionPointer->destOperator = perm_op_eq;
				strncpy(permissionPointer->destService, command.part[tempInt], sizeof(permissionPointer->destService));
				tempInt++;
			}
			else if (strncmp(command.part[tempInt], "neq", 3) == 0)
			{
				tempInt++;
				permissionPointer->destOperator = perm_op_neq;
				strncpy(permissionPointer->destService, command.part[tempInt], sizeof(permissionPointer->destService));
				tempInt++;
			}
			else if (strncmp(command.part[tempInt], "range", 5) == 0)
			{
				tempInt++;
				permissionPointer->destOperator = perm_op_range;
				strncpy(permissionPointer->destService, command.part[tempInt], sizeof(permissionPointer->destService));
				tempInt++;
				strncpy(permissionPointer->destServiceEnd, command.part[tempInt], sizeof(permissionPointer->destServiceEnd));
				tempInt++;
			}

			// If no more, then exit
			if (tempInt == command.parts)
				return;

			// established?
			if (strncmp(command.part[tempInt], "established", 11) == 0)
			{
				permissionPointer->established = true;
				tempInt++;
			}

			// fragments?
			if (strncmp(command.part[tempInt], "fragments", 9) == 0)
			{
				permissionPointer->fragments = true;
				tempInt++;
			}

			// If no more, then exit
			if (tempInt == command.parts)
				return;

			// Log?
			if ((strncmp(command.part[tempInt], "log", 3) == 0) || (strncmp(command.part[tempInt], "log-input", 9) == 0))
			{
				permissionPointer->log = true;
				tempInt++;
			}
		}
	}
}

