/***************************************************************************
 *   Copyright (C) 2006 - 2007 by Ian Ventura-Whiting (Fizz)               *
 *   Fizz@Titania.co.uk                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


// FTP
struct ftpConfig
{
	char username[128];
	char password[128];
	char passwordEncrypted[128];
	int encryption;
	int weak;
	int dictionary;
	struct netInterface *interface;
};


// Process FTP
void processFtp(char *line, struct ciscoIOSConfig *config, const int debugMode, FILE *dictionaryFile)
{
	// Variables
	struct netInterface *interfacePointer;

	// Debug
	if (debugMode == true)
	{
		printf("FTP Line: %s\n", line);
	}

	// Check to see if FTP already exists
	if (config->ftp == 0)
	{
		// Create struct
		config->ftp = malloc(sizeof(struct ftpConfig));
		memset(config->ftp, 0, sizeof(struct ftpConfig));

		// Init
		config->ftp->encryption = -1;
		config->ftp->weak = false;
		config->ftp->dictionary = false;
	}

	// Check for username
	if (strncmp(line, "ip ftp username ", 16) == 0)
	{
		strncpy(config->ftp->username, line + 16, sizeof(config->ftp->username));
	}

	// Check for interface
	if (strncmp(line, "ip ftp source-interface ", 24) == 0)
	{
		// Allocate struct
		interfacePointer = malloc(sizeof(struct netInterface));
		memset(interfacePointer, 0, sizeof(struct netInterface));

		// Pointers
		interfacePointer->next = config->ftp->interface;
		config->ftp->interface = interfacePointer;

		// Copy interface
		strncpy(interfacePointer->interface, line + 24, sizeof(interfacePointer->interface));
	}

	// Check for password
	else if (strncmp("ip ftp password ", line, 16) == 0)
	{
		switch (line[16])
		{
			case '7':
				config->ftp->encryption = encrypt_type7;
				if (password7(line+18, config->ftp->password, sizeof(config->ftp->password), debugMode) != 0)
					strcpy(config->ftp->password, "<unknown>");
				strncpy(config->ftp->passwordEncrypted, line+18, sizeof(config->ftp->passwordEncrypted));
				break;

			case '5':
				config->ftp->encryption = encrypt_md5;
				strcpy(config->ftp->password, "<unknown>");
				strncpy(config->ftp->passwordEncrypted, line+18, sizeof(config->ftp->passwordEncrypted));
				addJohnPassword(config, config->ftp->username, config->ftp->passwordEncrypted);
				break;

			case '0':
				config->ftp->encryption = encrypt_none;
				strncpy(config->ftp->password, line+18, sizeof(config->ftp->password));
				strcpy(config->enable->passwordEncrypted, "");
				break;

			default:
				config->ftp->encryption = encrypt_unknown;
				strcpy(config->ftp->password, "<unknown>");
				strncpy(config->ftp->passwordEncrypted, line+18, sizeof(config->ftp->passwordEncrypted));
				break;
		}
		if ((strcmp(config->ftp->password, "<unknown>") != 0) && (strlen(config->ftp->password) != 0))
		{
			if (simplePassword(config->ftp->password, dictionaryFile) == true)
			{
				config->simplePasswords++;
				config->ftp->dictionary = true;
			}
			if (passwordStrength(config->ftp->password) == false)
			{
				config->passwordStrengths++;
				config->ftp->weak = true;
			}
		}
	}
}


