/***************************************************************************
 *   Copyright (C) 2006 - 2007 by Ian Ventura-Whiting (Fizz)               *
 *   Fizz@Titania.co.uk                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


struct ripConfig
{
	int version;
};

struct ospfConfig
{
	struct ospfNetworkConfig *ospfNetwork;
	struct ospfAreaConfig *ospfArea;
	struct ospfNeighbor *neighbor;
};

struct ospfNetworkConfig
{
	char	network[16];
	char	networkMask[16];
	char	areaID[16];
	struct ospfNetworkConfig *next;
};

struct ospfAreaConfig
{
	char	areaID[16];
	int		areaAuth;
	int		defaultCost;
	struct ospfAreaConfig *next;
};

struct ospfNeighbor
{
	char	host[16];
	int		priority;
	int		pollInterval;
	int		costNumber;
	int		databaseFilter;
	struct ospfNeighbor *next;
};

struct isisConfig
{
};

struct eigrpConfig
{
};

struct bgpNeighbor
{
	char host[32];
	int encryption;					// encrypt_type7, encrypt_md5...
	char password[128];
	char passwordEncrypted[128];
	int dictionary;
	int weak;
	char description[128];
	int autonomousNo;
	int ttlHops;
	struct bgpNeighbor *next;
};

struct bgpConfig
{
	int autonomousNo;
	int logNeighborChanges;
	int dampening;
	struct bgpNeighbor *neighbor;
};

struct staticRoute
{
	char ipAddress[20];
	char netMask[16];
	char gateway[20];
	struct staticRoute *next;
};


// Process Static Route
void processRoute(char *line, struct ciscoIOSConfig *config, int lineSize, const int debugMode)
{
	// Variables
	struct ciscoCommand command;
	struct staticRoute *routePointer;

	// Debug
	if (debugMode == true)
	{
		printf("Static Route Line: %s\n", line);
	}

	// If first?
	if (config->route == 0)
	{
		config->route = malloc(sizeof(struct staticRoute));
		routePointer = config->route;
	}
	else
	{
		routePointer = config->route;
		while (routePointer->next != 0)
			routePointer = routePointer->next;
		routePointer->next = malloc(sizeof(struct staticRoute));
		routePointer = routePointer->next;
	}

	//init
	command = splitLine(line);
	memset(routePointer, 0, sizeof(struct staticRoute));

	// IP Address
	strncpy(routePointer->ipAddress, command.part[2], sizeof(routePointer->ipAddress) - 1);

	// Net Mask
	strncpy(routePointer->netMask, command.part[3], sizeof(routePointer->netMask) - 1);

	// Gateway
	strncpy(routePointer->gateway, command.part[4], sizeof(routePointer->gateway) - 1);
}


// Process Router
void processRouter(char *line, struct ciscoIOSConfig *config, FILE *input, int lineSize, const int debugMode, FILE *dictionaryFile)
{
	// Variables
	struct ciscoCommand command;
	struct bgpNeighbor *neighborPointer;
	struct ospfNetworkConfig *ospfNetworkPointer;
	struct ospfAreaConfig *ospfAreaPointer;
	struct ospfNeighbor *ospfNeighborPointer;
	char tempString[lineSize];
	int tempInt;

	// Debug
	if (debugMode == true)
	{
		printf("Router Line: %s\n", line);
	}

	//init
	command = splitLine(line);

	// What type of routing protocol was it?
	// ISIS
	if (strncmp(line+7, "isis", 4) == 0)
	{
		readLine(input, line, lineSize);
		while ((feof(input) == 0) && (line[0] == ' '))
		{
			// Debug
			if (debugMode == true)
			{
				printf("ISIS Router Line: %s\n", line);
			}

			// Get next
			readLine(input, line, lineSize);
		}
	}

	// BGP
	else if (strncmp(line+7, "bgp", 3) == 0)
	{
		// If struct doesn't exist
		if (config->bgp == 0)
		{
			config->bgp = malloc(sizeof(struct bgpConfig));
			memset(config->bgp, 0, sizeof(struct bgpConfig));

			// Init
			config->bgp->autonomousNo = atoi(command.part[2]);
			config->bgp->logNeighborChanges = false;
			config->bgp->dampening = false;
			config->bgp->neighbor = 0;
		}

		readLine(input, line, lineSize);
		while ((feof(input) == 0) && (line[0] == ' '))
		{
			// Debug
			if (debugMode == true)
			{
				printf("BGP Router Line: %s\n", line);
			}

			// Init
			command = splitLine(line);

			if (strcmp(command.part[0], "neighbor") == 0)
			{
				// If this is the first neighbor
				if (config->bgp->neighbor == 0)
				{
					neighborPointer = malloc(sizeof(struct bgpNeighbor));
					memset(neighborPointer, 0, sizeof(struct bgpNeighbor));

					// Init
					config->bgp->neighbor = neighborPointer;
					strncpy(neighborPointer->host, command.part[1], sizeof(neighborPointer->host));
					neighborPointer->encryption = encrypt_none;
					neighborPointer->autonomousNo = -1;
					neighborPointer->ttlHops = -1;
					neighborPointer->weak = false;
					neighborPointer->dictionary = false;
				}

				// neighbors already exist
				else
				{
					// Search
					neighborPointer = config->bgp->neighbor;
					while ((neighborPointer->next != 0) && (strcmp(command.part[1], neighborPointer->host) != 0))
						neighborPointer = neighborPointer->next;

					// If not found
					if (strcmp(command.part[1], neighborPointer->host) != 0)
					{
						neighborPointer->next = malloc(sizeof(struct bgpNeighbor));
						memset(neighborPointer->next, 0, sizeof(struct bgpNeighbor));

						// Sort out pointers
						neighborPointer = neighborPointer->next;

						// Init
						strncpy(neighborPointer->host, command.part[1], sizeof(neighborPointer->host));
						neighborPointer->encryption = encrypt_none;
						neighborPointer->autonomousNo = -1;
						neighborPointer->ttlHops = -1;
						neighborPointer->weak = false;
						neighborPointer->dictionary = false;
					}
				}

				if (command.parts > 2)
				{
					// Description?
					if (strncmp(command.part[2], "description", 11) == 0)
					{
						int loop;
						strncpy(neighborPointer->description, command.part[3], sizeof(neighborPointer->description));
						tempString[0] = 0;
						for (loop = 4; loop < command.parts; loop++)
						{
							sprintf(tempString, "%s %s", neighborPointer->description, command.part[loop]);
							strncpy(neighborPointer->description, tempString, sizeof(neighborPointer->description));
						}
					}

					// Password?
					else if (strncmp(command.part[2], "password", 8) == 0)
					{
						switch(command.part[3][0])
						{
							case '7':
								neighborPointer->encryption = encrypt_type7;
								if (password7(command.part[4], neighborPointer->password, sizeof(neighborPointer->password), debugMode) != 0)
									strcpy(neighborPointer->password, "<unknown>");
								strncpy(neighborPointer->passwordEncrypted, command.part[4], sizeof(neighborPointer->passwordEncrypted));
								break;
							case '5':
								neighborPointer->encryption = encrypt_md5;
								strcpy(neighborPointer->password, "<unknown>");
								strncpy(neighborPointer->passwordEncrypted, command.part[4], sizeof(neighborPointer->passwordEncrypted));
								addJohnPassword(config, "bgp", neighborPointer->passwordEncrypted);
								break;
							case '0':
								neighborPointer->encryption = encrypt_none;
								strncpy(neighborPointer->password, command.part[4], sizeof(neighborPointer->password));
								strcpy(neighborPointer->passwordEncrypted, "");
								break;
							default:
								if (command.parts == 3)
								{
									neighborPointer->encryption = encrypt_none;
									strncpy(neighborPointer->password, command.part[3], sizeof(neighborPointer->password));
									strcpy(neighborPointer->passwordEncrypted, "");
								}
								else
								{
									neighborPointer->encryption = encrypt_unknown;
									strcpy(neighborPointer->password, "<unknown>");
									strncpy(neighborPointer->passwordEncrypted, command.part[command.parts - 1], sizeof(neighborPointer->passwordEncrypted));
								}
								break;
						}
						if ((strcmp(neighborPointer->password, "<unknown>") != 0) && (strlen(neighborPointer->password) != 0))
						{
							if (simplePassword(neighborPointer->password, dictionaryFile) == true)
							{
								config->simplePasswords++;
								neighborPointer->dictionary = true;
							}
							if (passwordStrength(neighborPointer->password) == false)
							{
								config->passwordStrengths++;
								neighborPointer->weak = true;
							}
						}
					}

					// Autonomous No?
					else if (strncmp(command.part[2], "remote-as", 9) == 0)
						neighborPointer->autonomousNo = atoi(command.part[3]);

					// TTL Hops?
					else if ((strncmp(command.part[2], "ttl-security", 12) == 0) && (strncmp(command.part[3], "hops", 4) == 0))
						neighborPointer->ttlHops = atoi(command.part[4]);
				}
			}

			else if ((strcmp(command.part[0], "bgp") == 0) && (strcmp(command.part[1], "log-neighbor-changes") == 0))
				config->bgp->logNeighborChanges = true;

			else if ((strcmp(command.part[0], "bgp") == 0) && (strcmp(command.part[1], "dampening") == 0))
				config->bgp->dampening = true;

			// Get next
			readLine(input, line, lineSize);
		}
	}

	// RIP
	else if (strncmp(line+7, "rip", 3) == 0)
	{
		readLine(input, line, lineSize);
		while ((feof(input) == 0) && (line[0] == ' '))
		{
			// Debug
			if (debugMode == true)
			{
				printf("RIP Router Line: %s\n", line);
			}

			// Get next
			readLine(input, line, lineSize);
		}
	}

	// OSPF
	else if (strncmp(line+7, "ospf", 4) == 0)
	{
		// Init OSPF struct?
		if (config->ospf == 0)
		{
			config->ospf = malloc(sizeof(struct ospfConfig));
			memset(config->ospf, 0, sizeof(struct ospfConfig));
		}

		readLine(input, line, lineSize);
		while ((feof(input) == 0) && (line[0] == ' '))
		{
			// Debug
			if (debugMode == true)
			{
				printf("OSPF Router Line: %s\n", line);
			}

			// Init
			command = splitLine(line);

			// Area
			if (strncmp(command.part[0], "area", 4) == 0)
			{
				// No area config exists...
				if (config->ospf->ospfArea == 0)
				{
					config->ospf->ospfArea = malloc(sizeof(struct ospfAreaConfig));
					ospfAreaPointer = config->ospf->ospfArea;

					// Init
					memset(ospfAreaPointer, 0, sizeof(struct ospfAreaConfig));
					strncpy(ospfAreaPointer->areaID, command.part[1], sizeof(ospfAreaPointer->areaID));
					ospfAreaPointer->areaAuth = auth_none;
					ospfAreaPointer->defaultCost = 1;
				}
				else
				{
					// Search for the area...
					ospfAreaPointer = config->ospf->ospfArea;
					while ((ospfAreaPointer->next != 0) && (strcmp(command.part[1], ospfAreaPointer->areaID) != 0))
						ospfAreaPointer = ospfAreaPointer->next;

					// If area doesn't exists, create...
					if (strcmp(command.part[1], ospfAreaPointer->areaID) != 0)
					{
						ospfAreaPointer->next = malloc(sizeof(struct ospfAreaConfig));
						ospfAreaPointer = ospfAreaPointer->next;

						// Init
						memset(ospfAreaPointer, 0, sizeof(struct ospfAreaConfig));
						strncpy(ospfAreaPointer->areaID, command.part[1], sizeof(ospfAreaPointer->areaID));
						ospfAreaPointer->areaAuth = auth_none;
						ospfAreaPointer->defaultCost = 1;
					}
				}

				// Is authentication?
				if (strcmp(command.part[2], "authentication") == 0)
				{
					ospfAreaPointer->areaAuth = auth_clear;

					// Is message digest used?
					if (command.parts > 3)
					{
						if (strcmp(command.part[3], "message-digest") == 0)
							ospfAreaPointer->areaAuth = auth_md5;
					}
				}

				// Is default cost?
				else if (strcmp(command.part[2], "default-cost") == 0)
					ospfAreaPointer->defaultCost = atoi(command.part[3]);
			}

			// Network
			else if (strncmp(command.part[0], "network", 7) == 0)
			{
				// Reserve memory
				if (config->ospf->ospfNetwork == 0)
				{
					config->ospf->ospfNetwork = malloc(sizeof(struct ospfNetworkConfig));
					ospfNetworkPointer = config->ospf->ospfNetwork;
				}
				else
				{
					ospfNetworkPointer = config->ospf->ospfNetwork;
					while (ospfNetworkPointer->next != 0)
						ospfNetworkPointer = ospfNetworkPointer->next;
					ospfNetworkPointer->next = malloc(sizeof(struct ospfNetworkConfig));
					ospfNetworkPointer = ospfNetworkPointer->next;
				}

				// Init
				memset(ospfNetworkPointer, 0, sizeof(struct ospfNetworkConfig));
				ospfNetworkPointer->next = 0;
				strncpy(ospfNetworkPointer->network, command.part[1], sizeof(ospfNetworkPointer->network));
				strncpy(ospfNetworkPointer->networkMask, command.part[2], sizeof(ospfNetworkPointer->networkMask));
				strncpy(ospfNetworkPointer->areaID, command.part[4], sizeof(ospfNetworkPointer->areaID));
			}

			// Neighbor
			else if (strncmp(command.part[0], "neighbor", 8) == 0)
			{
				// Reserve memory
				if (config->ospf->neighbor == 0)
				{
					config->ospf->neighbor = malloc(sizeof(struct ospfNeighbor));
					ospfNeighborPointer = config->ospf->neighbor;
				}
				else
				{
					ospfNeighborPointer = config->ospf->neighbor;
					while (ospfNeighborPointer->next != 0)
						ospfNeighborPointer = ospfNeighborPointer->next;
					ospfNeighborPointer->next = malloc(sizeof(struct ospfNeighbor));
					ospfNeighborPointer = ospfNeighborPointer->next;
				}

				// Init
				memset(ospfNeighborPointer, 0, sizeof(struct ospfNeighbor));
				ospfNeighborPointer->next = 0;
				strncpy(ospfNeighborPointer->host, command.part[1], sizeof(ospfNeighborPointer->host));
				ospfNeighborPointer->priority = 0;
				ospfNeighborPointer->pollInterval = 120;
				ospfNeighborPointer->costNumber = 0;
				ospfNeighborPointer->databaseFilter = false;

				for (tempInt = 2; tempInt < command.parts; tempInt = tempInt + 2)
				{
					if (strncmp(command.part[tempInt], "cost", 4) == 0)
						ospfNeighborPointer->costNumber = atoi(command.part[tempInt + 1]);

					else if (strncmp(command.part[tempInt], "priority", 8) == 0)
						ospfNeighborPointer->priority = atoi(command.part[tempInt + 1]);

					else if (strncmp(command.part[tempInt], "poll-interval", 13) == 0)
						ospfNeighborPointer->pollInterval = atoi(command.part[tempInt + 1]);

					else if ((strncmp(command.part[tempInt], "database-filter", 15) == 0) && (strncmp(command.part[tempInt + 1], "all", 3) == 0))
						ospfNeighborPointer->databaseFilter = true;
				}
			}

			// Get next
			readLine(input, line, lineSize);
		}
	}
}


