/***************************************************************************
 *   Copyright (C) 2006 - 2007 by Ian Ventura-Whiting (Fizz)               *
 *   Fizz@Titania.co.uk                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


// TACACS config
struct tacacsConfig
{
	char key[128];
	char keyEncrypted[128];
	int encryption;
	int weak;
	int dictionary;
	struct host *host;
};


// Process TACACS
void processTacacs(char *line, struct ciscoIOSConfig *config, const int debugMode, FILE *dictionaryFile)
{
	// Variables
	struct host *tacacsHostPointer;

	// Debug
	if (debugMode == true)
	{
		printf("TACACS Line: %s\n", line);
	}

	// Create logging structure
	if (config->tacacs == 0)
	{
		config->tacacs = malloc(sizeof(struct tacacsConfig));
		memset(config->tacacs, 0, sizeof(struct tacacsConfig));

		// init
		strcpy(config->tacacs->key, "");
		config->tacacs->host = 0;
		config->tacacs->dictionary = false;
		config->tacacs->weak = false;
	}

	// Key...
	if (strncmp("tacacs-server key ", line, 18) == 0)
	{
		switch (line[18])
		{
			case '7':
				config->tacacs->encryption = encrypt_type7;
				if (password7(line+20, config->tacacs->key, sizeof(config->tacacs->key), debugMode) != 0)
					strcpy(config->tacacs->key, "<unknown>");
				strncpy(config->tacacs->keyEncrypted, line + 20, sizeof(config->tacacs->keyEncrypted));
				break;

			case '5':
				config->tacacs->encryption = encrypt_md5;
				strcpy(config->tacacs->key, "<unknown>");
				strncpy(config->tacacs->keyEncrypted, line + 20, sizeof(config->tacacs->keyEncrypted));
				addJohnPassword(config, "tacacs", config->tacacs->keyEncrypted);
				break;

			case '0':
				config->tacacs->encryption = encrypt_none;
				strncpy(config->tacacs->key, line + 20, sizeof(config->tacacs->key));
				strcpy(config->tacacs->keyEncrypted, "");
				break;

			default:
				config->tacacs->encryption = encrypt_none;
				strncpy(config->tacacs->key, line + 18, sizeof(config->tacacs->key));
				strcpy(config->tacacs->keyEncrypted, "");
				break;
		}
		if ((strcmp(config->tacacs->key, "<unknown>") != 0) && (strlen(config->tacacs->key) != 0))
		{
			if (simplePassword(config->tacacs->key, dictionaryFile) == true)
			{
				config->simplePasswords++;
				config->tacacs->dictionary = true;
			}
			if (passwordStrength(config->tacacs->key) == false)
			{
				config->passwordStrengths++;
				config->tacacs->weak = true;
			}
		}
	}

	// Host...
	else if (strncmp("tacacs-server host ", line, 19) == 0)
	{
		// Create struct
		tacacsHostPointer = malloc(sizeof(struct host));
		memset(tacacsHostPointer, 0, sizeof(struct host));

		// Pointers
		tacacsHostPointer->next = config->tacacs->host;
		config->tacacs->host = tacacsHostPointer;

		// Copy host
		strncpy(tacacsHostPointer->server, line + 19, sizeof(tacacsHostPointer->server));
	}
}
