/***************************************************************************
 *   Copyright (C) 2006 - 2007 by Ian Ventura-Whiting (Fizz)               *
 *   Fizz@Titania.co.uk                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


void generateIOSReport(int reportFormat, struct ciscoIOSConfig *config, const char *outputFile, int maxSize, int passwords, int fullReport, FILE *cssFile, struct latexConfig *latex, const char *companyName, int abs, int appendix, int glossary, int commonPorts)
{
	// Variables
	FILE *reportFile;
	struct accessList *accessPointer;
	struct accessListPermission *permissionPointer;
	struct ntpHost *ntpHostPointer;
	struct ntpKey *ntpKeyPointer;
	struct ntpAccessGroup *ntpGroupPointer;
	struct enablePassword *enablePointer;
	struct usernameList *usernamePointer;
	struct privilageLevels *privilagePointer;
	struct snmpCommunity *communityPointer;
	struct snmpView *snmpViewPointer;
	struct snmpViewItem *snmpViewItemPointer;
	struct snmpGroup *snmpGroupPointer;
	struct snmpUser *snmpUserPointer;
	struct snmpTrap *snmpTrapPointer;
	struct snmpHost *snmpHostPointer;
	struct host *hostPointer;
	struct aaaGroupServer *aaaGroupPointer;
	struct aaaServer *aaaServerPointer;
	struct netInterface *interfacePointer;
	struct interfaceConfig *interfaceConfigPointer;
	struct bgpNeighbor *neighborPointer;
	struct ospfNetworkConfig *ospfNetworkPointer;
	struct ospfAreaConfig *ospfAreaPointer;
	struct ospfNeighbor *ospfNeighborPointer;
	struct keyChain *keyChainPointer;
	struct keyConfig *keyPointer;
	struct lineConfig *linePointer;
	struct bannerConfig *bannerPointer;
	struct staticRoute *routePointer;
	int tempInt;
	int tableNum = 1;
	int lineCount;
	int issues = 0;
	char tempString[maxSize];
	char tempString2[maxSize];
	char friendlyHostname[maxSize];
	char *appendixFile;

	// Counter Variables...
	int accessListIssues;
	int ipDirectedIssues;
	int timeoutIssues;
	int auxIssues;
	int ospfAuthNoneIssues;
	int ospfAuthClearIssues;
	int ospfAuthMD5;
	int redirectsIssues;
	int enableIssues;
	int proxyArpIssues;
	int unreachIssues;
	int maskReplyIssues;
	int mopIssues;
	int httpReport;

	// Table Pointers
	const char *tablerow_start;
	const char *tablerow_mid;
	const char *tablerow_end;
	switch (reportFormat)
	{
		case output_html:
			tablerow_start = table_html_rowstart;
			tablerow_mid = table_html_rowmid;
			tablerow_end = table_html_rowend;
			break;
		case output_xml:
			tablerow_start = table_xml_rowstart;
			tablerow_mid = table_xml_rowmid;
			tablerow_end = table_xml_rowend;
			break;
		case output_text:
			tablerow_start = table_text_rowstart;
			tablerow_mid = table_text_rowmid;
			tablerow_end = table_text_rowend;
			break;
		case output_latex:
			tablerow_start = table_latex_rowstart;
			tablerow_mid = table_latex_rowmid;
			tablerow_end = table_latex_rowend;
			break;
	}

	// Create / open file for output
	if (outputFile[0] == 0)
		reportFile = stdout;
	else
	{
		reportFile = fopen(outputFile, "w");
		if (reportFile == NULL)
		{
			printf("\n%sERROR: Could not create file '%s'%s\n", COL_RED, outputFile, RESET);
			return;
		}
	}

	// Set output friendly hostname
	outputFriendly(config->hostname, friendlyHostname, maxSize, reportFormat);

	// Output file heading (if required)
	output_filestart(reportFile, reportFormat, type_router, friendlyHostname, fullReport, cssFile, latex, companyName, maxSize);

	// Report section?
	if (fullReport == true)
	{

		// Output report introduction
		sprintf(tempString, "Cisco Device %s Security-Related Issues", friendlyHostname);
		output_parseTitle(tempString, reportFile, reportFormat, friendlyHostname, rate_none, section_main_sec);
		output_parseTitle("Introduction", reportFile, reportFormat, friendlyHostname, rate_none, section_security);
		output_parseText(report_security_cisco_intro, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, 0, 0);

		// ---------------------------------------
		// Get List of Issues (and generate index)
		// ---------------------------------------

		// Start of index
		switch (reportFormat)
		{
			case output_html:
				fprintf(reportFile, "<ul>\n");
				break;
		}

		// Simple Passwords
		if (config->simplePasswords > 1)
		{
			issues++;
			output_parseIndex(report_dict_title1, reportFile, reportFormat, friendlyHostname, section_security);
		}
		else if (config->simplePasswords > 0)
		{
			output_parseIndex(report_dict_title2, reportFile, reportFormat, friendlyHostname, section_security);
			issues++;
		}

		// Password Strength
		if (config->passwordStrengths > 1)
		{
			issues++;
			output_parseIndex(report_strength_title1, reportFile, reportFormat, friendlyHostname, section_security);
		}
		else if (config->passwordStrengths > 0)
		{
			output_parseIndex(report_strength_title2, reportFile, reportFormat, friendlyHostname, section_security);
			issues++;
		}

		// Auto configuration
		if ((config->bootNetwork == true) || (config->serviceConfig == true))
		{
			issues++;
			output_parseIndex(report_configauto_title, reportFile, reportFormat, friendlyHostname, section_security);
		}

		// IP directed broadcasts
		ipDirectedIssues = 0;
		interfaceConfigPointer = config->interface;
		while (interfaceConfigPointer != 0)
		{
			if ((interfaceConfigPointer->ipDirectBroadcast == true) && (interfaceConfigPointer->shutdown == false))
				ipDirectedIssues++;
			interfaceConfigPointer = interfaceConfigPointer->next;
		}
		if (ipDirectedIssues > 0)
		{
			issues++;
			output_parseIndex(report_ipdirected_title, reportFile, reportFormat, friendlyHostname, section_security);
		}

		// BGP Route Dampening
		if (config->bgp != 0)
		{
			if (config->bgp->dampening != true)
			{
				output_parseIndex(report_bgpdamp_title, reportFile, reportFormat, friendlyHostname, section_security);
				issues++;
			}
		}

		// OSPF Authentication
		ospfAuthNoneIssues = 0;
		ospfAuthClearIssues = 0;
		ospfAuthMD5 = 0;
		if (config->ospf !=0)
		{
			ospfAreaPointer = config->ospf->ospfArea;
			while (ospfAreaPointer != 0)
			{
				if (ospfAreaPointer->areaAuth == auth_none)
					ospfAuthNoneIssues++;
				else if (ospfAreaPointer->areaAuth == auth_clear)
					ospfAuthClearIssues++;
				else
					ospfAuthMD5++;
				ospfAreaPointer = ospfAreaPointer->next;
			}
		}
		if ((ospfAuthNoneIssues > 0) || (ospfAuthClearIssues > 0))
		{
			output_parseIndex(report_ospfauth_title, reportFile, reportFormat, friendlyHostname, section_security);
			issues++;
		}

		// Connection Timeout
		timeoutIssues = 0;
		linePointer = config->line;
		while (linePointer != 0)
		{
			if ((linePointer->lineLogin != false) && ((linePointer->timeout == 0) || (linePointer->timeout > connectionTimeout)) && ((linePointer->absoluteTimeout == 0) || (linePointer->absoluteTimeout > connectionTimeout)) && ((linePointer->sessionTimeout == 0) || (linePointer->sessionTimeout > connectionTimeout)) && ((linePointer->execTimeout == 0) || (linePointer->execTimeout > connectionTimeout)))
				timeoutIssues++;
			linePointer = linePointer->next;
		}
		if (timeoutIssues > 0)
		{
			output_parseIndex(report_timeout_title, reportFile, reportFormat, friendlyHostname, section_security);
			issues++;
		}

		// Aux port
		auxIssues = 0;
		linePointer = config->line;
		while (linePointer != 0)
		{
			if ((linePointer->callback == false) && (linePointer->exec == true) && (linePointer->lineType == line_aux))
				auxIssues++;
			linePointer = linePointer->next;
		}
		if (auxIssues > 0)
		{
			output_parseIndex(report_aux_title, reportFile, reportFormat, friendlyHostname, section_security);
			issues++;
		}

		// Source Routing
		if (config->sourceRouting != false)
		{
			issues++;
			output_parseIndex(report_source_title, reportFile, reportFormat, friendlyHostname, section_security);
		}

		// Finger
		if (config->finger != false)
		{
			issues++;
			output_parseIndex(report_finger_title, reportFile, reportFormat, friendlyHostname, section_security);
		}

		// HTTP
		httpReport = false;
		if (config->http == 0)
			httpReport = true;
		else if (config->http->enabled == true)
			httpReport = true;
		if (httpReport == true)
		{
			issues++;
			output_parseIndex(report_http_title, reportFile, reportFormat, friendlyHostname, section_security);
		}

		// Redirects
		redirectsIssues = 0;
		interfaceConfigPointer = config->interface;
		while (interfaceConfigPointer != 0)
		{
			if ((interfaceConfigPointer->ipRedirects == true) && (interfaceConfigPointer->shutdown == false) && (strcmp(interfaceConfigPointer->ipAddress, "None") != 0))
			{
				redirectsIssues++;
			}
			interfaceConfigPointer = interfaceConfigPointer->next;
		}
		if (redirectsIssues > 0)
		{
			issues++;
			output_parseIndex(report_redirects_title, reportFile, reportFormat, friendlyHostname, section_security);
		}

		// Access Lists
		accessListIssues = 0;
		accessPointer = config->access;
		while (accessPointer != 0)
		{
			permissionPointer = accessPointer->permission;
			while (permissionPointer != 0)
			{
				if (permissionPointer->permission == perm_permit)
				{
					// Standard Access List...
					if (accessPointer->listNumber < 100)
					{
						// From any source...
						if (strcmp(permissionPointer->source, "any") == 0)
							accessListIssues++;
					}
					// Extended Access List / old...
					else
					{
						if ((((strcmp(permissionPointer->dest, "any") == 0) && (strcmp(permissionPointer->source, "any") == 0)) || (strcmp(permissionPointer->source, "any") == 0) || (strcmp(permissionPointer->destService, "any") == 0)) && (strcmp(permissionPointer->protocol, "icmp") != 0))
							accessListIssues++;
					}
				}
				// Deny & Log last...
				if ((permissionPointer->next == 0) && ((permissionPointer->permission == perm_permit) || (permissionPointer->log == false) || (strcmp(permissionPointer->source, "any") != 0)))
				{
					if (accessPointer->listNumber < 100)
						accessListIssues++;
					else if ((strcmp(permissionPointer->dest, "any") != 0) || (permissionPointer->destOperator == perm_op_none) || (permissionPointer->sourceOperator == perm_op_none) || (strcmp(permissionPointer->destService, "any") != 0) || (strcmp(permissionPointer->protocol, "ip") != 0))
						accessListIssues++;
				}
				permissionPointer = permissionPointer->next;
			}
			accessPointer = accessPointer->next;
		}
		if (config->access == 0)
			accessListIssues = -1;
		if (accessListIssues != 0)
		{
			issues++;
			output_parseIndex(report_accesslist_title, reportFile, reportFormat, friendlyHostname, section_security);
		}

		// Proxy ARP
		proxyArpIssues = 0;
		interfaceConfigPointer = config->interface;
		while (interfaceConfigPointer != 0)
		{
			if ((interfaceConfigPointer->proxyArp == true) && (interfaceConfigPointer->shutdown == false) && (strcmp(interfaceConfigPointer->ipAddress, "None") != 0))
			{
				proxyArpIssues++;
			}
			interfaceConfigPointer = interfaceConfigPointer->next;
		}
		if (proxyArpIssues > 0)
		{
			output_parseIndex(report_proxyarp_title, reportFile, reportFormat, friendlyHostname, section_security);
			issues++;
		}

		// CDP
		if (config->cdp != false)
		{
			issues++;
			output_parseIndex(report_cdp_title, reportFile, reportFormat, friendlyHostname, section_security);
		}

		// Bootp
		if (config->bootp == true)
		{
			issues++;
			output_parseIndex(report_bootp_title, reportFile, reportFormat, friendlyHostname, section_security);
		}

		// Small Servers
		if ((config->udpSmallServers != false) || (config->tcpSmallServers != false))
		{
			issues++;
			output_parseIndex(report_small_title, reportFile, reportFormat, friendlyHostname, section_security);
		}

		// IP unreachables
		unreachIssues = 0;
		interfaceConfigPointer = config->interface;
		while (interfaceConfigPointer != 0)
		{
			if ((interfaceConfigPointer->ipUnreachables == true) && (interfaceConfigPointer->shutdown == false) && (strcmp(interfaceConfigPointer->ipAddress, "None") != 0))
			{
				unreachIssues++;
			}
			interfaceConfigPointer = interfaceConfigPointer->next;
		}
		if (unreachIssues > 0)
		{
			issues++;
			output_parseIndex(report_icmpunreach_title, reportFile, reportFormat, friendlyHostname, section_security);
		}

		// IP Mask Reply
		maskReplyIssues = 0;
		interfaceConfigPointer = config->interface;
		while (interfaceConfigPointer != 0)
		{
			if ((interfaceConfigPointer->ipMaskReply == true) && (interfaceConfigPointer->shutdown == false))
			{
				maskReplyIssues++;
			}
			interfaceConfigPointer = interfaceConfigPointer->next;
		}
		if (maskReplyIssues > 0)
		{
			issues++;
			output_parseIndex(report_maskreply_title, reportFile, reportFormat, friendlyHostname, section_security);
		}

		// Enable Secret Passwords
		enableIssues = 0;
		enablePointer = config->enable;
		while (enablePointer != 0)
		{
			if (enablePointer->secret == false)
				enableIssues++;
			enablePointer = enablePointer->next;
		}
		if (enableIssues > 0)
		{
			issues++;
			output_parseIndex(report_enablesec_title, reportFile, reportFormat, friendlyHostname, section_security);
		}

		// Password Encryption Service
		if (config->servicePasswordEncryption == false)
		{
			issues++;
			output_parseIndex(report_passenc_title, reportFile, reportFormat, friendlyHostname, section_security);
		}

		// Banners
		if (config->banner == 0)
		{
			issues++;
			output_parseIndex(report_banner_title, reportFile, reportFormat, friendlyHostname, section_security);
		}

		// PAD
		if (config->pad == true)
		{
			issues++;
			output_parseIndex(report_pad_title, reportFile, reportFormat, friendlyHostname, section_security);
		}

		// MOP
		mopIssues = 0;
		interfaceConfigPointer = config->interface;
		while (interfaceConfigPointer != 0)
		{
			if ((interfaceConfigPointer->mop == true) && (interfaceConfigPointer->shutdown == false))
			{
				mopIssues++;
			}
			interfaceConfigPointer = interfaceConfigPointer->next;
		}
		if (mopIssues > 0)
		{
			issues++;
			output_parseIndex(report_mop_title, reportFile, reportFormat, friendlyHostname, section_security);
		}

		// End of index
		switch (reportFormat)
		{
			case output_html:
				fprintf(reportFile, "</ul><br>\n");
				break;
			case output_text:
				fprintf(reportFile, "\n\n");
				break;
		}

		// If No Issues
		if (issues == 0)
			output_parseText(report_noissues, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, 0, 0);

		output_endsection(reportFile, reportFormat, subsection_end);

		// ----------------------
		// Security Issues Report
		// ----------------------

		// Dictionary-based Passwords
		if (config->simplePasswords > 0)
		{
			if (config->simplePasswords > 1)
				output_parseTitle(report_dict_title1, reportFile, reportFormat, friendlyHostname, rate_high, section_security);
			else
				output_parseTitle(report_dict_title2, reportFile, reportFormat, friendlyHostname, rate_high, section_security);
			output_parseText(report_dict_obs, reportFile, reportFormat, companyName, friendlyHostname, section_obs, rate_none, config->simplePasswords, tableNum);
			if (config->simplePasswords > 1)
			{
				output_table(reportFile, reportFormat, true, passwords, report_dict_table, &report_dict_heading, tableNum, friendlyHostname);
				output_IOSTablePasswords(config, true, passwords, reportFormat, reportFile, maxSize);
				output_table(reportFile, reportFormat, false, passwords, report_dict_table, &report_dict_heading, tableNum, friendlyHostname);
				tableNum++;
			}
			else
				output_IOSLinePasswords(config, true, passwords, reportFormat, reportFile, maxSize);
			output_endsection(reportFile, reportFormat, section_obs);
			output_parseText(report_dict_impact, reportFile, reportFormat, companyName, friendlyHostname, section_imp, rate_high, config->simplePasswords, tableNum);
			output_endsection(reportFile, reportFormat, section_imp);
			output_parseText(report_dict_ease, reportFile, reportFormat, companyName, friendlyHostname, section_ease, rate_trivial, config->simplePasswords, tableNum);
			output_endsection(reportFile, reportFormat, section_ease);
			output_parseText(report_dict_rec, reportFile, reportFormat, companyName, friendlyHostname, section_rec, rate_none, config->simplePasswords, tableNum);
			output_endsection(reportFile, reportFormat, section_rec);
		}

		// Weak Passwords
		if (config->passwordStrengths > 0)
		{
			if (config->passwordStrengths > 1)
				output_parseTitle(report_strength_title1, reportFile, reportFormat, friendlyHostname, rate_high, section_security);
			else
				output_parseTitle(report_strength_title2, reportFile, reportFormat, friendlyHostname, rate_high, section_security);
			output_parseText(report_strength_obs, reportFile, reportFormat, companyName, friendlyHostname, section_obs, rate_none, config->passwordStrengths, tableNum);
			if (config->passwordStrengths > 1)
			{
				output_table(reportFile, reportFormat, true, passwords, report_strength_table, &report_dict_heading, tableNum, friendlyHostname);
				output_IOSTablePasswords(config, false, passwords, reportFormat, reportFile, maxSize);
				output_table(reportFile, reportFormat, false, passwords, report_strength_table, &report_dict_heading, tableNum, friendlyHostname);
				tableNum++;
			}
			else
				output_IOSLinePasswords(config, false, passwords, reportFormat, reportFile, maxSize);
			output_endsection(reportFile, reportFormat, section_obs);
			output_parseText(report_strength_impact, reportFile, reportFormat, companyName, friendlyHostname, section_imp, rate_high, config->passwordStrengths, 0);
			output_endsection(reportFile, reportFormat, section_imp);
			output_parseText(report_strength_ease, reportFile, reportFormat, companyName, friendlyHostname, section_ease, rate_trivial, config->passwordStrengths, 0);
			output_endsection(reportFile, reportFormat, section_ease);
			output_parseText(report_strength_rec, reportFile, reportFormat, companyName, friendlyHostname, section_rec, rate_trivial, config->passwordStrengths, 0);
			numToWord(reportFile, passwordLength);
			fprintf(reportFile, " characters in length and contain");
			if (passwordUppers == true)
				tempInt = 1;
			else
				tempInt = 0;
			if (passwordLowers == true)
				tempInt++;
			if (passwordEither == true)
				tempInt++;
			if (passwordNumbers == true)
				tempInt++;
			if (passwordSpecials == true)
				tempInt++;
			if (passwordUppers == true)
			{
				fprintf(reportFile, " uppercase");
				if (tempInt > 2)
					fprintf(reportFile, ",");
				else if (tempInt == 2)
					fprintf(reportFile, " and");
				tempInt--;
			}
			if (passwordLowers == true)
			{
				fprintf(reportFile, " lowercase");
				if (tempInt > 2)
					fprintf(reportFile, ",");
				else if (tempInt == 2)
					fprintf(reportFile, " and");
				tempInt--;
			}
			if (passwordEither == true)
			{
				fprintf(reportFile, " either uppercase or lowercase characters");
				if (tempInt > 2)
					fprintf(reportFile, ",");
				else if (tempInt == 2)
					fprintf(reportFile, " and");
				tempInt--;
			}
			if (passwordNumbers == true)
			{
				fprintf(reportFile, " numbers");
				if (tempInt > 2)
					fprintf(reportFile, ",");
				else if (tempInt == 2)
					fprintf(reportFile, " and");
				tempInt--;
			}
			if (passwordSpecials == true)
			{
				fprintf(reportFile, " special");
				if (tempInt > 2)
					fprintf(reportFile, ",");
				else if (tempInt == 2)
					fprintf(reportFile, " and");
				tempInt--;
			}
			if (((passwordNumbers == true) && (passwordSpecials == false)) || ((passwordEither == false) && (passwordNumbers == false) && (passwordSpecials == false)))
				fprintf(reportFile, ".");
			else
				fprintf(reportFile, "characters.");
			if (reportFormat == output_xml)
				fprintf(reportFile, "</text>\n");
			output_newPara(reportFile, reportFormat);
			output_endsection(reportFile, reportFormat, section_rec);
		}

		// Configuration Auto-Loading
		if ((config->bootNetwork == true) || (config->serviceConfig == true))
		{
			output_parseTitle(report_configauto_title, reportFile, reportFormat, friendlyHostname, rate_high, section_security);
			output_parseText(report_configauto_obs, reportFile, reportFormat, companyName, friendlyHostname, section_obs, rate_none, 0, 0);
			output_endsection(reportFile, reportFormat, section_obs);
			output_parseText(report_configauto_impact, reportFile, reportFormat, companyName, friendlyHostname, section_imp, rate_high, 0, 0);
			output_endsection(reportFile, reportFormat, section_imp);
			output_parseText(report_configauto_ease, reportFile, reportFormat, companyName, friendlyHostname, section_ease, rate_trivial, 0, 0);
			output_endsection(reportFile, reportFormat, section_ease);
			output_parseText(report_configauto_rec, reportFile, reportFormat, companyName, friendlyHostname, section_rec, rate_trivial, 0, 0);
			output_endsection(reportFile, reportFormat, section_rec);
		}

		// Directed Broadcasts
		if (ipDirectedIssues > 0)
		{
			output_parseTitle(report_ipdirected_title, reportFile, reportFormat, friendlyHostname, rate_high, section_security);
			output_parseText(report_ipdirected_obs, reportFile, reportFormat, companyName, friendlyHostname, section_obs, rate_none, ipDirectedIssues, tableNum);
			if (ipDirectedIssues == 1)
			{
				
				interfaceConfigPointer = config->interface;
				while (interfaceConfigPointer != 0)
				{
					if ((interfaceConfigPointer->ipDirectBroadcast == true) && (interfaceConfigPointer->shutdown == false))
						fprintf(reportFile, "%s", interfaceConfigPointer->name);
					interfaceConfigPointer = interfaceConfigPointer->next;
				}
				fprintf(reportFile, ".");
				output_newPara(reportFile, reportFormat);
			}
			else
			{
				fprintf(reportFile, ".");
				output_newPara(reportFile, reportFormat);
				output_table(reportFile, reportFormat, true, passwords, report_ipdirected_table, &report_interface_heading, tableNum, friendlyHostname);
				interfaceConfigPointer = config->interface;
				while (interfaceConfigPointer != 0)
				{
					if ((interfaceConfigPointer->ipDirectBroadcast == true) && (interfaceConfigPointer->shutdown == false))
					{
						outputFriendly(interfaceConfigPointer->name, tempString, maxSize, reportFormat);
						fprintf(reportFile, "%s%s%s", tablerow_start, tempString, tablerow_mid);
						outputFriendly(interfaceConfigPointer->description, tempString, maxSize, reportFormat);
						fprintf(reportFile, "%s%s", tempString, tablerow_end);
					}
					interfaceConfigPointer = interfaceConfigPointer->next;
				}
				output_table(reportFile, reportFormat, false, passwords, report_ipdirected_table, &report_interface_heading, tableNum, friendlyHostname);
				tableNum++;
			}
			output_endsection(reportFile, reportFormat, section_obs);
			output_parseText(report_ipdirected_impact, reportFile, reportFormat, companyName, friendlyHostname, section_imp, rate_high, 0, 0);
			output_endsection(reportFile, reportFormat, section_imp);
			output_parseText(report_ipdirected_ease, reportFile, reportFormat, companyName, friendlyHostname, section_ease, rate_trivial, 0, 0);
			output_endsection(reportFile, reportFormat, section_ease);
			output_parseText(report_ipdirected_rec, reportFile, reportFormat, companyName, friendlyHostname, section_rec, rate_none, 0, 0);
			output_endsection(reportFile, reportFormat, section_rec);
		}

		// BGP Route Dampening
		if (config->bgp != 0)
		{
			if (config->bgp->dampening != true)
			{
				output_parseTitle(report_bgpdamp_title, reportFile, reportFormat, friendlyHostname, rate_medhigh, section_security);
				output_parseText(report_bgpdamp_obs, reportFile, reportFormat, companyName, friendlyHostname, section_obs, rate_none, 0, 0);
				output_endsection(reportFile, reportFormat, section_obs);
				output_parseText(report_bgpdamp_impact, reportFile, reportFormat, companyName, friendlyHostname, section_imp, rate_high, 0, 0);
				output_endsection(reportFile, reportFormat, section_imp);
				output_parseText(report_bgpdamp_ease, reportFile, reportFormat, companyName, friendlyHostname, section_ease, rate_moderate, 0, 0);
				output_endsection(reportFile, reportFormat, section_ease);
				output_parseText(report_bgpdamp_rec, reportFile, reportFormat, companyName, friendlyHostname, section_rec, rate_none, 0, 0);
				output_endsection(reportFile, reportFormat, section_rec);
			}
		}

		// OSPF Authentication
		if ((ospfAuthNoneIssues > 0) || (ospfAuthClearIssues > 0))
		{
			output_parseTitle(report_ospfauth_title, reportFile, reportFormat, friendlyHostname, rate_medhigh, section_security);
			output_parseText(report_ospfauth_obs1, reportFile, reportFormat, companyName, friendlyHostname, section_obs, rate_none, 0, 0);
			if (ospfAuthMD5 > 0)
				fprintf(reportFile, "%s", report_ospfauth_obs2);
			if ((ospfAuthNoneIssues > 0) && (ospfAuthClearIssues > 0))
				output_parseText(report_ospfauth_obs3, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, ospfAuthNoneIssues + ospfAuthClearIssues, tableNum);
			else if (ospfAuthNoneIssues > 0)
				output_parseText(report_ospfauth_obs4, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, ospfAuthNoneIssues, tableNum);
			else
				output_parseText(report_ospfauth_obs5, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, ospfAuthClearIssues, tableNum);
			if (ospfAuthNoneIssues + ospfAuthClearIssues > 1)
			{
				output_table(reportFile, reportFormat, true, passwords, report_ospfauth_table, &report_ospfauth_heading, tableNum, friendlyHostname);
				ospfAreaPointer = config->ospf->ospfArea;
				while (ospfAreaPointer != 0)
				{
					if (ospfAreaPointer->areaAuth == auth_none)
						fprintf(reportFile, "%s%s%sNo Authentication%s", tablerow_start, ospfAreaPointer->areaID, tablerow_mid, tablerow_end);
					else if (ospfAreaPointer->areaAuth == auth_clear)
						fprintf(reportFile, "%s%s%sClear-text Authentication%s", tablerow_start, ospfAreaPointer->areaID, tablerow_mid, tablerow_end);
					ospfAreaPointer = ospfAreaPointer->next;
				}
				output_table(reportFile, reportFormat, false, passwords, report_ospfauth_table, &report_ospfauth_heading, tableNum, friendlyHostname);
				tableNum++;
			}
			else
			{
				ospfAreaPointer = config->ospf->ospfArea;
				while (ospfAreaPointer != 0)
				{
					if (ospfAreaPointer->areaAuth == auth_none)
						fprintf(reportFile, "%s.", ospfAreaPointer->areaID);
					else if (ospfAreaPointer->areaAuth == auth_clear)
						fprintf(reportFile, "%s.", ospfAreaPointer->areaID);
					ospfAreaPointer = ospfAreaPointer->next;
				}
				output_newPara(reportFile, reportFormat);
			}
			output_endsection(reportFile, reportFormat, section_obs);
			output_parseText(report_ospfauth_impact, reportFile, reportFormat, companyName, friendlyHostname, section_imp, rate_high, 0, 0);
			output_endsection(reportFile, reportFormat, section_imp);
			output_parseText(report_ospfauth_ease, reportFile, reportFormat, companyName, friendlyHostname, section_ease, rate_moderate, 0, 0);
			output_endsection(reportFile, reportFormat, section_ease);
			output_parseText(report_ospfauth_rec, reportFile, reportFormat, companyName, friendlyHostname, section_rec, rate_none, 0, 0);
			output_endsection(reportFile, reportFormat, section_rec);
		}

		// Timeout
		if (timeoutIssues > 0)
			tableNum = output_IOSTimeout(config, reportFormat, reportFile, timeoutIssues, tableNum, friendlyHostname, companyName);

		// Auxiliary Port
		if (auxIssues > 0)
		{
			output_parseTitle(report_aux_title, reportFile, reportFormat, friendlyHostname, rate_med, section_security);
			output_parseText(report_aux_obs, reportFile, reportFormat, companyName, friendlyHostname, section_obs, rate_none, 0, 0);
			output_endsection(reportFile, reportFormat, section_obs);
			output_parseText(report_aux_impact, reportFile, reportFormat, companyName, friendlyHostname, section_imp, rate_high, 0, 0);
			output_endsection(reportFile, reportFormat, section_imp);
			output_parseText(report_aux_ease, reportFile, reportFormat, companyName, friendlyHostname, section_ease, rate_difficult, 0, 0);
			output_endsection(reportFile, reportFormat, section_ease);
			output_parseText(report_aux_rec, reportFile, reportFormat, companyName, friendlyHostname, section_rec, rate_none, 0, 0);
			output_endsection(reportFile, reportFormat, section_rec);
		}

		// IP Source Routing
		if (config->sourceRouting != false)
		{
			output_parseTitle(report_source_title, reportFile, reportFormat, friendlyHostname, rate_med, section_security);
			output_parseText(report_source_obs, reportFile, reportFormat, companyName, friendlyHostname, section_obs, rate_none, 0, 0);
			output_endsection(reportFile, reportFormat, section_obs);
			output_parseText(report_source_impact, reportFile, reportFormat, companyName, friendlyHostname, section_imp, rate_med, 0, 0);
			output_endsection(reportFile, reportFormat, section_imp);
			output_parseText(report_source_ease, reportFile, reportFormat, companyName, friendlyHostname, section_ease, rate_trivial, 0, 0);
			output_endsection(reportFile, reportFormat, section_ease);
			output_parseText(report_source_rec, reportFile, reportFormat, companyName, friendlyHostname, section_rec, rate_none, 0, 0);
			output_endsection(reportFile, reportFormat, section_rec);
		}

		// Finger
		if (config->finger != false)
		{
			output_parseTitle(report_finger_title, reportFile, reportFormat, friendlyHostname, rate_med, section_security);
			output_parseText(report_finger_obs, reportFile, reportFormat, companyName, friendlyHostname, section_obs, rate_none, 0, 0);
			output_endsection(reportFile, reportFormat, section_obs);
			output_parseText(report_finger_impact, reportFile, reportFormat, companyName, friendlyHostname, section_imp, rate_med, 0, 0);
			output_endsection(reportFile, reportFormat, section_imp);
			output_parseText(report_finger_ease, reportFile, reportFormat, companyName, friendlyHostname, section_ease, rate_trivial, 0, 0);
			output_endsection(reportFile, reportFormat, section_ease);
			output_parseText(report_finger_rec, reportFile, reportFormat, companyName, friendlyHostname, section_rec, rate_none, 0, 0);
			output_endsection(reportFile, reportFormat, section_rec);
		}

		// HTTP
		if (httpReport == true)
		{
			output_parseTitle(report_http_title, reportFile, reportFormat, friendlyHostname, rate_med, section_security);
			output_parseText(report_http_obs1, reportFile, reportFormat, companyName, friendlyHostname, section_obs, rate_none, 0, 0);
			if (config->http == 0)
				output_parseText(report_http_obs2, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, 0, 0);
			else if ((config->http->authentication == http_auth_enable) && (config->http->accessClass == 0))
				output_parseText(report_http_obs3, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, 0, 0);
			else if (config->http->accessClass == 0)
				output_parseText(report_http_obs4, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, 0, 0);
			else if (config->http->authentication != http_auth_enable)
				output_parseText(report_http_obs5, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, 0, 0);
			else
				output_parseText(report_http_obs6, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, 0, 0);
			output_endsection(reportFile, reportFormat, section_obs);
			output_parseText(report_http_impact1, reportFile, reportFormat, companyName, friendlyHostname, section_imp, rate_med, 0, 0);
			if (config->http != 0)
			{
				if (config->http->authentication == http_auth_enable)
					output_parseText(report_http_impact2, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_med, 0, 0);
			}
			else
			{
				if (reportFormat == output_xml)
					fprintf(reportFile, "</text>\n");
			}
			if (config->http != 0)
			{
				if (config->http->accessClass != 0)
					output_parseText(report_http_impact3, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_med, 0, 0);
				else
				{
					if (reportFormat == output_xml)
						fprintf(reportFile, "</text>\n");
				}
			}
			output_newPara(reportFile, reportFormat);
			output_endsection(reportFile, reportFormat, section_imp);
			output_parseText(report_http_ease1, reportFile, reportFormat, companyName, friendlyHostname, section_ease, rate_trivial, 0, 0);
			if (config->http != 0)
			{
				if (config->http->accessClass != 0)
					output_parseText(report_http_ease2, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, 0, 0);
				else
				{
					if (reportFormat == output_xml)
						fprintf(reportFile, "</text>\n");
				}
			}
			else
			{
				if (reportFormat == output_xml)
					fprintf(reportFile, "</text>\n");
			}
			output_endsection(reportFile, reportFormat, section_ease);
			output_newPara(reportFile, reportFormat);
			output_parseText(report_http_rec1, reportFile, reportFormat, companyName, friendlyHostname, section_rec, rate_none, 0, 0);
			if (config->http == 0)
				output_parseText(report_http_rec2, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, 0, 0);
			else if (config->http->accessClass != 0)
				output_parseText(report_http_rec2, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, 0, 0);
			else if (reportFormat == output_xml)
				fprintf(reportFile, "</text>\n");
			if (config->http == 0)
				output_parseText(report_http_rec3, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, 0, 0);
			else if (config->http->authentication == http_auth_enable)
				output_parseText(report_http_rec3, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, 0, 0);
			output_endsection(reportFile, reportFormat, section_rec);
		}

		// ICMP Redirects
		if (redirectsIssues > 0)
		{
			output_parseTitle(report_redirects_title, reportFile, reportFormat, friendlyHostname, rate_med, section_security);
			output_parseText(report_redirects_obs, reportFile, reportFormat, companyName, friendlyHostname, section_obs, rate_none, redirectsIssues, tableNum);
			if (redirectsIssues == 1)
			{
				interfaceConfigPointer = config->interface;
				while (interfaceConfigPointer != 0)
				{
					if ((interfaceConfigPointer->ipRedirects == true) && (interfaceConfigPointer->shutdown == false) && (strcmp(interfaceConfigPointer->ipAddress, "None") != 0))
						fprintf(reportFile, "%s.", interfaceConfigPointer->name);
					interfaceConfigPointer = interfaceConfigPointer->next;
				}
				output_newPara(reportFile, reportFormat);
			}
			else
			{
				output_table(reportFile, reportFormat, true, passwords, report_redirects_table, &report_interface_heading, tableNum, friendlyHostname);
				interfaceConfigPointer = config->interface;
				while (interfaceConfigPointer != 0)
				{
					if ((interfaceConfigPointer->ipRedirects == true) && (interfaceConfigPointer->shutdown == false) && (strcmp(interfaceConfigPointer->ipAddress, "None") != 0))
					{
						outputFriendly(interfaceConfigPointer->name, tempString, maxSize, reportFormat);
						fprintf(reportFile, "%s%s%s", tablerow_start, tempString, tablerow_mid);
						outputFriendly(interfaceConfigPointer->description, tempString, maxSize, reportFormat);
						fprintf(reportFile, "%s %s", tempString, tablerow_end);
					}
					interfaceConfigPointer = interfaceConfigPointer->next;
				}
				output_table(reportFile, reportFormat, false, passwords, report_redirects_table, &report_interface_heading, tableNum, friendlyHostname);
				tableNum++;
			}
			output_endsection(reportFile, reportFormat, section_obs);
			output_parseText(report_redirects_impact, reportFile, reportFormat, companyName, friendlyHostname, section_imp, rate_med, 0, 0);
			output_endsection(reportFile, reportFormat, section_imp);
			output_parseText(report_redirects_ease, reportFile, reportFormat, companyName, friendlyHostname, section_ease, rate_trivial, 0, 0);
			output_endsection(reportFile, reportFormat, section_ease);
			output_parseText(report_redirects_rec, reportFile, reportFormat, companyName, friendlyHostname, section_rec, rate_none, 0, 0);
			output_endsection(reportFile, reportFormat, section_rec);
		}

		// Access Lists
		if (accessListIssues != 0)
		{
			// Init...
			int denyIssue = false;
			// Output...
			output_parseTitle(report_accesslist_title, reportFile, reportFormat, friendlyHostname, rate_med, section_security);
			output_parseText(report_accesslist_obs1, reportFile, reportFormat, companyName, friendlyHostname, section_obs, rate_none, accessListIssues, tableNum);
			// One Access List issue...
			if (accessListIssues == 1)
			{
				accessPointer = config->access;
				while (accessPointer != 0)
				{
					permissionPointer = accessPointer->permission;
					lineCount = 1;
					while (permissionPointer != 0)
					{
						if (permissionPointer->permission == perm_permit)
						{
							// Standard Access List...
							if (accessPointer->listNumber < 100)
							{
								// From any source...
								if (strcmp(permissionPointer->source, "any") == 0)
								{
									fprintf(reportFile, "Line %d in ACL %d %s", lineCount, accessPointer->listNumber, report_accesslist_obs6);
									output_newPara(reportFile, reportFormat);
								}
							}
							// Extended Access List...
							else
							{
								// To any destination and service from any source...
								if ((strcmp(permissionPointer->dest, "any") == 0) && (strcmp(permissionPointer->source, "any") == 0) && (strcmp(permissionPointer->destService, "any") == 0) && (strcmp(permissionPointer->protocol, "icmp") != 0))
								{
									fprintf(reportFile, "Line %d in ACL %d %s", lineCount, accessPointer->listNumber, report_accesslist_obs8);
									output_newPara(reportFile, reportFormat);
								}

								// To any destination and service...
								else if ((strcmp(permissionPointer->dest, "any") == 0) && (strcmp(permissionPointer->destService, "any") == 0) && (strcmp(permissionPointer->protocol, "icmp") != 0))
								{
									fprintf(reportFile, "Line %d in ACL %d %s", lineCount, accessPointer->listNumber, report_accesslist_obs5);
									output_newPara(reportFile, reportFormat);
								}

								// To any destination from any source...
								else if ((strcmp(permissionPointer->dest, "any") == 0) && (strcmp(permissionPointer->source, "any") == 0) && (strcmp(permissionPointer->protocol, "icmp") != 0))
								{
									fprintf(reportFile, "Line %d in ACL %d %s", lineCount, accessPointer->listNumber, report_accesslist_obs9);
									output_newPara(reportFile, reportFormat);
								}

								// To any service...
								else if ((strcmp(permissionPointer->destService, "any") == 0) && (strcmp(permissionPointer->protocol, "icmp") != 0))
								{
									fprintf(reportFile, "Line %d in ACL %d %s", lineCount, accessPointer->listNumber, report_accesslist_obs10);
									output_newPara(reportFile, reportFormat);
								}

								// From any source...
								else if ((strcmp(permissionPointer->source, "any") == 0) && (strcmp(permissionPointer->protocol, "icmp") != 0))
								{
									fprintf(reportFile, "Line %d in ACL %d %s", lineCount, accessPointer->listNumber, report_accesslist_obs6);
									output_newPara(reportFile, reportFormat);
								}
							}
						}
						// Deny & Log last...
						if ((permissionPointer->next == 0) && ((permissionPointer->permission == perm_permit) || (permissionPointer->log == false) || (strcmp(permissionPointer->source, "any") != 0)))
						{
							if (accessPointer->listNumber < 100)
							{
								fprintf(reportFile, "%s %d %s", report_accesslist_obs11, accessPointer->listNumber, report_accesslist_obs13);
								output_newPara(reportFile, reportFormat);
								denyIssue = true;
							}
							else if ((strcmp(permissionPointer->dest, "any") != 0) || (permissionPointer->destOperator == perm_op_none) || (permissionPointer->sourceOperator == perm_op_none) || (strcmp(permissionPointer->destService, "any") != 0) || (strcmp(permissionPointer->protocol, "ip") != 0))
							{
								fprintf(reportFile, "%s %d %s", report_accesslist_obs12, accessPointer->listNumber, report_accesslist_obs13);
								output_newPara(reportFile, reportFormat);
								denyIssue = true;
							}
						}
						permissionPointer = permissionPointer->next;
						lineCount++;
					}
					accessPointer = accessPointer->next;
				}
			}

			// More than one issue...
			else if (accessListIssues != -1)
			{
				output_table(reportFile, reportFormat, true, passwords, report_accesslist_table1, &report_accesslist_heading, tableNum, friendlyHostname);
				accessPointer = config->access;
				while (accessPointer != 0)
				{
					permissionPointer = accessPointer->permission;
					lineCount = 1;
					while (permissionPointer != 0)
					{
						if (permissionPointer->permission == perm_permit)
						{
							// Standard Access List...
							if (accessPointer->listNumber < 100)
							{
								// From any source...
								if (strcmp(permissionPointer->source, "any") == 0)
									fprintf(reportFile, "%s%d%s%d%sStandard%s%s%s", tablerow_start, accessPointer->listNumber, tablerow_mid, lineCount, tablerow_mid, tablerow_mid, report_accesslist_table6, tablerow_end);
							}
							// Extended Access List...
							else
							{
								// To any destination from any source on any service...
								if ((strcmp(permissionPointer->dest, "any") == 0) && (strcmp(permissionPointer->source, "any") == 0) && ((strcmp(permissionPointer->destService, "any") == 0) || (permissionPointer->destOperator == perm_op_none)) && (strcmp(permissionPointer->protocol, "icmp") != 0))
									fprintf(reportFile, "%s%d%s%d%sExtended%s%s%s", tablerow_start, accessPointer->listNumber, tablerow_mid, lineCount, tablerow_mid, tablerow_mid, report_accesslist_table5, tablerow_end);

								// To any destination from any source...
								else if ((strcmp(permissionPointer->dest, "any") == 0) && (strcmp(permissionPointer->source, "any") == 0) && (strcmp(permissionPointer->protocol, "icmp") != 0))
									fprintf(reportFile, "%s%d%s%d%sExtended%s%s%s", tablerow_start, accessPointer->listNumber, tablerow_mid, lineCount, tablerow_mid, tablerow_mid, report_accesslist_table4, tablerow_end);

								// To any service...
								else if ((strcmp(permissionPointer->destService, "any") == 0) && (strcmp(permissionPointer->protocol, "icmp") != 0))
									fprintf(reportFile, "%s%d%s%d%sExtended%s%s%s", tablerow_start, accessPointer->listNumber, tablerow_mid, lineCount, tablerow_mid, tablerow_mid, report_accesslist_table3, tablerow_end);
							}
						}
						// Deny & Log last...
						if ((permissionPointer->next == 0) && ((permissionPointer->permission == perm_permit) || (permissionPointer->log == false) || (strcmp(permissionPointer->source, "any") != 0)))
						{
							if (accessPointer->listNumber < 100)
							{
								fprintf(reportFile, "%s%d%s %sStandard%s%s%s", tablerow_start, accessPointer->listNumber, tablerow_mid, tablerow_mid, tablerow_mid, report_accesslist_table2, tablerow_end);
								denyIssue = true;
							}
							else if ((strcmp(permissionPointer->dest, "any") != 0) || (permissionPointer->destOperator == perm_op_none) || (permissionPointer->sourceOperator == perm_op_none) || (strcmp(permissionPointer->destService, "any") != 0) || (strcmp(permissionPointer->protocol, "ip") != 0))
							{
								fprintf(reportFile, "%s%d%s %sExtended%s%s%s", tablerow_start, accessPointer->listNumber, tablerow_mid, tablerow_mid, tablerow_mid, report_accesslist_table2, tablerow_end);
								denyIssue = true;
							}
						}
						permissionPointer = permissionPointer->next;
						lineCount++;
					}
					accessPointer = accessPointer->next;
				}
				output_table(reportFile, reportFormat, false, passwords, report_accesslist_table1, &report_accesslist_heading, tableNum, friendlyHostname);
				tableNum++;
			}
			output_endsection(reportFile, reportFormat, section_obs);
			output_parseText(report_accesslist_impact, reportFile, reportFormat, companyName, friendlyHostname, section_imp, rate_med, 0, 0);
			output_endsection(reportFile, reportFormat, section_imp);
			output_parseText(report_accesslist_ease, reportFile, reportFormat, companyName, friendlyHostname, section_ease, rate_noease, 0, 0);
			output_endsection(reportFile, reportFormat, section_ease);
			if (denyIssue == true)
				output_parseText(report_accesslist_rec, reportFile, reportFormat, companyName, friendlyHostname, section_rec, rate_none, 2, 0);
			else
				output_parseText(report_accesslist_rec, reportFile, reportFormat, companyName, friendlyHostname, section_rec, rate_none, 1, 0);
			output_endsection(reportFile, reportFormat, section_rec);
		}

		// Proxy ARP
		if (proxyArpIssues > 0)
		{
			output_parseTitle(report_proxyarp_title, reportFile, reportFormat, friendlyHostname, rate_lowmed, section_security);
			output_parseText(report_proxyarp_obs, reportFile, reportFormat, companyName, friendlyHostname, section_obs, rate_none, proxyArpIssues, tableNum);
			if (proxyArpIssues > 1)
			{
				output_table(reportFile, reportFormat, true, passwords, report_proxyarp_table, &report_interface_heading, tableNum, friendlyHostname);
				interfaceConfigPointer = config->interface;
				while (interfaceConfigPointer != 0)
				{
					if ((interfaceConfigPointer->proxyArp == true) && (interfaceConfigPointer->shutdown == false) && (strcmp(interfaceConfigPointer->ipAddress, "None") != 0))
					{
						outputFriendly(interfaceConfigPointer->name, tempString, maxSize, reportFormat);
						fprintf(reportFile, "%s%s%s", tablerow_start, tempString, tablerow_mid);
						outputFriendly(interfaceConfigPointer->description, tempString, maxSize, reportFormat);
						fprintf(reportFile, "%s %s", tempString, tablerow_end);
					}
					interfaceConfigPointer = interfaceConfigPointer->next;
				}
				output_table(reportFile, reportFormat, false, passwords, report_proxyarp_table, &report_interface_heading, tableNum, friendlyHostname);
				tableNum++;
			}
			else
			{
				interfaceConfigPointer = config->interface;
				while (interfaceConfigPointer != 0)
				{
					if ((interfaceConfigPointer->proxyArp == true) && (interfaceConfigPointer->shutdown == false) && (strcmp(interfaceConfigPointer->ipAddress, "None") != 0))
						fprintf(reportFile, "%s", interfaceConfigPointer->name);
					interfaceConfigPointer = interfaceConfigPointer->next;
				}
				output_newPara(reportFile, reportFormat);
			}
			output_endsection(reportFile, reportFormat, section_obs);
			output_parseText(report_proxyarp_impact, reportFile, reportFormat, companyName, friendlyHostname, section_imp, rate_lowmed, 0, 0);
			output_endsection(reportFile, reportFormat, section_imp);
			output_parseText(report_proxyarp_ease, reportFile, reportFormat, companyName, friendlyHostname, section_ease, rate_trivial, 0, 0);
			output_endsection(reportFile, reportFormat, section_ease);
			output_parseText(report_proxyarp_rec, reportFile, reportFormat, companyName, friendlyHostname, section_rec, rate_none, 0, 0);
			output_endsection(reportFile, reportFormat, section_rec);
		}

		// CDP
		if (config->cdp != false)
		{
			output_parseTitle(report_cdp_title, reportFile, reportFormat, friendlyHostname, rate_low, section_security);

			// Look to see if CDP has been disabled on the network interfaces
			int cdpDisabled = true;
			interfaceConfigPointer = config->interface;
			while ((interfaceConfigPointer != 0) && (cdpDisabled == true))
			{
				if ((interfaceConfigPointer->cdp == true) && (interfaceConfigPointer->shutdown == false))
					cdpDisabled = false;
				interfaceConfigPointer = interfaceConfigPointer->next;
			}
			// If CDP has been disabled on all interfaces...
			if (cdpDisabled == true)
				output_parseText(report_cdp_obs, reportFile, reportFormat, companyName, friendlyHostname, section_obs, rate_none, 1, 0);
			else
				output_parseText(report_cdp_obs, reportFile, reportFormat, companyName, friendlyHostname, section_obs, rate_none, 2, 0);
			output_endsection(reportFile, reportFormat, section_obs);
			output_parseText(report_cdp_impact, reportFile, reportFormat, companyName, friendlyHostname, section_imp, rate_low, 0, 0);
			output_endsection(reportFile, reportFormat, section_imp);
			output_parseText(report_cdp_ease, reportFile, reportFormat, companyName, friendlyHostname, section_ease, rate_trivial, 0, 0);
			output_endsection(reportFile, reportFormat, section_ease);
			output_parseText(report_cdp_rec1, reportFile, reportFormat, companyName, friendlyHostname, section_rec, rate_none, 0, 0);
			output_endsection(reportFile, reportFormat, section_rec);
		}

		// Bootp
		if (config->bootp == true)
		{
			output_parseTitle(report_bootp_title, reportFile, reportFormat, friendlyHostname, rate_low, section_security);
			output_parseText(report_bootp_obs, reportFile, reportFormat, companyName, friendlyHostname, section_obs, rate_none, 0, 0);
			output_endsection(reportFile, reportFormat, section_obs);
			output_parseText(report_bootp_impact, reportFile, reportFormat, companyName, friendlyHostname, section_imp, rate_low, 0, 0);
			output_endsection(reportFile, reportFormat, section_imp);
			output_parseText(report_bootp_ease, reportFile, reportFormat, companyName, friendlyHostname, section_ease, rate_trivial, 0, 0);
			output_endsection(reportFile, reportFormat, section_ease);
			output_parseText(report_bootp_rec, reportFile, reportFormat, companyName, friendlyHostname, section_rec, rate_none, 0, 0);
			output_endsection(reportFile, reportFormat, section_rec);
		}

		// TCP & UDP Small Servers
		if ((config->udpSmallServers != false) || (config->tcpSmallServers != false))
		{
			// Check for IOS version...
			int olderIOS = false;
			if ((config->versionMajor < 11) || ((config->versionMajor == 11) && (config->versionMinor < 3)))
				olderIOS = true;
			// Report...
			output_parseTitle(report_small_title, reportFile, reportFormat, friendlyHostname, rate_low, section_security);
			output_parseText(report_small_obs1, reportFile, reportFormat, companyName, friendlyHostname, section_obs, rate_none, 0, 0);
			if (olderIOS == true)
				output_parseText(report_small_obs2, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, 0, 0);
			else if ((config->udpSmallServers == true) && (config->tcpSmallServers == true))
				output_parseText(report_small_obs3, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, 0, 0);
			else if (config->tcpSmallServers == true)
				output_parseText(report_small_obs4, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, 0, 0);
			else if (config->udpSmallServers == true)
				output_parseText(report_small_obs5, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, 0, 0);
			output_endsection(reportFile, reportFormat, section_obs);
			output_parseText(report_small_impact, reportFile, reportFormat, companyName, friendlyHostname, section_imp, rate_low, 0, 0);
			output_endsection(reportFile, reportFormat, section_imp);
			output_parseText(report_small_ease, reportFile, reportFormat, companyName, friendlyHostname, section_ease, rate_trivial, 0, 0);
			output_endsection(reportFile, reportFormat, section_ease);
			output_parseText(report_small_rec, reportFile, reportFormat, companyName, friendlyHostname, section_rec, rate_none, 0, 0);
			if ((olderIOS == true) || (config->tcpSmallServers == true))
				output_parseText(report_small_rec_tcp, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, 0, 0);
			if ((olderIOS == true) || (config->udpSmallServers == true))
				output_parseText(report_small_rec_udp, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, 0, 0);
			output_endsection(reportFile, reportFormat, section_rec);
		}

		// IP Unreachables
		if (unreachIssues > 0)
		{
			output_parseTitle(report_icmpunreach_title, reportFile, reportFormat, friendlyHostname, rate_low, section_security);
			output_parseText(report_icmpunreach_obs, reportFile, reportFormat, companyName, friendlyHostname, section_obs, rate_none, unreachIssues, tableNum);
			if (unreachIssues == 1)
			{
				interfaceConfigPointer = config->interface;
				while (interfaceConfigPointer != 0)
				{
					if ((interfaceConfigPointer->ipUnreachables == true) && (interfaceConfigPointer->shutdown == false) && (strcmp(interfaceConfigPointer->ipAddress, "None") != 0))
						fprintf(reportFile, "%s", interfaceConfigPointer->name);
					interfaceConfigPointer = interfaceConfigPointer->next;
				}
				fprintf(reportFile, ".");
				output_newPara(reportFile, reportFormat);
			}
			else
			{
				output_table(reportFile, reportFormat, true, passwords, report_icmpunreach_table, &report_interface_heading, tableNum, friendlyHostname);
				interfaceConfigPointer = config->interface;
				while (interfaceConfigPointer != 0)
				{
					if ((interfaceConfigPointer->ipUnreachables == true) && (interfaceConfigPointer->shutdown == false) && (strcmp(interfaceConfigPointer->ipAddress, "None") != 0))
					{
						outputFriendly(interfaceConfigPointer->name, tempString, maxSize, reportFormat);
						fprintf(reportFile, "%s%s%s", tablerow_start, tempString, tablerow_mid);
						outputFriendly(interfaceConfigPointer->description, tempString, maxSize, reportFormat);
						fprintf(reportFile, "%s%s", tempString, tablerow_end);
					}
					interfaceConfigPointer = interfaceConfigPointer->next;
				}
				output_table(reportFile, reportFormat, false, passwords, report_icmpunreach_table, &report_interface_heading, tableNum, friendlyHostname);
				tableNum++;
			}
			output_endsection(reportFile, reportFormat, section_obs);
			output_parseText(report_icmpunreach_impact, reportFile, reportFormat, companyName, friendlyHostname, section_imp, rate_low, 0, 0);
			output_endsection(reportFile, reportFormat, section_imp);
			output_parseText(report_icmpunreach_ease, reportFile, reportFormat, companyName, friendlyHostname, section_ease, rate_trivial, 0, 0);
			output_endsection(reportFile, reportFormat, section_ease);
			output_parseText(report_icmpunreach_rec1, reportFile, reportFormat, companyName, friendlyHostname, section_rec, rate_none, 0, 0);
			output_endsection(reportFile, reportFormat, section_rec);
		}

		// ICMP Mask Reply
		if (maskReplyIssues > 0)
		{
			output_parseTitle(report_maskreply_title, reportFile, reportFormat, friendlyHostname, rate_low, section_security);
			output_parseText(report_maskreply_obs, reportFile, reportFormat, companyName, friendlyHostname, section_obs, rate_none, maskReplyIssues, tableNum);
			if (maskReplyIssues == 1)
			{
				interfaceConfigPointer = config->interface;
				while (interfaceConfigPointer != 0)
				{
					if ((interfaceConfigPointer->ipMaskReply == true) && (interfaceConfigPointer->shutdown == false))
						fprintf(reportFile, "%s", interfaceConfigPointer->name);
					interfaceConfigPointer = interfaceConfigPointer->next;
				}
				fprintf(reportFile, ".");
				output_newPara(reportFile, reportFormat);
			}
			else
			{
				output_table(reportFile, reportFormat, true, passwords, report_maskreply_table, &report_interface_heading, tableNum, friendlyHostname);
				interfaceConfigPointer = config->interface;
				while (interfaceConfigPointer != 0)
				{
					if ((interfaceConfigPointer->ipMaskReply == true) && (interfaceConfigPointer->shutdown == false))
					{
						outputFriendly(interfaceConfigPointer->name, tempString, maxSize, reportFormat);
						fprintf(reportFile, "%s%s%s", tablerow_start, tempString, tablerow_mid);
						outputFriendly(interfaceConfigPointer->description, tempString, maxSize, reportFormat);
						fprintf(reportFile, "%s%s", tempString, tablerow_end);
					}
					interfaceConfigPointer = interfaceConfigPointer->next;
				}
				output_table(reportFile, reportFormat, false, passwords, report_icmpunreach_table, &report_interface_heading, tableNum, friendlyHostname);
				tableNum++;
			}
			output_endsection(reportFile, reportFormat, section_obs);
			output_parseText(report_maskreply_impact, reportFile, reportFormat, companyName, friendlyHostname, section_imp, rate_low, 0, 0);
			output_endsection(reportFile, reportFormat, section_imp);
			output_parseText(report_maskreply_ease, reportFile, reportFormat, companyName, friendlyHostname, section_ease, rate_trivial, 0, 0);
			output_endsection(reportFile, reportFormat, section_ease);
			output_parseText(report_maskreply_rec, reportFile, reportFormat, companyName, friendlyHostname, section_rec, rate_none, 0, 0);
			output_endsection(reportFile, reportFormat, section_rec);
		}

		// Enable Secret Passwords
		if (enableIssues > 0)
		{
			output_parseTitle(report_enablesec_title, reportFile, reportFormat, friendlyHostname, rate_low, section_security);
			output_parseText(report_enablesec_obs, reportFile, reportFormat, companyName, friendlyHostname, section_obs, rate_none, enableIssues, 0);
			output_endsection(reportFile, reportFormat, section_obs);
			output_parseText(report_enablesec_impact, reportFile, reportFormat, companyName, friendlyHostname, section_imp, rate_low, 0, 0);
			output_endsection(reportFile, reportFormat, section_imp);
			output_parseText(report_enablesec_ease, reportFile, reportFormat, companyName, friendlyHostname, section_ease, rate_trivial, 0, 0);
			output_endsection(reportFile, reportFormat, section_ease);
			output_parseText(report_enablesec_rec, reportFile, reportFormat, companyName, friendlyHostname, section_rec, rate_none, 0, 0);
			output_endsection(reportFile, reportFormat, section_rec);
		}

		// Password Encryption Service
		if (config->servicePasswordEncryption == false)
		{
			output_parseTitle(report_passenc_title, reportFile, reportFormat, friendlyHostname, rate_low, section_security);
			output_parseText(report_passenc_obs, reportFile, reportFormat, companyName, friendlyHostname, section_obs, rate_none, 0, 0);
			output_endsection(reportFile, reportFormat, section_obs);
			output_parseText(report_passenc_impact, reportFile, reportFormat, companyName, friendlyHostname, section_imp, rate_low, 0, 0);
			output_endsection(reportFile, reportFormat, section_imp);
			output_parseText(report_passenc_ease, reportFile, reportFormat, companyName, friendlyHostname, section_ease, rate_trivial, 0, 0);
			output_endsection(reportFile, reportFormat, section_ease);
			output_parseText(report_passenc_rec, reportFile, reportFormat, companyName, friendlyHostname, section_rec, rate_none, 0, 0);
			output_endsection(reportFile, reportFormat, section_rec);
		}

		// Banner
		if (config->banner == 0)
		{
			output_parseTitle(report_banner_title, reportFile, reportFormat, friendlyHostname, rate_low, section_security);
			output_parseText(report_banner_obs, reportFile, reportFormat, companyName, friendlyHostname, section_obs, rate_none, 0, 0);
			output_endsection(reportFile, reportFormat, section_obs);
			output_parseText(report_banner_impact, reportFile, reportFormat, companyName, friendlyHostname, section_imp, rate_low, 0, 0);
			output_endsection(reportFile, reportFormat, section_imp);
			output_parseText(report_banner_ease, reportFile, reportFormat, companyName, friendlyHostname, section_ease, rate_noease, 0, 0);
			output_endsection(reportFile, reportFormat, section_ease);
			output_parseText(report_banner_rec, reportFile, reportFormat, companyName, friendlyHostname, section_rec, rate_none, 0, 0);
			output_endsection(reportFile, reportFormat, section_rec);
		}

		// PAD
		if (config->pad == true)
		{
			output_parseTitle(report_pad_title, reportFile, reportFormat, friendlyHostname, rate_low, section_security);
			output_parseText(report_pad_obs, reportFile, reportFormat, companyName, friendlyHostname, section_obs, rate_none, 0, 0);
			output_endsection(reportFile, reportFormat, section_obs);
			output_parseText(report_pad_impact, reportFile, reportFormat, companyName, friendlyHostname, section_imp, rate_low, 0, 0);
			output_endsection(reportFile, reportFormat, section_imp);
			output_parseText(report_pad_ease, reportFile, reportFormat, companyName, friendlyHostname, section_ease, rate_noease, 0, 0);
			output_endsection(reportFile, reportFormat, section_ease);
			output_parseText(report_pad_rec, reportFile, reportFormat, companyName, friendlyHostname, section_rec, rate_none, 0, 0);
			output_endsection(reportFile, reportFormat, section_rec);
		}

		// MOP
		if (mopIssues > 0)
		{
			output_parseTitle(report_mop_title, reportFile, reportFormat, friendlyHostname, rate_low, section_security);
			output_parseText(report_mop_obs, reportFile, reportFormat, companyName, friendlyHostname, section_obs, rate_none, 0, 0);
			output_endsection(reportFile, reportFormat, section_obs);
			output_parseText(report_mop_impact, reportFile, reportFormat, companyName, friendlyHostname, section_imp, rate_low, 0, 0);
			output_endsection(reportFile, reportFormat, section_imp);
			output_parseText(report_mop_ease, reportFile, reportFormat, companyName, friendlyHostname, section_ease, rate_noease, 0, 0);
			output_endsection(reportFile, reportFormat, section_ease);
			output_parseText(report_mop_rec, reportFile, reportFormat, companyName, friendlyHostname, section_rec, rate_none, 0, 0);
			output_endsection(reportFile, reportFormat, section_rec);
		}

		// End Section
		output_endsection(reportFile, reportFormat, section_end);
	}

	// --------------------
	// Configuration Report
	// --------------------

	// Output report introduction
	sprintf(tempString, "Cisco Device %s Configuration", friendlyHostname);
	output_parseTitle(tempString, reportFile, reportFormat, friendlyHostname, rate_none, section_main_con);
	output_parseTitle("Introduction", reportFile, reportFormat, friendlyHostname, rate_none, section_config);
	output_parseText(report_config_cisco_intro, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, 0, 0);

	// ------------------------------------------------------
	// Get List of Configuration options (and generate index)
	// ------------------------------------------------------

	// Start of index
	switch (reportFormat)
	{
		case output_html:
			fprintf(reportFile, "<ul>\n");
			break;
	}

	// In Configuration Index
	output_parseIndex(settings_general_title, reportFile, reportFormat, friendlyHostname, section_config);
	if (((config->enable != 0) || (config->username != 0)) && (passwords == true))
		output_parseIndex(settings_users_title, reportFile, reportFormat, friendlyHostname, section_config);
	output_parseIndex(settings_services_title, reportFile, reportFormat, friendlyHostname, section_config);
	if (config->logging != 0)
		output_parseIndex(settings_logging_title, reportFile, reportFormat, friendlyHostname, section_config);
	if (config->snmp != 0)
		output_parseIndex(settings_snmp_title, reportFile, reportFormat, friendlyHostname, section_config);
	if (config->ssh != 0)
		output_parseIndex(settings_ssh_title, reportFile, reportFormat, friendlyHostname, section_config);
	if (config->ntp != 0)
		output_parseIndex(settings_ntp_title, reportFile, reportFormat, friendlyHostname, section_config);
	if (config->ftp != 0)
		output_parseIndex(settings_ftp_title, reportFile, reportFormat, friendlyHostname, section_config);
	if (config->http != 0)
		output_parseIndex(settings_http_title, reportFile, reportFormat, friendlyHostname, section_config);
	if (config->tacacs != 0)
		output_parseIndex(settings_tacacs_title, reportFile, reportFormat, friendlyHostname, section_config);
	if ((config->aaa !=0) && (config->aaa->groupServer !=0))
		output_parseIndex(settings_aaa_title, reportFile, reportFormat, friendlyHostname, section_config);
	if ((config->bgp != 0) || (config->eigrp != 0) || (config->isis != 0) || (config->rip != 0) || (config->ospf != 0) || (config->route != 0))
		output_parseIndex(settings_routing_title, reportFile, reportFormat, friendlyHostname, section_config);
	if ((passwords == true) && (config->chain != 0))
		output_parseIndex(settings_keys_title, reportFile, reportFormat, friendlyHostname, section_config);
	if (config->line != 0)
		output_parseIndex(settings_lines_title, reportFile, reportFormat, friendlyHostname, section_config);
	if (config->interface != 0)
		output_parseIndex(settings_inter_title, reportFile, reportFormat, friendlyHostname, section_config);
	if (config->access !=0)
		output_parseIndex(settings_accesslists_title, reportFile, reportFormat, friendlyHostname, section_config);

	// End of index
	switch (reportFormat)
	{
		case output_html:
			fprintf(reportFile, "</ul><br>\n");
			break;
		case output_text:
			fprintf(reportFile, "\n\n");
			break;
	}
	output_endsection(reportFile, reportFormat, subsection_end);

	// General Settings
	output_parseTitle(settings_general_title, reportFile, reportFormat, friendlyHostname, rate_none, section_config);
	output_table(reportFile, reportFormat, true, passwords, settings_general_table, &settings_general_heading, tableNum, friendlyHostname);
	fprintf(reportFile, "%sHostname%s%s%s", tablerow_start, tablerow_mid, friendlyHostname, tablerow_end);
	if (config->version[0] != 0)
		fprintf(reportFile, "%sIOS Version%s%s%s", tablerow_start, tablerow_mid, config->version, tablerow_end);
	if (config->servicePasswordEncryption == false)
		fprintf(reportFile, "%sService Password Encryption%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
	else
		fprintf(reportFile, "%sService Password Encryption%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
	if (config->sourceRouting == false)
		fprintf(reportFile, "%sIP Source Routing%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
	else
		fprintf(reportFile, "%sIP Source Routing%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
	if (config->bootp == unconfigured)
		fprintf(reportFile, "%sBOOTP%sUnconfigured%s", tablerow_start, tablerow_mid, tablerow_end);
	else if (config->bootp == false)
		fprintf(reportFile, "%sBOOTP%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
	else
		fprintf(reportFile, "%sBOOTP%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
	if (config->serviceConfig == unconfigured)
		fprintf(reportFile, "%sService Config%sUnconfigured%s", tablerow_start, tablerow_mid, tablerow_end);
	else if (config->serviceConfig == false)
		fprintf(reportFile, "%sService Config%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
	else
		fprintf(reportFile, "%sService Config%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
	if (config->domainLookup == false)
		fprintf(reportFile, "%sDomain Lookup%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
	else
		fprintf(reportFile, "%sDomain Lookup%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
	output_table(reportFile, reportFormat, false, passwords, settings_general_table, &settings_general_heading, tableNum, friendlyHostname);
	tableNum++;
	output_endsection(reportFile, reportFormat, subsection_end);
	addAbbreviation("IOS", false);
	addAbbreviation("IP", false);
	addAbbreviation("BOOTP", false);

	// Authentication
	if ((config->enable != 0) || (config->username != 0) || (config->privilage != 0) || (config->banner != 0))
	{
		output_parseTitle(settings_users_title, reportFile, reportFormat, friendlyHostname, rate_none, section_config);

		// Enable Password
		if (config->enable != 0)
		{
			output_table(reportFile, reportFormat, true, passwords, settings_enable_table, &report_enable_heading, tableNum, friendlyHostname);
			enablePointer = config->enable;
			while (enablePointer != 0)
			{
				fprintf(reportFile, "%s%d%s", tablerow_start, enablePointer->level, tablerow_mid);
				if (passwords == true)
				{
					outputFriendly(enablePointer->password, tempString, maxSize, reportFormat);
					fprintf(reportFile, "%s %s", tempString, tablerow_mid);
				}
				switch (enablePointer->encryption)
				{
					case encrypt_type7:
						fprintf(reportFile, "Type-7%s", tablerow_end);
						break;
					case encrypt_md5:
						fprintf(reportFile, "MD5%s", tablerow_end);
						addAbbreviation("MD5", false);
						break;
					case encrypt_none:
						fprintf(reportFile, "None%s", tablerow_end);
						break;
					case encrypt_unknown:
						fprintf(reportFile, "Unknown%s", tablerow_end);
						break;
				}
				enablePointer = enablePointer->next;
			}
			output_table(reportFile, reportFormat, false, passwords, settings_enable_table, &report_enable_heading, tableNum, friendlyHostname);
			tableNum++;
		}

		// Usernames
		if (config->username != 0)
		{
			output_table(reportFile, reportFormat, true, passwords, settings_users_table, &report_users_heading, tableNum, friendlyHostname);
			usernamePointer = config->username;
			while (usernamePointer != 0)
			{
				outputFriendly(usernamePointer->username, tempString, maxSize, reportFormat);
				fprintf(reportFile, "%s%s%s%d%s", tablerow_start, tempString, tablerow_mid, usernamePointer->privilage, tablerow_mid);
				if (passwords == true)
				{
					outputFriendly(usernamePointer->password, tempString, maxSize, reportFormat);
					fprintf(reportFile, "%s %s", tempString, tablerow_mid);
				}
				switch (usernamePointer->encryption)
				{
					case encrypt_type7:
						fprintf(reportFile, "Type-7%s", tablerow_end);
						break;
					case encrypt_md5:
						fprintf(reportFile, "MD5%s", tablerow_end);
						addAbbreviation("MD5", false);
						break;
					case encrypt_none:
						fprintf(reportFile, "None%s", tablerow_end);
						break;
					case encrypt_unknown:
						fprintf(reportFile, "Unknown%s", tablerow_end);
						break;
				}
				usernamePointer = usernamePointer->next;
			}
			output_table(reportFile, reportFormat, false, passwords, settings_users_table, &report_users_heading, tableNum, friendlyHostname);
			tableNum++;
		}

		// Privilage
		if (config->privilage != 0)
		{
			output_parseText(settings_privs_desc, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, 0, 0);
			output_table(reportFile, reportFormat, true, passwords, settings_privs_table, &report_privs_heading, tableNum, friendlyHostname);
			privilagePointer = config->privilage;
			while (privilagePointer != 0)
			{
				printf("%s%d%s%s%s", tablerow_start, privilagePointer->level, tablerow_mid, privilagePointer->command, tablerow_end);
				privilagePointer = privilagePointer->next;
			}
			output_table(reportFile, reportFormat, false, passwords, settings_privs_table, &report_privs_heading, tableNum, friendlyHostname);
			tableNum++;
		}

		// Banner
		if (config->banner != 0)
		{
			output_parseText(settings_banner_desc, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, 0, 0);
			bannerPointer = config->banner;
			switch (reportFormat)
			{
				case output_html:
					fprintf(reportFile, "<quote>\n");
					break;
				case output_xml:
					fprintf(reportFile, "   <code>");
					break;
			}
			while (bannerPointer != 0)
			{
				fprintf(reportFile, "%s", bannerPointer->line);
				switch (reportFormat)
				{
					case output_html:
						fprintf(reportFile, "<br>\n");
						break;
					default:
						fprintf(reportFile, "\n");
						break;
				}
				bannerPointer = bannerPointer->next;
			}
			switch (reportFormat)
			{
				case output_html:
					fprintf(reportFile, "</quote>");
					break;
				case output_xml:
					fprintf(reportFile, "</code>\n");
					break;
			}
			output_newPara(reportFile, reportFormat);
		}
		output_endsection(reportFile, reportFormat, subsection_end);
	}

	// Services
	output_parseTitle(settings_services_title, reportFile, reportFormat, friendlyHostname, rate_none, section_config);
	output_table(reportFile, reportFormat, true, passwords, settings_services_table, &report_services_heading, tableNum, friendlyHostname);
	switch (config->finger)
	{
		case unconfigured:
			fprintf(reportFile, "%sFinger%sUnconfigured%s", tablerow_start, tablerow_mid, tablerow_end);
			break;
		case false:
			fprintf(reportFile, "%sFinger%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
			break;
		case true:
			fprintf(reportFile, "%sFinger%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
			break;
	}
	if (config->ssh != 0)
	{
		switch (config->ssh->enabled)
		{
			case true:
				fprintf(reportFile, "%sSSH%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
				break;
			case false:
				fprintf(reportFile, "%sSSH%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
				break;
		}
	}
	if (config->http != 0)
	{
		switch (config->http->enabled)
		{
			case true:
				fprintf(reportFile, "%sHTTP%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
				break;
			case false:
				fprintf(reportFile, "%sHTTP%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
				break;
		}
	}
	else
		fprintf(reportFile, "%sHTTP%sUnconfigured%s", tablerow_start, tablerow_mid, tablerow_end);
	switch (config->tcpSmallServers)
	{
		case false:
			fprintf(reportFile, "%sTCP Small Services%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
			break;
		case true:
			fprintf(reportFile, "%sTCP Small Services%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
			break;
	}
	switch (config->udpSmallServers)
	{
		case false:
			fprintf(reportFile, "%sUDP Small Services%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
			break;
		case true:
			fprintf(reportFile, "%sUDP Small Services%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
			break;
	}
	if (config->ntp != 0)
	{
		if (config->ntp->enabled == true)
			fprintf(reportFile, "%sNTP%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
		else
			fprintf(reportFile, "%sNTP%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
	}
	if (config->snmp != 0)
	{
		if (config->snmp->enabled == true)
			fprintf(reportFile, "%sSNMP%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
		else
			fprintf(reportFile, "%sSNMP%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
	}
	if (config->logging != 0)
	{
		if (config->logging->localLogging == true)
			fprintf(reportFile, "%sLog Server%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
		else
			fprintf(reportFile, "%sLog Server%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
	}
	if (config->cdp == false)
		fprintf(reportFile, "%sCDP%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
	else
		fprintf(reportFile, "%sCDP%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
	if (config->pad == true)
		fprintf(reportFile, "%sPAD%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
	else
		fprintf(reportFile, "%sPAD%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
	output_table(reportFile, reportFormat, false, passwords, settings_services_table, &report_services_heading, tableNum, friendlyHostname);
	tableNum++;
	output_endsection(reportFile, reportFormat, subsection_end);
	addAbbreviation("CDP", false);
	addAbbreviation("PAD", false);
	addAbbreviation("SSH", false);
	addAbbreviation("NTP", false);
	addAbbreviation("SNMP", false);
	addAbbreviation("UDP", false);
	addAbbreviation("TCP", false);
	addAbbreviation("HTTP", false);

	// Logging
	if (config->logging != 0)
	{
		output_parseTitle(settings_logging_title, reportFile, reportFormat, friendlyHostname, rate_none, section_config);
		output_table(reportFile, reportFormat, true, passwords, settings_logging_table, &settings_general_heading, tableNum, friendlyHostname);
		if (config->logging->localLogging == true)
			fprintf(reportFile, "%sLog Server%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
		else
			fprintf(reportFile, "%sLog Server%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
		if (config->logging->consoleLogging == true)
			fprintf(reportFile, "%sConsole Logging%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
		else
			fprintf(reportFile, "%sConsole Logging%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
		output_table(reportFile, reportFormat, false, passwords, settings_logging_table, &settings_general_heading, tableNum, friendlyHostname);
		tableNum++;

		if (config->logging->logServer != 0)
		{
			output_table(reportFile, reportFormat, true, passwords, settings_logserv_table, &report_logserv_heading, tableNum, friendlyHostname);
			hostPointer = config->logging->logServer;
			while (hostPointer != 0)
			{
				fprintf(reportFile, "%s%s%s", tablerow_start, hostPointer->server, tablerow_end);
				hostPointer = hostPointer->next;
			}
			output_table(reportFile, reportFormat, false, passwords, settings_logserv_table, &report_logserv_heading, tableNum, friendlyHostname);
			tableNum++;
		}
		output_endsection(reportFile, reportFormat, subsection_end);
	}

	// SNMP
	if (config->snmp != 0)
	{
		output_parseTitle(settings_snmp_title, reportFile, reportFormat, friendlyHostname, rate_none, section_config);
		output_table(reportFile, reportFormat, true, passwords, settings_snmp_table, &settings_general_heading, tableNum, friendlyHostname);
		if (config->snmp->enabled == true)
			fprintf(reportFile, "%sService enabled%sYes%s", tablerow_start, tablerow_mid, tablerow_end);
		else
			fprintf(reportFile, "%sService enabled%sNo%s", tablerow_start, tablerow_mid, tablerow_end);
		if (config->snmp->tftpServerList == true)
			fprintf(reportFile, "%sTFTP Server List%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
		else
			fprintf(reportFile, "%sTFTP Server List%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
		outputFriendly(config->snmp->contact, tempString, maxSize, reportFormat);
		fprintf(reportFile, "%sContact%s%s%s", tablerow_start, tablerow_mid, tempString, tablerow_end);
		outputFriendly(config->snmp->location, tempString, maxSize, reportFormat);
		fprintf(reportFile, "%sLocation%s%s%s", tablerow_start, tablerow_mid, tempString, tablerow_end);
		output_table(reportFile, reportFormat, false, passwords, settings_snmp_table, &settings_general_heading, tableNum, friendlyHostname);
		tableNum++;
		addAbbreviation("SNMP", false);
		addAbbreviation("TFTP", false);

		if (config->snmp->community != 0)
		{
			output_table(reportFile, reportFormat, true, passwords, settings_snmpstring_table, &settings_snmpstring_heading, tableNum, friendlyHostname);
			communityPointer = config->snmp->community;
			while (communityPointer != 0)
			{
				fprintf(reportFile, "%s", tablerow_start);
				if (passwords == true)
				{
					outputFriendly(communityPointer->community, tempString, maxSize, reportFormat);
					fprintf(reportFile, "%s %s", tempString, tablerow_mid);
				}
				if (communityPointer->readonly == true)
					fprintf(reportFile, "Read-Only%s%s%s%d%s", tablerow_mid, communityPointer->viewName, tablerow_mid, communityPointer->accessList, tablerow_mid);
				else
					fprintf(reportFile, "Read/Write%s%s%s%d%s", tablerow_mid, communityPointer->viewName, tablerow_mid, communityPointer->accessList, tablerow_mid);
				if (communityPointer->disabled == true)
					fprintf(reportFile, "No%s", tablerow_end);
				else
					fprintf(reportFile, "Yes%s", tablerow_end);
				communityPointer = communityPointer->next;
			}
			output_table(reportFile, reportFormat, false, passwords, settings_snmpstring_table, &settings_snmpstring_heading, tableNum, friendlyHostname);
			tableNum++;
		}

		if (config->snmp->view != 0)
		{
			snmpViewPointer = config->snmp->view;
			while (snmpViewPointer != 0)
			{
				sprintf(tempString, "%s %s", settings_snmpview_table, snmpViewPointer->viewName);
				outputFriendly(tempString, tempString2, maxSize, reportFormat);
				output_table(reportFile, reportFormat, true, passwords, tempString2, &settings_snmpview_heading, tableNum, friendlyHostname);
				snmpViewItemPointer = snmpViewPointer->viewItem;
				while (snmpViewItemPointer != 0)
				{
					fprintf(reportFile, "%s%s%s", tablerow_start, snmpViewItemPointer->mibId, tablerow_mid);
					if (snmpViewItemPointer->include == true)
						fprintf(reportFile, "Include%s", tablerow_end);
					else
						fprintf(reportFile, "Exclude%s", tablerow_end);
					snmpViewItemPointer = snmpViewItemPointer->next;
				}
				output_table(reportFile, reportFormat, false, passwords, tempString2, &settings_snmpview_heading, tableNum, friendlyHostname);
				tableNum++;
				snmpViewPointer = snmpViewPointer->next;
			}
			addAbbreviation("MIB", false);
		}

		if (config->snmp->group != 0)
		{
			snmpGroupPointer = config->snmp->group;
			while (snmpGroupPointer != 0)
			{
				switch (snmpGroupPointer->snmpSecurity)
				{
					case snmp3_no_auth:
						sprintf(tempString, "%s%s%s%s", settings_snmpgroup_table1, snmpGroupPointer->groupName, settings_snmpgroup_table2, settings_snmpgroup_table3);
						outputFriendly(tempString, tempString2, maxSize, reportFormat);
						output_table(reportFile, reportFormat, true, passwords, tempString2, &settings_snmpnoauth_heading, tableNum, friendlyHostname);
						snmpUserPointer = snmpGroupPointer->user;
						while (snmpUserPointer != 0)
						{
							outputFriendly(snmpUserPointer->userName, tempString, maxSize, reportFormat);
							fprintf(reportFile, "%s%s%s", tablerow_start, tempString, tablerow_end);
							snmpUserPointer = snmpUserPointer->next;
						}
						output_table(reportFile, reportFormat, false, passwords, tempString2, &settings_snmpnoauth_heading, tableNum, friendlyHostname);
						break;
					case snmp3_auth:
						sprintf(tempString, "%s%s%s%s", settings_snmpgroup_table1, snmpGroupPointer->groupName, settings_snmpgroup_table2, settings_snmpgroup_table4);
						outputFriendly(tempString, tempString2, maxSize, reportFormat);
						output_table(reportFile, reportFormat, true, passwords, tempString2, &settings_snmpauth_heading, tableNum, friendlyHostname);
						snmpUserPointer = snmpGroupPointer->user;
						while (snmpUserPointer != 0)
						{
							outputFriendly(snmpUserPointer->userName, tempString, maxSize, reportFormat);
							fprintf(reportFile, "%s%s", tablerow_start, tempString);
							if (passwords == true)
							{
								outputFriendly(snmpUserPointer->authPass, tempString, maxSize, reportFormat);
								fprintf(reportFile, "%s%s", tablerow_mid, tempString);
							}
							fprintf(reportFile, "%s", tablerow_end);
							snmpUserPointer = snmpUserPointer->next;
						}
						output_table(reportFile, reportFormat, false, passwords, tempString2, &settings_snmpauth_heading, tableNum, friendlyHostname);
						addAbbreviation("MD5", false);
						break;
					case snmp3_priv:
						sprintf(tempString, "%s%s%s%s", settings_snmpgroup_table1, snmpGroupPointer->groupName, settings_snmpgroup_table2, settings_snmpgroup_table5);
						outputFriendly(tempString, tempString2, maxSize, reportFormat);
						output_table(reportFile, reportFormat, true, passwords, tempString2, &settings_snmppriv_heading, tableNum, friendlyHostname);
						snmpUserPointer = snmpGroupPointer->user;
						while (snmpUserPointer != 0)
						{
							outputFriendly(snmpUserPointer->userName, tempString, maxSize, reportFormat);
							fprintf(reportFile, "%s%s", tablerow_start, tempString);
							if (passwords == true)
							{
								outputFriendly(snmpUserPointer->authPass, tempString, maxSize, reportFormat);
								fprintf(reportFile, "%s%s ", tablerow_mid, tempString);
								outputFriendly(snmpUserPointer->privPass, tempString, maxSize, reportFormat);
								fprintf(reportFile, "%s%s ", tablerow_mid, tempString);
							}
							fprintf(reportFile, "%s", tablerow_end);
							snmpUserPointer = snmpUserPointer->next;
						}
						output_table(reportFile, reportFormat, false, passwords, tempString2, &settings_snmppriv_heading, tableNum, friendlyHostname);
						addAbbreviation("MD5", false);
						addAbbreviation("DES", false);
						break;
				}
				tableNum++;
				snmpGroupPointer = snmpGroupPointer->next;
			}
		}

		if (config->snmp->host != 0)
		{
			output_table(reportFile, reportFormat, true, passwords, settings_snmphost_table, &settings_snmphost_heading, tableNum, friendlyHostname);
			snmpHostPointer = config->snmp->host;
			while (snmpHostPointer != 0)
			{
				outputFriendly(snmpHostPointer->host, tempString, maxSize, reportFormat);
				fprintf(reportFile, "%s%s", tablerow_start, tempString);
				if (passwords == true)
				{
					outputFriendly(snmpHostPointer->community, tempString, maxSize, reportFormat);
					fprintf(reportFile, "%s%s ", tablerow_mid, tempString);
				}
				fprintf(reportFile, "%s", tablerow_end);
				snmpHostPointer = snmpHostPointer->next;
			}
			output_table(reportFile, reportFormat, false, passwords, settings_snmphost_table, &settings_snmphost_heading, tableNum, friendlyHostname);
			tableNum++;
		}

		if (config->snmp->trap != 0)
		{
			output_table(reportFile, reportFormat, true, passwords, settings_snmptrap_table, &settings_snmptrap_heading, tableNum, friendlyHostname);
			snmpTrapPointer = config->snmp->trap;
			while (snmpTrapPointer != 0)
			{
				outputFriendly(snmpTrapPointer->trap, tempString, maxSize, reportFormat);
				fprintf(reportFile, "%s%s%s", tablerow_start, tempString, tablerow_end);
				snmpTrapPointer = snmpTrapPointer->next;
			}
			output_table(reportFile, reportFormat, false, passwords, settings_snmptrap_table, &settings_snmptrap_heading, tableNum, friendlyHostname);
			tableNum++;
		}

		output_endsection(reportFile, reportFormat, subsection_end);
	}

	// SSH
	if (config->ssh != 0)
	{
		output_parseTitle(settings_ssh_title, reportFile, reportFormat, friendlyHostname, rate_none, section_config);
		output_table(reportFile, reportFormat, true, passwords, settings_ssh_table, &settings_general_heading, tableNum, friendlyHostname);
		switch (config->ssh->protocolVersion)
		{
			case ssh_ver_1:
				fprintf(reportFile, "%sProtocol version%s1%s", tablerow_start, tablerow_mid, tablerow_end);
				break;
			case ssh_ver_2:
				fprintf(reportFile, "%sProtocol version%s2%s", tablerow_start, tablerow_mid, tablerow_end);
				break;
			default:
				fprintf(reportFile, "%sProtocol version%sDefault%s", tablerow_start, tablerow_mid, tablerow_end);
				break;
		}
		if (config->ssh->timeout > -1)
			fprintf(reportFile, "%sLogin time-out%s%d%s", tablerow_start, tablerow_mid, config->ssh->timeout, tablerow_end);
		else
			fprintf(reportFile, "%sLogin time-out%sDefault%s", tablerow_start, tablerow_mid, tablerow_end);
		if (config->ssh->retries > -1)
			fprintf(reportFile, "%sLogin retries%s%d%s", tablerow_start, tablerow_mid, config->ssh->retries, tablerow_end);
		else
			fprintf(reportFile, "%sLogin retries%sDefault%s", tablerow_start, tablerow_mid, tablerow_end);
		output_table(reportFile, reportFormat, false, passwords, settings_ssh_table, &settings_general_heading, tableNum, friendlyHostname);
		tableNum++;
		addAbbreviation("SSH", false);

		// Show SSH interfaces
		if (config->ssh->interface != 0)
		{
			output_table(reportFile, reportFormat, true, passwords, settings_sshint_table, &settings_sshint_heading, tableNum, friendlyHostname);
			fprintf(reportFile, "<table><caption>Table %d: %s.</caption><thead>\n<tr><th>Network Interface</th></tr></thead><tbody>\n", tableNum, settings_sshint_table);
			interfacePointer = config->ssh->interface;
			while (interfacePointer != 0)
			{
				outputFriendly(interfacePointer->interface, tempString, maxSize, reportFormat);
				fprintf(reportFile, "%s%s%s", tablerow_start, tempString, tablerow_end);
				interfacePointer = interfacePointer->next;
			}
			output_table(reportFile, reportFormat, false, passwords, settings_sshint_table, &settings_sshint_heading, tableNum, friendlyHostname);
			tableNum++;
		}

		output_endsection(reportFile, reportFormat, subsection_end);
	}

	// NTP
	if (config->ntp != 0)
	{
		output_parseTitle(settings_ntp_title, reportFile, reportFormat, friendlyHostname, rate_none, section_config);
		output_table(reportFile, reportFormat, true, passwords, settings_ntp_table, &settings_general_heading, tableNum, friendlyHostname);
		if (config->ntp->enabled == true)
			fprintf(reportFile, "%sNTP Server%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
		else
			fprintf(reportFile, "%sNTP Server%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
		if (config->ntp->master == true)
			fprintf(reportFile, "%sNTP Master%sYes%s", tablerow_start, tablerow_mid, tablerow_end);
		else
			fprintf(reportFile, "%sNTP Master%sNo%s", tablerow_start, tablerow_mid, tablerow_end);
		if (config->ntp->authentication == true)
			fprintf(reportFile, "%sNTP Authentication%sYes%s", tablerow_start, tablerow_mid, tablerow_end);
		else
			fprintf(reportFile, "%sNTP Authentication%sNo%s", tablerow_start, tablerow_mid, tablerow_end);
		output_table(reportFile, reportFormat, false, passwords, settings_ntp_table, &settings_general_heading, tableNum, friendlyHostname);
		tableNum++;
		addAbbreviation("NTP", false);

		if (config->ntp->server != 0)
		{
			output_table(reportFile, reportFormat, true, passwords, settings_ntpserv_table, &settings_ntpserv_heading, tableNum, friendlyHostname);
			ntpHostPointer = config->ntp->server;
			while (ntpHostPointer != 0)
			{
				outputFriendly(ntpHostPointer->ntpHost, tempString, maxSize, reportFormat);
				fprintf(reportFile, "%s%s%s", tablerow_start, tempString, tablerow_mid);
				if (ntpHostPointer->keyNum == -1)
					fprintf(reportFile, "None%s", tablerow_end);
				else
					fprintf(reportFile, "%d%s", ntpHostPointer->keyNum, tablerow_end);
				ntpHostPointer = ntpHostPointer->next;
			}
			output_table(reportFile, reportFormat, false, passwords, settings_ntpserv_table, &settings_ntpserv_heading, tableNum, friendlyHostname);
			tableNum++;
		}

		if (config->ntp->peer != 0)
		{
			output_table(reportFile, reportFormat, true, passwords, settings_ntppeer_table, &settings_ntppeer_heading, tableNum, friendlyHostname);
			ntpHostPointer = config->ntp->peer;
			while (ntpHostPointer != 0)
			{
				outputFriendly(ntpHostPointer->ntpHost, tempString, maxSize, reportFormat);
				fprintf(reportFile, "%s%s%s%d%s", tablerow_start, tempString, tablerow_mid, ntpHostPointer->keyNum, tablerow_end);
				ntpHostPointer = ntpHostPointer->next;
			}
			output_table(reportFile, reportFormat, false, passwords, settings_ntppeer_table, &settings_ntppeer_heading, tableNum, friendlyHostname);
			tableNum++;
		}

		if ((config->ntp->key != 0) && (passwords == true))
		{
			output_table(reportFile, reportFormat, true, passwords, settings_ntpkeys_table, &settings_ntpkeys_heading, tableNum, friendlyHostname);
			ntpKeyPointer = config->ntp->key;
			while (ntpKeyPointer != 0)
			{
				fprintf(reportFile, "%s%d%s", tablerow_start, ntpKeyPointer->keyNum, tablerow_mid);
				if (ntpKeyPointer->trusted == false)
					fprintf(reportFile, "No");
				else
					fprintf(reportFile, "Yes");
				if (passwords == true)
				{
					outputFriendly(ntpKeyPointer->key, tempString, maxSize, reportFormat);
					fprintf(reportFile, "%s%s", tablerow_mid, tempString);
				}
				fprintf(reportFile, "%s", tablerow_end);
				ntpKeyPointer = ntpKeyPointer->next;
			}
			output_table(reportFile, reportFormat, false, passwords, settings_ntpkeys_table, &settings_ntpkeys_heading, tableNum, friendlyHostname);
			tableNum++;
		}

		if (config->ntp->access != 0)
		{
			output_table(reportFile, reportFormat, true, passwords, settings_ntpaccess_table, &settings_ntpaccess_heading, tableNum, friendlyHostname);
			ntpGroupPointer = config->ntp->access;
			while (ntpGroupPointer != 0)
			{
				switch (ntpGroupPointer->accessType)
				{
					case ntp_peer:
						fprintf(reportFile, "%sPeer%s%d%s", tablerow_start, tablerow_mid, ntpGroupPointer->accessList, tablerow_end);
						break;
					case ntp_serve:
						fprintf(reportFile, "%sServe%s%d%s", tablerow_start, tablerow_mid, ntpGroupPointer->accessList, tablerow_end);
						break;
					case ntp_serve_only:
						fprintf(reportFile, "%sServe-Only%s%d%s", tablerow_start, tablerow_mid, ntpGroupPointer->accessList, tablerow_end);
						break;
					case ntp_query_only:
						fprintf(reportFile, "%sQuery-Only%s%d%s", tablerow_start, tablerow_mid, ntpGroupPointer->accessList, tablerow_end);
						break;
				}
				ntpGroupPointer = ntpGroupPointer->next;
			}
			output_table(reportFile, reportFormat, false, passwords, settings_ntpaccess_table, &settings_ntpaccess_heading, tableNum, friendlyHostname);
			tableNum++;
		}

		output_endsection(reportFile, reportFormat, subsection_end);
	}

	// FTP
	if (config->ftp != 0)
	{
		output_parseTitle(settings_ftp_title, reportFile, reportFormat, friendlyHostname, rate_none, section_config);
		if (config->ftp->username[0] != 0)
		{
			output_table(reportFile, reportFormat, true, passwords, settings_ftp_table, &settings_general_heading, tableNum, friendlyHostname);
			outputFriendly(config->ftp->username, tempString, maxSize, reportFormat);
			fprintf(reportFile, "%sFTP Username%s%s%s", tablerow_start, tablerow_mid, tempString, tablerow_end);
			if (passwords == true)
			{
				outputFriendly(config->ftp->password, tempString, maxSize, reportFormat);
				fprintf(reportFile, "%sFTP Password%s%s %s", tablerow_start, tablerow_mid, tempString, tablerow_end);
			}
			switch(config->ftp->encryption)
			{
				case encrypt_type7:
					fprintf(reportFile, "%sPassword Encryption Type%sType-7%s", tablerow_start, tablerow_mid, tablerow_end);
					break;
				case encrypt_md5:
					fprintf(reportFile, "%sPassword Encryption Type%sMD5%s", tablerow_start, tablerow_mid, tablerow_end);
					break;
				case encrypt_none:
					fprintf(reportFile, "%sPassword Encryption Type%sNone%s", tablerow_start, tablerow_mid, tablerow_end);
					break;
				case encrypt_unknown:
					fprintf(reportFile, "%sPassword Encryption Type%sUnknown%s", tablerow_start, tablerow_mid, tablerow_end);
					break;
			}
			output_table(reportFile, reportFormat, false, passwords, settings_ftp_table, &settings_general_heading, tableNum, friendlyHostname);
			tableNum++;
		}
		addAbbreviation("FTP", false);

		// Show FTP interfaces
		if (config->ftp->interface != 0)
		{
			output_table(reportFile, reportFormat, true, passwords, settings_ftpint_table, &settings_sshint_heading, tableNum, friendlyHostname);
			interfacePointer = config->ftp->interface;
			while (interfacePointer != 0)
			{
				outputFriendly(interfacePointer->interface, tempString, maxSize, reportFormat);
				fprintf(reportFile, "%s%s%s", tablerow_start, tempString, tablerow_end);
				interfacePointer = interfacePointer->next;
			}
			output_table(reportFile, reportFormat, false, passwords, settings_ftpint_table, &settings_sshint_heading, tableNum, friendlyHostname);
			tableNum++;
		}

		output_endsection(reportFile, reportFormat, subsection_end);
	}

	// HTTP
	if (config->http != 0)
	{
		output_parseTitle(settings_http_title, reportFile, reportFormat, friendlyHostname, rate_none, section_config);
		output_table(reportFile, reportFormat, true, passwords, settings_http_table, &settings_general_heading, tableNum, friendlyHostname);
		if (config->http->enabled == true)
			fprintf(reportFile, "%sHTTP Server%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
		else
			fprintf(reportFile, "%sHTTP Server%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
		switch(config->http->authentication)
		{
			case http_auth_enable:
				fprintf(reportFile, "%sAuthentication Type%sEnable Password%s", tablerow_start, tablerow_mid, tablerow_end);
				break;
			case http_auth_aaa:
				fprintf(reportFile, "%sAuthentication Type%sAAA%s", tablerow_start, tablerow_mid, tablerow_end);
				addAbbreviation("AAA", false);
				break;
			case http_auth_tacacs:
				fprintf(reportFile, "%sAuthentication Type%sTACACS%s", tablerow_start, tablerow_mid, tablerow_end);
				addAbbreviation("TACACS", false);
				break;
			case http_auth_local:
				fprintf(reportFile, "%sAuthentication Type%sLocal Users%s", tablerow_start, tablerow_mid, tablerow_end);
				break;
		}
		if (config->http->accessClass == 0)
			fprintf(reportFile, "%sAccess Class (Access List Number)%sUnconfigured%s", tablerow_start, tablerow_mid, tablerow_end);
		else
			fprintf(reportFile, "%sAccess Class (Access List Number)%s%d%sn", tablerow_start, tablerow_mid, config->http->accessClass, tablerow_end);
		output_table(reportFile, reportFormat, false, passwords, settings_http_table, &settings_general_heading, tableNum, friendlyHostname);
		tableNum++;
		output_endsection(reportFile, reportFormat, subsection_end);
		addAbbreviation("HTTP", false);
	}

	// TACACS
	if (config->tacacs != 0)
	{
		output_parseTitle(settings_tacacs_title, reportFile, reportFormat, friendlyHostname, rate_none, section_config);
		output_table(reportFile, reportFormat, true, passwords, settings_tacacs_table, &settings_general_heading, tableNum, friendlyHostname);
		if (passwords == true)
		{
			outputFriendly(config->tacacs->key, tempString, maxSize, reportFormat);
			fprintf(reportFile, "%sTACACS Key%s%s %s", tablerow_start, tablerow_mid, tempString, tablerow_end);
		}
		switch(config->tacacs->encryption)
		{
			case encrypt_type7:
				fprintf(reportFile, "%sKey Encryption Type%sType-7%s", tablerow_start, tablerow_mid, tablerow_end);
				break;
			case encrypt_md5:
				fprintf(reportFile, "%sKey Encryption Type%sMD5%s", tablerow_start, tablerow_mid, tablerow_end);
				addAbbreviation("MD5", false);
				break;
			case encrypt_none:
				fprintf(reportFile, "%sKey Encryption Type%sNone%s", tablerow_start, tablerow_mid, tablerow_end);
				break;
			case encrypt_unknown:
				fprintf(reportFile, "%sKey Encryption Type%sUnknown%s", tablerow_start, tablerow_mid, tablerow_end);
				break;
		}
		output_table(reportFile, reportFormat, false, passwords, settings_tacacs_table, &settings_general_heading, tableNum, friendlyHostname);
		tableNum++;
		addAbbreviation("TACACS", false);

		if (config->tacacs->host != 0)
		{
			output_table(reportFile, reportFormat, true, passwords, settings_tacacsserv_table, &settings_tacacsserv_heading, tableNum, friendlyHostname);
			hostPointer = config->tacacs->host;
			while (hostPointer != 0)
			{
				outputFriendly(hostPointer->server, tempString, maxSize, reportFormat);
				fprintf(reportFile, "%s%s%s", tablerow_start, tempString, tablerow_end);
				hostPointer = hostPointer->next;
			}
			output_table(reportFile, reportFormat, false, passwords, settings_tacacsserv_table, &settings_tacacsserv_heading, tableNum, friendlyHostname);
			tableNum++;
		}

		output_endsection(reportFile, reportFormat, subsection_end);
	}

	// AAA
	if ((config->aaa !=0) && (config->aaa->groupServer !=0))
	{
		output_parseTitle(settings_aaa_title, reportFile, reportFormat, friendlyHostname, rate_none, section_config);
		aaaGroupPointer = config->aaa->groupServer;
		while (aaaGroupPointer != 0)
		{
			switch (aaaGroupPointer->type)
			{
				case aaa_radius:
					sprintf(tempString, "%s %s %s", settings_aaa_table1, aaaGroupPointer->username, settings_aaa_table2);
					addAbbreviation("RADIUS", false);
					break;
				case aaa_tacacs:
					sprintf(tempString, "%s %s %s", settings_aaa_table1, aaaGroupPointer->username, settings_aaa_table3);
					addAbbreviation("TACACS", false);
					break;
				case aaa_kerberos:
					sprintf(tempString, "%s %s %s", settings_aaa_table1, aaaGroupPointer->username, settings_aaa_table4);
					addAbbreviation("KERBEROS", false);
					break;
			}
			outputFriendly(tempString, tempString2, maxSize, reportFormat);
			output_table(reportFile, reportFormat, true, passwords, tempString2, &settings_aaa_heading, tableNum, friendlyHostname);
			aaaServerPointer = aaaGroupPointer->server;
			while (aaaServerPointer != 0)
			{
				outputFriendly(aaaServerPointer->server, tempString, maxSize, reportFormat);
				fprintf(reportFile, "%s%s%s%d%s%d%s", tablerow_start, tempString, tablerow_mid, aaaServerPointer->authPort, tablerow_mid, aaaServerPointer->acctPort, tablerow_mid);
				if (passwords == true)
				{
					outputFriendly(aaaServerPointer->key, tempString, maxSize, reportFormat);
					fprintf(reportFile, "%s %s", tempString, tablerow_mid);
				}
				switch(aaaServerPointer->encryption)
				{
					case encrypt_type7:
						fprintf(reportFile, "Type-7%s", tablerow_end);
						break;
					case encrypt_md5:
						fprintf(reportFile, "MD5%s", tablerow_end);
						addAbbreviation("MD5", false);
						break;
					case encrypt_none:
						fprintf(reportFile, "None%s", tablerow_end);
						break;
					case encrypt_unknown:
						fprintf(reportFile, "Unknown%s", tablerow_end);
						break;
				}
				aaaServerPointer = aaaServerPointer->next;
			}
			output_table(reportFile, reportFormat, false, passwords, tempString2, &settings_aaa_heading, tableNum, friendlyHostname);
			tableNum++;
			aaaGroupPointer = aaaGroupPointer->next;
		}
		output_endsection(reportFile, reportFormat, subsection_end);
		addAbbreviation("AAA", false);
	}

	// Routing
	if ((config->bgp != 0) || (config->eigrp != 0) || (config->isis != 0) || (config->rip != 0) || (config->ospf != 0) || (config->route != 0))
	{
		output_parseTitle(settings_routing_title, reportFile, reportFormat, friendlyHostname, rate_none, section_config);
		output_parseText(settings_routing_desc, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, 0, 0);

		// Static Routes
		if (config->route != 0)
		{
			output_table(reportFile, reportFormat, true, passwords, settings_static_route_table, &settings_routeIOS_heading, tableNum, friendlyHostname);
			routePointer = config->route;
			while (routePointer != 0)
			{
				outputFriendly(routePointer->ipAddress, tempString, maxSize, reportFormat);
				fprintf(reportFile, "%s%s%s", tablerow_start, tempString, tablerow_mid);
				outputFriendly(routePointer->netMask, tempString, maxSize, reportFormat);
				fprintf(reportFile, "%s%s", tempString, tablerow_mid);
				outputFriendly(routePointer->gateway, tempString, maxSize, reportFormat);
				fprintf(reportFile, "%s%s", tempString, tablerow_end);
				routePointer = routePointer->next;
			}
			output_table(reportFile, reportFormat, false, passwords, settings_static_route_table, &settings_routeIOS_heading, tableNum, friendlyHostname);
			tableNum++;
		}

		// BGP
		if (config->bgp != 0)
		{
			output_parseText(settings_bgp_desc, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, 0, 0);
			output_table(reportFile, reportFormat, true, passwords, settings_bgp_table, &settings_general_heading, tableNum, friendlyHostname);
			fprintf(reportFile, "%sAutonomous Number%s%d%s", tablerow_start, tablerow_mid, config->bgp->autonomousNo, tablerow_end);
			if (config->bgp->logNeighborChanges == true)
				fprintf(reportFile, "%sLog Neighbor Changes%sYes%s", tablerow_start, tablerow_mid, tablerow_end);
			else
				fprintf(reportFile, "%sLog Neighbor Changes%sNo%s", tablerow_start, tablerow_mid, tablerow_end);
			if (config->bgp->dampening == true)
				fprintf(reportFile, "%sRoute Flap Dampening%sOn%s", tablerow_start, tablerow_mid, tablerow_end);
			else
				fprintf(reportFile, "%sRoute Flap Dampening%sOff%s", tablerow_start, tablerow_mid, tablerow_end);
			output_table(reportFile, reportFormat, false, passwords, settings_bgp_table, &settings_general_heading, tableNum, friendlyHostname);
			tableNum++;

			if (config->bgp->neighbor != 0)
			{
				output_table(reportFile, reportFormat, true, passwords, settings_bgpneighbor_table, &settings_bgp_heading, tableNum, friendlyHostname);
				neighborPointer = config->bgp->neighbor;
				while (neighborPointer != 0)
				{
					outputFriendly(neighborPointer->host, tempString, maxSize, reportFormat);
					fprintf(reportFile, "%s%s%s", tablerow_start, tempString, tablerow_mid);
					outputFriendly(neighborPointer->description, tempString, maxSize, reportFormat);
					fprintf(reportFile, "%s%s", tempString, tablerow_mid);
					if (passwords == true)
					{
						outputFriendly(neighborPointer->password, tempString, maxSize, reportFormat);
						fprintf(reportFile, "%s %s", tempString, tablerow_mid);
					}
					switch (neighborPointer->encryption)
					{
						case encrypt_type7:
							fprintf(reportFile, "Type-7%s", tablerow_mid);
							break;
						case encrypt_md5:
							fprintf(reportFile, "MD5%s", tablerow_mid);
							addAbbreviation("MD5", false);
							break;
						case encrypt_none:
							if (neighborPointer->password[0] == 0)
								fprintf(reportFile, " %s", tablerow_mid);
							else
								fprintf(reportFile, "None%s", tablerow_mid);
							break;
						case encrypt_unknown:
							fprintf(reportFile, "Unknown%s", tablerow_mid);
							break;
					}
					if (neighborPointer->autonomousNo != -1)
						fprintf(reportFile, "%d%s", neighborPointer->autonomousNo, tablerow_mid);
					else
						fprintf(reportFile, " %s", tablerow_mid);
					if (neighborPointer->ttlHops == -1)
						fprintf(reportFile, "Off%s", tablerow_end);
					else
						fprintf(reportFile, "%d%s", neighborPointer->ttlHops, tablerow_end);
					neighborPointer = neighborPointer->next;
				}
				output_table(reportFile, reportFormat, false, passwords, settings_bgpneighbor_table, &settings_bgp_heading, tableNum, friendlyHostname);
				tableNum++;
			}
			addAbbreviation("BGP", false);
			addAbbreviation("TTL", false);
		}

		// OSPF
		if (config->ospf != 0)
		{
			output_parseText(settings_ospf_desc, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, 0, 0);

			// OSPF Network
			if (config->ospf->ospfNetwork != 0)
			{
				output_table(reportFile, reportFormat, true, passwords, settings_ospf_network_table, &settings_ospf_network_heading, tableNum, friendlyHostname);
				ospfNetworkPointer = config->ospf->ospfNetwork;
				while (ospfNetworkPointer != 0)
				{
					fprintf(reportFile, "%s%s%s%s%s%s%s", tablerow_start, ospfNetworkPointer->network, tablerow_mid, ospfNetworkPointer->networkMask, tablerow_mid, ospfNetworkPointer->areaID, tablerow_end);
					ospfNetworkPointer = ospfNetworkPointer->next;
				}
				output_table(reportFile, reportFormat, false, passwords, settings_ospf_network_table, &settings_ospf_network_heading, tableNum, friendlyHostname);
				tableNum++;
			}

			// OSPF Areas
			if (config->ospf->ospfArea != 0)
			{
				output_table(reportFile, reportFormat, true, passwords, settings_ospf_area_table, &settings_ospf_area_heading, tableNum, friendlyHostname);
				ospfAreaPointer = config->ospf->ospfArea;
				while (ospfAreaPointer != 0)
				{
					switch (ospfAreaPointer->areaAuth)
					{
						case auth_none:
							fprintf(reportFile, "%s%s%s%d%sNone%s", tablerow_start, ospfAreaPointer->areaID, tablerow_mid, ospfAreaPointer->defaultCost, tablerow_mid, tablerow_end);
							break;
						case auth_clear:
							fprintf(reportFile, "%s%s%s%d%sClear-text%s", tablerow_start, ospfAreaPointer->areaID, tablerow_mid, ospfAreaPointer->defaultCost, tablerow_mid, tablerow_end);
							break;
						case auth_md5:
							fprintf(reportFile, "%s%s%s%d%sMD5%s", tablerow_start, ospfAreaPointer->areaID, tablerow_mid, ospfAreaPointer->defaultCost, tablerow_mid, tablerow_end);
							break;
					}
					ospfAreaPointer = ospfAreaPointer->next;
				}
				output_table(reportFile, reportFormat, false, passwords, settings_ospf_area_table, &settings_ospf_area_heading, tableNum, friendlyHostname);
				tableNum++;
			}

			// OSPF Interfaces
			if (config->interface !=0)
			{
				output_parseText(settings_ospf_interface_desc, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, 0, 0);
				output_table(reportFile, reportFormat, true, passwords, settings_ospf_interface_table, &settings_ospf_interface_heading, tableNum, friendlyHostname);
				interfaceConfigPointer = config->interface;
				while (interfaceConfigPointer != 0)
				{
					// Check interface is valid for OSPF
					if ((interfaceConfigPointer->shutdown == false) && (strcmp(interfaceConfigPointer->ipAddress, "None") != 0))
					{
						outputFriendly(interfaceConfigPointer->description, tempString, maxSize, reportFormat);
						fprintf(reportFile, "%s%s%s%s%s%s%s", tablerow_start, interfaceConfigPointer->name, tablerow_mid, tempString, tablerow_mid, interfaceConfigPointer->ipAddress, tablerow_mid);
						switch (interfaceConfigPointer->ospfAuthentication)
						{
							case auth_none:
								fprintf(reportFile, "None%s", tablerow_mid);
								break;
							case auth_clear:
								fprintf(reportFile, "Clear-Text%s", tablerow_mid);
								break;
							case auth_md5:
								fprintf(reportFile, "MD5%s", tablerow_mid);
								addAbbreviation("MD5", false);
								break;
						}
						if (passwords == true)
						{
							outputFriendly(interfaceConfigPointer->ospfKey, tempString, maxSize, reportFormat);
							fprintf(reportFile, "%s %s", tempString, tablerow_mid);
						}
						switch (interfaceConfigPointer->ospfKeyEncryption)
						{
							case encrypt_type7:
								fprintf(reportFile, "Type-7%s", tablerow_mid);
								break;
							case encrypt_md5:
								fprintf(reportFile, "MD5%s", tablerow_mid);
								addAbbreviation("MD5", false);
								break;
							case encrypt_none:
								fprintf(reportFile, "None%s", tablerow_mid);
								break;
							case encrypt_unknown:
								fprintf(reportFile, "Unknown%s", tablerow_mid);
								break;
						}
						switch (interfaceConfigPointer->ospfFloodReduction)
						{
							case true:
								fprintf(reportFile, "On%s", tablerow_mid);
								break;
							case false:
								fprintf(reportFile, "Off%s", tablerow_mid);
								break;
							case unconfigured:
								fprintf(reportFile, "N/A%s", tablerow_mid);
								break;
						}
						switch (interfaceConfigPointer->ospfNetwork)
						{
							case ospf_default:
								fprintf(reportFile, "Default");
								break;
							case ospf_broadcast:
								fprintf(reportFile, "Broadcast");
								break;
							case ospf_nonbroadcast:
								fprintf(reportFile, "Non-Broadcast");
								break;
							case ospf_p2p:
								fprintf(reportFile, "Point to Point");
								addAbbreviation("PPP", false);
								break;
							case ospf_p2multi:
								fprintf(reportFile, "Point to Multipoint");
								break;
							case ospf_p2multinon:
								fprintf(reportFile, "Point to Multipoint (Non-Broadcast)");
								break;
						}
						fprintf(reportFile, "%s", tablerow_end);
					}
					interfaceConfigPointer = interfaceConfigPointer->next;
				}
				output_table(reportFile, reportFormat, false, passwords, settings_ospf_interface_table, &settings_ospf_interface_heading, tableNum, friendlyHostname);
				tableNum++;
			}

			// OSPF Neighbors
			if (config->ospf->neighbor != 0)
			{
				output_table(reportFile, reportFormat, true, passwords, settings_ospf_neighbor_table, &settings_ospf_neighbor_heading, tableNum, friendlyHostname);
				ospfNeighborPointer = config->ospf->neighbor;
				while (ospfNeighborPointer != 0)
				{
					fprintf(reportFile, "%s%s%s%d%s%d%s%d%s", tablerow_start, ospfNeighborPointer->host, tablerow_mid, ospfNeighborPointer->priority, tablerow_mid, ospfNeighborPointer->pollInterval, tablerow_mid, ospfNeighborPointer->costNumber, tablerow_mid);
					if (ospfNeighborPointer->databaseFilter == false)
						fprintf(reportFile, "No%s", tablerow_end);
					else
						fprintf(reportFile, "Yes%s", tablerow_end);
					ospfNeighborPointer = ospfNeighborPointer->next;
				}
				output_table(reportFile, reportFormat, false, passwords, settings_ospf_neighbor_table, &settings_ospf_neighbor_heading, tableNum, friendlyHostname);
				tableNum++;
			}
		}

		output_endsection(reportFile, reportFormat, subsection_end);
	}

	// Key Chains and Keys
	if ((passwords == true) && (config->chain != 0))
	{
		output_parseTitle(settings_keys_title, reportFile, reportFormat, friendlyHostname, rate_none, section_config);
		output_table(reportFile, reportFormat, true, passwords, settings_keys_table, &settings_keys_heading, tableNum, friendlyHostname);
		keyChainPointer = config->chain;
		while (keyChainPointer != 0)
		{
			keyPointer = keyChainPointer->key;
			while (keyPointer != 0)
			{
				fprintf(reportFile, "%s%s%s%d", tablerow_start, keyChainPointer->name, tablerow_mid, keyPointer->keyNumber);
				if (passwords == true)
				{
					outputFriendly(keyPointer->key, tempString, maxSize, reportFormat);
					fprintf(reportFile, "%s%s", tablerow_mid, tempString);
				}
				fprintf(reportFile, "%s", tablerow_end);
				keyPointer = keyPointer->next;
			}
			keyChainPointer = keyChainPointer->next;
		}
		output_table(reportFile, reportFormat, false, passwords, settings_keys_table, &settings_keys_heading, tableNum, friendlyHostname);
		tableNum++;
		output_endsection(reportFile, reportFormat, subsection_end);
	}

	// Lines
	if (config->line !=0)
	{
		output_parseTitle(settings_lines_title, reportFile, reportFormat, friendlyHostname, rate_none, section_config);
		output_parseText(settings_lines_text, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, 0, 0);
		output_table(reportFile, reportFormat, true, passwords, settings_lines_table, &settings_lines_heading, tableNum, friendlyHostname);
		linePointer = config->line;
		while (linePointer != 0)
		{
			fprintf(reportFile, "%s%d%s", tablerow_start, linePointer->lineNumberStart, tablerow_mid);
			if (linePointer->lineNumberEnd == 0)
				fprintf(reportFile, " %s", tablerow_mid);
			else
				fprintf(reportFile, "%d%s", linePointer->lineNumberEnd, tablerow_mid);
			switch (linePointer->lineType)
			{
				case line_console:
					fprintf(reportFile, "Console%s", tablerow_mid);
					break;
				case line_aux:
					fprintf(reportFile, "Auxiliary%s", tablerow_mid);
					break;
				case line_vty:
					fprintf(reportFile, "VTY%s", tablerow_mid);
					addAbbreviation("VTY", false);
					break;
				case line_tty:
					fprintf(reportFile, "TTY%s", tablerow_mid);
					addAbbreviation("TTY", false);
					break;
			}
			switch (linePointer->lineLogin)
			{
				case true:
					fprintf(reportFile, "Allowed%s", tablerow_mid);
					break;
				case false:
					fprintf(reportFile, "Denied%s", tablerow_mid);
					break;
				case unconfigured:
					fprintf(reportFile, "Unconfigured%s", tablerow_mid);
					break;
			}
			switch (linePointer->exec)
			{
				case true:
					fprintf(reportFile, "Allowed%s", tablerow_mid);
					break;
				case false:
					fprintf(reportFile, "Denied%s", tablerow_mid);
					break;
			}
			if ((linePointer->lineType == line_console) || (linePointer->lineType == line_aux))
				fprintf(reportFile, "%d%s", linePointer->lineSpeed, tablerow_mid);
			else
				fprintf(reportFile, "N/A%s", tablerow_mid);
			fprintf(reportFile, "%ds%s%ds%s%ds%s%ds%s", linePointer->timeout, tablerow_mid, linePointer->execTimeout, tablerow_mid, linePointer->sessionTimeout, tablerow_mid, linePointer->absoluteTimeout, tablerow_mid);
			if (passwords == true)
				fprintf(reportFile, "%s%s", linePointer->password, tablerow_mid);
			if (linePointer->encryption == encrypt_type7)
				fprintf(reportFile, "Type-7%s", tablerow_end);
			else if (linePointer->encryption == encrypt_md5)
			{
				fprintf(reportFile, "MD5%s", tablerow_end);
				addAbbreviation("MD5", false);
			}
			else if (linePointer->encryption == encrypt_unknown)
				fprintf(reportFile, "Unknown%s", tablerow_end);
			else
			{
				if (linePointer->password[0] == 0)
					fprintf(reportFile, " %s", tablerow_end);
				else
					fprintf(reportFile, "None%s", tablerow_end);
			}
			linePointer = linePointer->next;
		}
		output_table(reportFile, reportFormat, false, passwords, settings_lines_table, &settings_lines_heading, tableNum, friendlyHostname);
		tableNum++;
		output_endsection(reportFile, reportFormat, subsection_end);
	}

	// Interfaces
	if (config->interface !=0)
	{
		output_parseTitle(settings_inter_title, reportFile, reportFormat, friendlyHostname, rate_none, section_config);
		output_table(reportFile, reportFormat, true, passwords, settings_inter_table, &settings_inter_heading, tableNum, friendlyHostname);
		interfaceConfigPointer = config->interface;
		while (interfaceConfigPointer != 0)
		{
			fprintf(reportFile, "%s%s%s", tablerow_start, interfaceConfigPointer->name, tablerow_mid);
			if (interfaceConfigPointer->shutdown == true)
				fprintf(reportFile, "No%s%s%s", tablerow_mid, interfaceConfigPointer->ipAddress, tablerow_mid);
			else
				fprintf(reportFile, "Yes%s%s%s", tablerow_mid, interfaceConfigPointer->ipAddress, tablerow_mid);
			if (interfaceConfigPointer->proxyArp == true)
				fprintf(reportFile, "Enabled%s", tablerow_mid);
			else
				fprintf(reportFile, "Disabled%s", tablerow_mid);
			if (interfaceConfigPointer->ipUnreachables == true)
				fprintf(reportFile, "Enabled%s", tablerow_mid);
			else
				fprintf(reportFile, "Disabled%s", tablerow_mid);
			if (interfaceConfigPointer->ipRedirects == true)
				fprintf(reportFile, "Enabled%s", tablerow_mid);
			else
				fprintf(reportFile, "Disabled%s", tablerow_mid);
			if (interfaceConfigPointer->ipMaskReply == true)
				fprintf(reportFile, "Enabled%s", tablerow_mid);
			else
				fprintf(reportFile, "Disabled%s", tablerow_mid);
			if (interfaceConfigPointer->ipDirectBroadcast == true)
				fprintf(reportFile, "Enabled%s", tablerow_mid);
			else if (interfaceConfigPointer->ipDirectBroadcast == false)
				fprintf(reportFile, "Disabled%s", tablerow_mid);
			else
				fprintf(reportFile, "Unconfigured%s", tablerow_mid);
			if (interfaceConfigPointer->ntp == true)
				fprintf(reportFile, "Enabled%s", tablerow_mid);
			else
				fprintf(reportFile, "Disabled%s", tablerow_mid);
			if (interfaceConfigPointer->cdp == true)
				fprintf(reportFile, "Enabled%s", tablerow_mid);
			else
				fprintf(reportFile, "Disabled%s", tablerow_mid);
			if (interfaceConfigPointer->mop == true)
				fprintf(reportFile, "Enabled%s", tablerow_end);
			else
				fprintf(reportFile, "Disabled%s", tablerow_end);
			interfaceConfigPointer = interfaceConfigPointer->next;
		}
		output_table(reportFile, reportFormat, false, passwords, settings_inter_table, &settings_inter_heading, tableNum, friendlyHostname);
		tableNum++;
		output_endsection(reportFile, reportFormat, subsection_end);
		addAbbreviation("ARP", false);
		addAbbreviation("NTP", false);
		addAbbreviation("CDP", false);
		addAbbreviation("MOP", false);
	}

	// Access Lists
	if (config->access !=0)
	{
		output_parseTitle(settings_accesslists_title, reportFile, reportFormat, friendlyHostname, rate_none, section_config);
		output_parseText(settings_accesslists1, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, 0, 0);
		output_parseText(settings_accesslists2, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, 0, 0);

		// Interfaces with ACL applied
		tempInt = 0;
		interfaceConfigPointer = config->interface;
		while (interfaceConfigPointer != 0)
		{
			if ((interfaceConfigPointer->accessGroupIn != 0) || (interfaceConfigPointer->accessGroupOut != 0))
				tempInt++;
			interfaceConfigPointer = interfaceConfigPointer->next;
		}
		if (tempInt > 0)
		{
			output_table(reportFile, reportFormat, true, passwords, settings_acl_int_table, &settings_aclinter_heading, tableNum, friendlyHostname);
			interfaceConfigPointer = config->interface;
			while (interfaceConfigPointer != 0)
			{
				if ((interfaceConfigPointer->accessGroupIn != 0) || (interfaceConfigPointer->accessGroupOut != 0))
				{
					fprintf(reportFile, "%s%s%s", tablerow_start, interfaceConfigPointer->name, tablerow_mid);
					if (interfaceConfigPointer->shutdown == true)
						fprintf(reportFile, "No%s%s%s", tablerow_mid, interfaceConfigPointer->ipAddress, tablerow_mid);
					else
						fprintf(reportFile, "Yes%s%s%s", tablerow_mid, interfaceConfigPointer->ipAddress, tablerow_mid);
					if (interfaceConfigPointer->accessGroupIn != 0)
						fprintf(reportFile, "%d%s", interfaceConfigPointer->accessGroupIn, tablerow_mid);
					else
						fprintf(reportFile, " %s", tablerow_mid);
					if (interfaceConfigPointer->accessGroupOut != 0)
						fprintf(reportFile, "%d%s", interfaceConfigPointer->accessGroupOut, tablerow_end);
					else
						fprintf(reportFile, " %s", tablerow_end);
				}
				interfaceConfigPointer = interfaceConfigPointer->next;
			}
			output_table(reportFile, reportFormat, false, passwords, settings_acl_int_table, &settings_aclinter_heading, tableNum, friendlyHostname);
			tableNum++;
		}

		accessPointer = config->access;
		while (accessPointer !=0)
		{
			// Standard Access List
			if (accessPointer->listNumber < 100)
			{
				tempInt = 1;
				if (accessPointer->remark[0] == 0)
					sprintf(tempString, "%s %d", settings_accessstd_table, accessPointer->listNumber);
				else
					sprintf(tempString, "%s %d (%s)", settings_accessstd_table, accessPointer->listNumber, accessPointer->remark);
				outputFriendly(tempString, tempString2, maxSize, reportFormat);
				output_table(reportFile, reportFormat, true, passwords, tempString2, &settings_accessstd_heading, tableNum, friendlyHostname);
				permissionPointer = accessPointer->permission;
				while (permissionPointer != 0)
				{
					fprintf(reportFile, "%s%d%s", tablerow_start, tempInt, tablerow_mid);
					tempInt++;
					if (permissionPointer->permission == perm_permit)
						fprintf(reportFile, "Permit%s", tablerow_mid);
					else
						fprintf(reportFile, "Deny%s", tablerow_mid);
					outputFriendly(permissionPointer->source, tempString, maxSize, reportFormat);
					fprintf(reportFile, "%s %s", tempString, tablerow_mid);
					if (permissionPointer->log == true)
						fprintf(reportFile, "Yes");
					else
						fprintf(reportFile, "No");
					fprintf(reportFile, "%s", tablerow_end);
					permissionPointer = permissionPointer->next;
				}
				output_table(reportFile, reportFormat, false, passwords, tempString2, &settings_accessstd_heading, tableNum, friendlyHostname);
				tableNum++;
			}

			// Extended Access List
			if (accessPointer->listNumber > 99)
			{
				tempInt = 1;
				if (accessPointer->remark[0] == 0)
					sprintf(tempString, "%s %d", settings_accessext_table, accessPointer->listNumber);
				else
					sprintf(tempString, "%s %d (%s)", settings_accessext_table, accessPointer->listNumber, accessPointer->remark);
				outputFriendly(tempString, tempString2, maxSize, reportFormat);
				output_table(reportFile, reportFormat, true, passwords, tempString2, &settings_accessext_heading, tableNum, friendlyHostname);
				permissionPointer = accessPointer->permission;
				while (permissionPointer != 0)
				{
					fprintf(reportFile, "%s%d%s", tablerow_start, tempInt, tablerow_mid);
					tempInt++;
					if (permissionPointer->permission == perm_permit)
						fprintf(reportFile, "Permit%s", tablerow_mid);
					else
						fprintf(reportFile, "Deny%s", tablerow_mid);
					outputFriendly(permissionPointer->source, tempString, maxSize, reportFormat);
					fprintf(reportFile, "%s %s%s %s", permissionPointer->protocol, tablerow_mid, tempString, tablerow_mid);
					addAbbreviation(permissionPointer->protocol, false);
					addAbbreviation(permissionPointer->sourceService, false);
					switch (permissionPointer->sourceOperator)
					{
						case perm_op_lt:
							if (reportFormat == output_html)
								fprintf(reportFile, "&lt;");
							else
								fprintf(reportFile, "<");
							fprintf(reportFile, " %s", permissionPointer->sourceService);
							break;
						case perm_op_gt:
							if (reportFormat == output_html)
								fprintf(reportFile, "&gt;");
							else
								fprintf(reportFile, ">");
							fprintf(reportFile, " %s", permissionPointer->sourceService);
							break;
						case perm_op_eq:
							fprintf(reportFile, "%s ", permissionPointer->sourceService);
							break;
						case perm_op_neq:
							fprintf(reportFile, "not %s", permissionPointer->sourceService);
							break;
						case perm_op_range:
							fprintf(reportFile, "%s - %s", permissionPointer->sourceService, permissionPointer->sourceServiceEnd);
							addAbbreviation(permissionPointer->sourceServiceEnd, false);
							break;
						default:
							fprintf(reportFile, "any");
							break;
					}
					outputFriendly(permissionPointer->dest, tempString, maxSize, reportFormat);
					fprintf(reportFile, "%s%s %s", tablerow_mid, tempString, tablerow_mid);
					addAbbreviation(permissionPointer->destService, false);
					switch (permissionPointer->destOperator)
					{
						case perm_op_lt:
							if (reportFormat == output_html)
								fprintf(reportFile, "&lt;");
							else
								fprintf(reportFile, "<");
							fprintf(reportFile, " %s", permissionPointer->destService);
							break;
						case perm_op_gt:
							if (reportFormat == output_html)
								fprintf(reportFile, "&gt;");
							else
								fprintf(reportFile, ">");
							fprintf(reportFile, " %s", permissionPointer->destService);
							break;
						case perm_op_eq:
							fprintf(reportFile, "%s ", permissionPointer->destService);
							break;
						case perm_op_neq:
							fprintf(reportFile, "not %s", permissionPointer->destService);
							break;
						case perm_op_range:
							fprintf(reportFile, "%s - %s", permissionPointer->destService, permissionPointer->destServiceEnd);
							addAbbreviation(permissionPointer->destServiceEnd, false);
							break;
						default:
							fprintf(reportFile, "any");
							break;
					}
					if (permissionPointer->log == true)
						fprintf(reportFile, "%sYes%s", tablerow_mid, tablerow_mid);
					else
						fprintf(reportFile, "%sNo%s", tablerow_mid, tablerow_mid);
					if (permissionPointer->established == true)
						fprintf(reportFile, "Established");
					if ((permissionPointer->established == true) && (permissionPointer->fragments == true))
						fprintf(reportFile, ", ");
					if (permissionPointer->fragments == true)
						fprintf(reportFile, "Fragments");
					fprintf(reportFile, " %s", tablerow_end);
					permissionPointer = permissionPointer->next;
				}
				output_table(reportFile, reportFormat, false, passwords, tempString2, &settings_accessext_heading, tableNum, friendlyHostname);
				tableNum++;
			}

			// Next Access List
			accessPointer = accessPointer->next;
		}

		output_endsection(reportFile, reportFormat, subsection_end);
	}

	// End Section
	output_endsection(reportFile, reportFormat, section_end);

	// Appendix
	if (appendix == true)
		generateAppendix(reportFile, reportFormat, friendlyHostname, tableNum, abs, commonPorts, glossary);

	// Output end of file stuff
	output_fileend(reportFormat, reportFile);
}

