/***************************************************************************
 *   Copyright (C) 2006-2007 by Ian Ventura-Whiting (Fizz)                 *
 *   Fizz@Titania.co.uk                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


void generatePIXReport(int reportFormat, struct ciscoPIXConfig *configPIX, const char *outputFile, int maxSize, int passwords, int fullReport, FILE *cssFile, struct latexConfig *latex, const char *companyName, int abs, int appendix, int glossary, int commonPorts, int expandACL)
{
	// Variables
	FILE *reportFile;
	struct interfacePIXConfig *interfacePIXConfigPointer;
	struct accessListPIX *accessPointer;
	struct accessListPIXPermission *permissionPointer;
	struct objectGroupList *objectGroupPointer;
	struct objectList *objectPointer;
	struct namePIXList *namePIXPointer;
	struct enablePIXPassword *enablePIXPointer;
	struct usernamePIX *usernamePIXPointer;
	struct staticPIX *staticPIXPointer;
	struct fixupList *fixupPIXPointer;
	struct icmpAccess *icmpPIXPointer;
	struct routePIX *routePIXPointer;
	int tableNum = 1;
	int tempInt;
	int lineCount;
	int issues = 0;
	char tempString[maxSize];
	char tempString2[maxSize];
	char friendlyHostname[maxSize];
	char *appendixFile;

	// Counter Variables...
	int accessListIssues = 0;
	int denyIssue = false;
	int timeoutIssues = 0;

	// Table Pointers
	const char *tablerow_start;
	const char *tablerow_mid;
	const char *tablerow_end;
	switch (reportFormat)
	{
		case output_html:
			tablerow_start = table_html_rowstart;
			tablerow_mid = table_html_rowmid;
			tablerow_end = table_html_rowend;
			break;
		case output_xml:
			tablerow_start = table_xml_rowstart;
			tablerow_mid = table_xml_rowmid;
			tablerow_end = table_xml_rowend;
			break;
		case output_text:
			tablerow_start = table_text_rowstart;
			tablerow_mid = table_text_rowmid;
			tablerow_end = table_text_rowend;
			break;
		case output_latex:
			tablerow_start = table_latex_rowstart;
			tablerow_mid = table_latex_rowmid;
			tablerow_end = table_latex_rowend;
			break;
	}

	// Create / open file for output
	if (outputFile[0] == 0)
		reportFile = stdout;
	else
	{
		reportFile = fopen(outputFile, "w");
		if (reportFile == NULL)
		{
			printf("\n%sERROR: Could not create file '%s'%s\n", COL_RED, outputFile, RESET);
			return;
		}
	}

	// Set output friendly hostname
	outputFriendly(configPIX->hostname, friendlyHostname, maxSize, reportFormat);

	// Output file heading (if required)
	output_filestart(reportFile, reportFormat, type_firewall, friendlyHostname, fullReport, cssFile, latex, companyName, maxSize);

	// Report section?
	if (fullReport == true)
	{

		// Output report introduction
		sprintf(tempString, "Cisco Firewall %s Security-Related Issues", friendlyHostname);
		output_parseTitle(tempString, reportFile, reportFormat, friendlyHostname, rate_none, section_main_sec);
		output_parseTitle("Introduction", reportFile, reportFormat, friendlyHostname, rate_none, section_security);
		output_parseText(report_security_cisco_intro, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, 0, 0);

		// ---------------------------------------
		// Get List of Issues (and generate index)
		// ---------------------------------------

		// Start of index
		switch (reportFormat)
		{
			case output_html:
				fprintf(reportFile, "<ul>\n");
				break;
		}

		// Timeout
		if (configPIX->ssh != 0)
		{
			if (configPIX->ssh->timeout * 60 > connectionTimeout)
				timeoutIssues++;
		}
		if (timeoutIssues > 0)
		{
			output_parseIndex(report_timeout_title, reportFile, reportFormat, friendlyHostname, section_security);
			issues++;
		}

		// Access Lists
		accessPointer = configPIX->access;
		while (accessPointer != 0)
		{
			permissionPointer = accessPointer->permission;
			while (permissionPointer != 0)
			{
				// If permit
				if (permissionPointer->permission == perm_permit)
				{
					// Standard Access List...
					if (accessPointer->type == access_std)
					{
						// From any source...
						if (strcmp(permissionPointer->source, "any") == 0)
							accessListIssues++;
					}
					// Extended Access List / old...
					else
					{
						if ((((strcmp(permissionPointer->dest, "any") == 0) && (strcmp(permissionPointer->source, "any") == 0)) || (strcmp(permissionPointer->source, "any") == 0) || (strcmp(permissionPointer->destService, "any") == 0)) && (strcmp(permissionPointer->protocol, "icmp") != 0))
							accessListIssues++;
					}
				}
				// Deny & Log...
				if ((permissionPointer->permission == perm_deny) && (permissionPointer->log == false) && (accessPointer->type != access_std))
					accessListIssues++;

				permissionPointer = permissionPointer->next;
			}
			accessPointer = accessPointer->next;
		}
		if (configPIX->access == 0)
			accessListIssues = -1;
		if (accessListIssues != 0)
		{
			issues++;
			output_parseIndex(report_accesslist_title, reportFile, reportFormat, friendlyHostname, section_security);
		}

		// SSH Protocol Version
		if (configPIX->ssh != 0)
		{
			if (configPIX->ssh->version < 2)
			{
				issues++;
				output_parseIndex(report_sshproto_title, reportFile, reportFormat, friendlyHostname, section_security);
			}
		}

		// End of index
		switch (reportFormat)
		{
			case output_html:
				fprintf(reportFile, "</ul><br>\n");
				break;
			case output_text:
				fprintf(reportFile, "\n\n");
				break;
		}

		// If No Issues
		if (issues == 0)
			output_parseText(report_noissues, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, 0, 0);

		output_endsection(reportFile, reportFormat, subsection_end);

		// ----------------------
		// Security Issues Report
		// ----------------------

		if (timeoutIssues > 0)
		{
			output_parseTitle(report_timeout_title, reportFile, reportFormat, friendlyHostname, rate_med, section_security);
			output_parseText(report_timeout_obs1, reportFile, reportFormat, companyName, friendlyHostname, section_obs, rate_none, timeoutIssues, tableNum);
			if ((connectionTimeout % 60) == 0)
			{
				numToWord(reportFile, (connectionTimeout / 60));
				fprintf(reportFile, " minutes");
			}
			else
			{
				numToWord(reportFile, connectionTimeout / 60);
				fprintf(reportFile, " minutes and ");
				numToWord(reportFile, connectionTimeout % 60);
				fprintf(reportFile, " seconds");
			}
			output_parseText(report_timeout_obs2, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, timeoutIssues, tableNum);
			if (timeoutIssues > 1)
			{
				output_table(reportFile, reportFormat, true, false, report_timeout_table, &report_timeout_heading, tableNum, friendlyHostname);
				if (configPIX->ssh != 0)
				{
					if ((configPIX->ssh->timeout * 60) > connectionTimeout)
					{
						fprintf(reportFile, "%sSSH Session Timeout%s", tablerow_start, tablerow_mid);
						numToWord(reportFile, configPIX->ssh->timeout);
						fprintf(reportFile, " minutes%s", tablerow_end);
					}
				}
				output_table(reportFile, reportFormat, false, false, report_timeout_table, &report_timeout_heading, tableNum, friendlyHostname);
				tableNum++;
			}
			else
			{
				if (configPIX->ssh != 0)
				{
					if ((configPIX->ssh->timeout * 60) > connectionTimeout)
					{
						fprintf(reportFile, "The SSH session timeout was ");
						numToWord(reportFile, configPIX->ssh->timeout);
						fprintf(reportFile, " minutes.");
					}
				}
				output_newPara(reportFile, reportFormat);
			}
			output_endsection(reportFile, reportFormat, section_obs);
			output_parseText(report_timeout_impact, reportFile, reportFormat, companyName, friendlyHostname, section_imp, rate_high, 0, 0);
			output_endsection(reportFile, reportFormat, section_imp);
			output_parseText(report_timeout_ease, reportFile, reportFormat, companyName, friendlyHostname, section_ease, rate_difficult, 0, 0);
			output_endsection(reportFile, reportFormat, section_ease);
			output_parseText(report_timeout_rec1, reportFile, reportFormat, companyName, friendlyHostname, section_rec, rate_none, 0, 0);
			if ((connectionTimeout % 60) == 0)
			{
				numToWord(reportFile, (connectionTimeout / 60));
				fprintf(reportFile, " minutes");
			}
			else
			{
				numToWord(reportFile, connectionTimeout / 60);
				fprintf(reportFile, " minutes and ");
				numToWord(reportFile, (connectionTimeout % 60));
				fprintf(reportFile, " seconds");
			}
			output_parseText(report_timeout_rec2, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, 0, 0);
			output_endsection(reportFile, reportFormat, section_rec);
		}

		// Access Lists
		if (accessListIssues != 0)
		{
			denyIssue = false;
			output_parseTitle(report_accesslist_title, reportFile, reportFormat, friendlyHostname, rate_med, section_security);
			output_parseText(report_accesslist_obs1, reportFile, reportFormat, companyName, friendlyHostname, section_obs, rate_none, accessListIssues, tableNum);
			// One Access List issue...
			if (accessListIssues == 1)
			{
				accessPointer = configPIX->access;
				while (accessPointer != 0)
				{
					permissionPointer = accessPointer->permission;
					lineCount = 1;
					while (permissionPointer != 0)
					{
						if (permissionPointer->permission == perm_permit)
						{
							// Standard Access List...
							if (accessPointer->type == access_std)
							{
								// From any source...
								if (strcmp(permissionPointer->source, "any") == 0)
									fprintf(reportFile, "Line %d in ACL %s %s", lineCount, accessPointer->name, report_accesslist_obs6);
							}
							// Extended Access List / old...
							else
							{
								// To any destination and service from any source...
								if ((strcmp(permissionPointer->dest, "any") == 0) && (strcmp(permissionPointer->source, "any") == 0) && (strcmp(permissionPointer->destService, "any") == 0) && (strcmp(permissionPointer->protocol, "icmp") != 0))
									fprintf(reportFile, "Line %d in ACL %s %s", lineCount, accessPointer->name, report_accesslist_obs8);

								// To any destination and service...
								else if ((strcmp(permissionPointer->dest, "any") == 0) && (strcmp(permissionPointer->destService, "any") == 0) && (strcmp(permissionPointer->protocol, "icmp") != 0))
									fprintf(reportFile, "Line %d in ACL %s %s", lineCount, accessPointer->name, report_accesslist_obs5);

								// To any destination from any source...
								else if ((strcmp(permissionPointer->dest, "any") == 0) && (strcmp(permissionPointer->source, "any") == 0) && (strcmp(permissionPointer->protocol, "icmp") != 0))
									fprintf(reportFile, "Line %d in ACL %s %s", lineCount, accessPointer->name, report_accesslist_obs9);

								// To any service...
								else if ((strcmp(permissionPointer->destService, "any") == 0) && (strcmp(permissionPointer->protocol, "icmp") != 0))
									fprintf(reportFile, "Line %d in ACL %s %s", lineCount, accessPointer->name, report_accesslist_obs10);

								// From any source...
								else if ((strcmp(permissionPointer->source, "any") == 0) && (strcmp(permissionPointer->protocol, "icmp") != 0))
									fprintf(reportFile, "Line %d in ACL %s %s", lineCount, accessPointer->name, report_accesslist_obs6);
							}
						}
						// Deny & Log...
						if ((permissionPointer->permission == perm_deny) && (permissionPointer->log == false) && (accessPointer->type != access_std))
						{
							fprintf(reportFile, "%s %s %s", report_accesslist_obs14, accessPointer->name, report_accesslist_obs13);
							denyIssue = true;
						}
						permissionPointer = permissionPointer->next;
						lineCount++;
					}
					accessPointer = accessPointer->next;
				}
				output_newPara(reportFile, reportFormat);
			}
			// More than one issue...
			else
			{
				output_table(reportFile, reportFormat, true, passwords, report_accesslist_table1, &report_accesslists_heading, tableNum, friendlyHostname);
				accessPointer = configPIX->access;
				while (accessPointer != 0)
				{
					permissionPointer = accessPointer->permission;
					lineCount = 1;
					while (permissionPointer != 0)
					{
						if (permissionPointer->permission == perm_permit)
						{
							// Standard Access List...
							if (accessPointer->type == access_std)
							{
								// From any source...
								if (strcmp(permissionPointer->source, "any") == 0)
								{
									outputFriendly(accessPointer->name, tempString, maxSize, reportFormat);
									fprintf(reportFile, "%s%s%s%d%s", tablerow_start, tempString, tablerow_mid, lineCount, tablerow_mid);
									fprintf(reportFile, "%s%s", report_accesslist_table6, tablerow_end);
								}
							}
							// Extended Access List / old...
							else
							{
								// To any destination and service from any source...
								if ((strcmp(permissionPointer->dest, "any") == 0) && (strcmp(permissionPointer->source, "any") == 0) && (strcmp(permissionPointer->destService, "any") == 0) && (strcmp(permissionPointer->protocol, "icmp") != 0))
								{
									outputFriendly(accessPointer->name, tempString, maxSize, reportFormat);
									fprintf(reportFile, "%s%s%s%d%s", tablerow_start, tempString, tablerow_mid, lineCount, tablerow_mid);
									fprintf(reportFile, "%s%s", report_accesslist_table5, tablerow_end);
								}
								// To any destination and service...
								else if ((strcmp(permissionPointer->dest, "any") == 0) && (strcmp(permissionPointer->destService, "any") == 0) && (strcmp(permissionPointer->protocol, "icmp") != 0))
								{
									outputFriendly(accessPointer->name, tempString, maxSize, reportFormat);
									fprintf(reportFile, "%s%s%s%d%s", tablerow_start, tempString, tablerow_mid, lineCount, tablerow_mid);
									fprintf(reportFile, "%s%s", report_accesslist_table7, tablerow_end);
								}
								// To any destination from any source...
								else if ((strcmp(permissionPointer->dest, "any") == 0) && (strcmp(permissionPointer->source, "any") == 0) && (strcmp(permissionPointer->protocol, "icmp") != 0))
								{
									outputFriendly(accessPointer->name, tempString, maxSize, reportFormat);
									fprintf(reportFile, "%s%s%s%d%s", tablerow_start, tempString, tablerow_mid, lineCount, tablerow_mid);
									fprintf(reportFile, "%s%s", report_accesslist_table4, tablerow_end);
								}
								// To any service...
								else if ((strcmp(permissionPointer->destService, "any") == 0) && (strcmp(permissionPointer->protocol, "icmp") != 0))
								{
									outputFriendly(accessPointer->name, tempString, maxSize, reportFormat);
									fprintf(reportFile, "%s%s%s%d%s", tablerow_start, tempString, tablerow_mid, lineCount, tablerow_mid);
									fprintf(reportFile, "%s%s", report_accesslist_table3, tablerow_end);
								}
								// From any source...
								else if ((strcmp(permissionPointer->source, "any") == 0) && (strcmp(permissionPointer->protocol, "icmp") != 0))
								{
									outputFriendly(accessPointer->name, tempString, maxSize, reportFormat);
									fprintf(reportFile, "%s%s%s%d%s", tablerow_start, tempString, tablerow_mid, lineCount, tablerow_mid);
									fprintf(reportFile, "%s%s", report_accesslist_table6, tablerow_end);
								}
							}
						}
						// Deny & Log last...
						if ((permissionPointer->permission == perm_deny) && (permissionPointer->log == false) && (accessPointer->type != access_std))
						{
							outputFriendly(accessPointer->name, tempString, maxSize, reportFormat);
							fprintf(reportFile, "%s%s%s%d%s", tablerow_start, tempString, tablerow_mid, lineCount, tablerow_mid);
							fprintf(reportFile, "%s%s", report_accesslist_table2, tablerow_end);
							denyIssue = true;
						}
						permissionPointer = permissionPointer->next;
						lineCount++;
					}
					accessPointer = accessPointer->next;
				}
				output_table(reportFile, reportFormat, false, passwords, report_accesslist_table1, &report_accesslists_heading, tableNum, friendlyHostname);
				tableNum++;
			}
			output_endsection(reportFile, reportFormat, section_obs);
			output_parseText(report_accesslist_impact, reportFile, reportFormat, companyName, friendlyHostname, section_imp, rate_med, 0, 0);
			output_endsection(reportFile, reportFormat, section_imp);
			output_parseText(report_accesslist_ease, reportFile, reportFormat, companyName, friendlyHostname, section_ease, rate_noease, 0, 0);
			output_endsection(reportFile, reportFormat, section_ease);
			if (denyIssue == true)
				output_parseText(report_accesslist_rec, reportFile, reportFormat, companyName, friendlyHostname, section_rec, rate_none, 2, 0);
			else
				output_parseText(report_accesslist_rec, reportFile, reportFormat, companyName, friendlyHostname, section_rec, rate_none, 1, 0);
			output_endsection(reportFile, reportFormat, section_rec);
		}

		// SSH Protocol Version
		if (configPIX->ssh != 0)
		{
			if (configPIX->ssh->version < 2)
			{
				output_parseTitle(report_sshproto_title, reportFile, reportFormat, friendlyHostname, rate_low, section_security);
				output_parseText(report_sshproto_obs, reportFile, reportFormat, companyName, friendlyHostname, section_obs, rate_none, 0, 0);
				output_endsection(reportFile, reportFormat, section_obs);
				output_parseText(report_sshproto_impact, reportFile, reportFormat, companyName, friendlyHostname, section_imp, rate_lowmed, 0, 0);
				output_endsection(reportFile, reportFormat, section_imp);
				output_parseText(report_sshproto_ease, reportFile, reportFormat, companyName, friendlyHostname, section_ease, rate_difficult, 0, 0);
				output_endsection(reportFile, reportFormat, section_ease);
				output_parseText(report_sshproto_rec1, reportFile, reportFormat, companyName, friendlyHostname, section_rec, rate_none, 0, 0);
				output_endsection(reportFile, reportFormat, section_rec);
			}
		}

		// End Section
		output_endsection(reportFile, reportFormat, section_end);
	}

	// --------------------
	// Configuration Report
	// --------------------

	// Output report introduction
	sprintf(tempString, "Cisco Firewall %s Configuration", friendlyHostname);
	output_parseTitle(tempString, reportFile, reportFormat, friendlyHostname, rate_none, section_main_con);
	output_parseTitle("Introduction", reportFile, reportFormat, friendlyHostname, rate_none, section_config);
	output_parseText(report_config_cisco_intro, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, 0, 0);

	// ------------------------------------------------------
	// Get List of Configuration options (and generate index)
	// ------------------------------------------------------

	// Start of index
	switch (reportFormat)
	{
		case output_html:
			fprintf(reportFile, "<ul>\n");
			break;
	}

	output_parseIndex(settings_general_title, reportFile, reportFormat, friendlyHostname, section_config);
	if (((configPIX->enable != 0) || (configPIX->password[0] != 0)) && (passwords == true))
		output_parseIndex(settings_users_title, reportFile, reportFormat, friendlyHostname, section_config);
	if (configPIX->ssh != 0)
		output_parseIndex(settings_ssh_title, reportFile, reportFormat, friendlyHostname, section_config);
	if (configPIX->interface != 0)
		output_parseIndex(settings_inter_title, reportFile, reportFormat, friendlyHostname, section_config);
	if (configPIX->staticNAT !=0)
		output_parseIndex(settings_natpat_title, reportFile, reportFormat, friendlyHostname, section_config);
	if (configPIX->route !=0)
		output_parseIndex(settings_routing_title, reportFile, reportFormat, friendlyHostname, section_config);
	if (configPIX->access != 0)
		output_parseIndex(settings_accesslists_title, reportFile, reportFormat, friendlyHostname, section_config);
	if (configPIX->icmp != 0)
		output_parseIndex(settings_icmpaccess_title, reportFile, reportFormat, friendlyHostname, section_config);
	if (configPIX->fixup != 0)
		output_parseIndex(settings_protoinspect_title, reportFile, reportFormat, friendlyHostname, section_config);
	if (configPIX->objectGroup != 0)
		output_parseIndex(settings_objects_title, reportFile, reportFormat, friendlyHostname, section_config);
	if (configPIX->nameList != 0)
		output_parseIndex(settings_namemap_title, reportFile, reportFormat, friendlyHostname, section_config);

	// End of index
	switch (reportFormat)
	{
		case output_html:
			fprintf(reportFile, "</ul><br>\n");
			break;
		case output_text:
			fprintf(reportFile, "\n\n");
			break;
	}
	output_endsection(reportFile, reportFormat, subsection_end);

	// General Device Settings
	output_parseTitle(settings_general_title, reportFile, reportFormat, friendlyHostname, rate_none, section_config);
	output_table(reportFile, reportFormat, true, passwords, settings_general_table, &settings_general_heading, tableNum, friendlyHostname);
	fprintf(reportFile, "%sHostname%s%s %s", tablerow_start, tablerow_mid, friendlyHostname, tablerow_end);
	if (configPIX->domainName[0] != 0)
		fprintf(reportFile, "%sDomain Name%s%s%s", tablerow_start, tablerow_mid, configPIX->domainName, tablerow_end);
	if ((configPIX->version[0] != 0) && (configPIX->type == firewall_type_pix))
	{
		addAbbreviation("PIX", false);
		fprintf(reportFile, "%sPIX Version%s%s %s", tablerow_start, tablerow_mid, configPIX->version, tablerow_end);
	}
	else if ((configPIX->version[0] != 0) && (configPIX->type == firewall_type_asa))
	{
		addAbbreviation("ASA", false);
		fprintf(reportFile, "%sASA Version%s%s %s", tablerow_start, tablerow_mid, configPIX->version, tablerow_end);
	}
	else if ((configPIX->version[0] != 0) && (configPIX->type == firewall_type_fwsm))
	{
		addAbbreviation("FWSM", false);
		fprintf(reportFile, "%sFWSM Version%s%s %s", tablerow_start, tablerow_mid, configPIX->version, tablerow_end);
	}
	if (configPIX->transparent == true)
		fprintf(reportFile, "%sTransparent Firewall%sYes%s", tablerow_start, tablerow_mid, tablerow_end);
	else
		fprintf(reportFile, "%sTransparent Firewall%sNo%s", tablerow_start, tablerow_mid, tablerow_end);
	output_table(reportFile, reportFormat, false, passwords, settings_general_table, &settings_general_heading, tableNum, friendlyHostname);
	tableNum++;
	output_endsection(reportFile, reportFormat, subsection_end);

	// Users / Passwords / Authentication
	if ((configPIX->enable != 0) || (configPIX->password[0] != 0) || (configPIX->username != 0))
	{
		output_parseTitle(settings_users_title, reportFile, reportFormat, friendlyHostname, rate_none, section_config);
		// Login Password
		if (configPIX->password[0] != 0)
		{
			if (reportFormat == output_xml)
				fprintf(reportFile, "    <text>");
			fprintf(reportFile, "%s %s.", settings_logon_password, configPIX->password);
			if (reportFormat == output_xml)
				fprintf(reportFile, "</text>\n");
			output_newPara(reportFile, reportFormat);
		}

		// Enable Password
		if (configPIX->enable != 0)
		{
			output_table(reportFile, reportFormat, true, passwords, settings_enable_table, &report_enablepix_heading, tableNum, friendlyHostname);
			enablePIXPointer = configPIX->enable;
			while (enablePIXPointer != 0)
			{
				fprintf(reportFile, "%s%d", tablerow_start, enablePIXPointer->level);
				fprintf(reportFile, "%s%s", tablerow_mid, enablePIXPointer->passwordEncrypted);
				fprintf(reportFile, "%s", tablerow_end);
				enablePIXPointer = enablePIXPointer->next;
			}
			output_table(reportFile, reportFormat, false, passwords, settings_enable_table, &report_enablepix_heading, tableNum, friendlyHostname);
			tableNum++;
		}

		// Usernames
		if (configPIX->username != 0)
		{
			output_table(reportFile, reportFormat, true, passwords, settings_users_table, &report_userspix_heading, tableNum, friendlyHostname);
			usernamePIXPointer = configPIX->username;
			while (usernamePIXPointer != 0)
			{
				fprintf(reportFile, "%s%s%s", tablerow_start, usernamePIXPointer->username, tablerow_mid);
				fprintf(reportFile, "%d%s", usernamePIXPointer->level, tablerow_mid);
				switch (usernamePIXPointer->encryption)
				{
					case encrypt_none:
						fprintf(reportFile, "N/A");
						break;
					case encrypt_pix:
						fprintf(reportFile, "Standard");
						break;
					case encrypt_mschap:
						fprintf(reportFile, "MS-CHAP");
						addAbbreviation("MS-CHAP", false);
						break;
					case encrypt_ntenc:
						fprintf(reportFile, "NT Encrypted");
						addAbbreviation("NT", false);
						break;
				}
				if (passwords == true)
				{
					if (usernamePIXPointer->passwordEncrypted[0] == 0)
						fprintf(reportFile, "%sNone%s", tablerow_mid, tablerow_end);
					else
						fprintf(reportFile, "%s%s%s", tablerow_mid, usernamePIXPointer->passwordEncrypted, tablerow_end);
				}
				else
					fprintf(reportFile, "%s", tablerow_end);
				usernamePIXPointer = usernamePIXPointer->next;
			}
			output_table(reportFile, reportFormat, false, passwords, settings_users_table, &report_userspix_heading, tableNum, friendlyHostname);
			tableNum++;
		}
		output_endsection(reportFile, reportFormat, subsection_end);
	}

	// SSH
	if (configPIX->ssh != 0)
	{
		output_parseTitle(settings_ssh_title, reportFile, reportFormat, friendlyHostname, rate_none, section_config);
		output_table(reportFile, reportFormat, true, passwords, settings_ssh_table, &settings_general_heading, tableNum, friendlyHostname);
		if (configPIX->ssh->version == 0)
			fprintf(reportFile, "%sProtocol Version%s1 and 2%s", tablerow_start, tablerow_mid, tablerow_end);
		else
			fprintf(reportFile, "%sProtocol Version%s%d%s", tablerow_start, tablerow_mid, configPIX->ssh->version, tablerow_end);
		fprintf(reportFile, "%sSession Timeout%s", tablerow_start, tablerow_mid);
		numToWord(reportFile, configPIX->ssh->timeout);
		fprintf(reportFile, " minutes%s", tablerow_end);
		if (configPIX->ssh->ipAddress[0] == 0)
			fprintf(reportFile, "%sManagement Host%sAny%s", tablerow_start, tablerow_mid, tablerow_end);
		else
			fprintf(reportFile, "%sManagement Host%s%s %s%s", tablerow_start, tablerow_mid, configPIX->ssh->ipAddress, configPIX->ssh->netMask, tablerow_end);
		fprintf(reportFile, "%sManagement Interface%s%s%s", tablerow_start, tablerow_mid, configPIX->ssh->interface, tablerow_end);
		output_table(reportFile, reportFormat, false, passwords, settings_ssh_table, &settings_general_heading, tableNum, friendlyHostname);
		tableNum++;
		output_endsection(reportFile, reportFormat, subsection_end);
		addAbbreviation("SSH", false);
	}

	// Interfaces
	if (configPIX->interface !=0)
	{
		output_parseTitle(settings_inter_title, reportFile, reportFormat, friendlyHostname, rate_none, section_config);
		output_table(reportFile, reportFormat, true, passwords, settings_inter_table, &settings_interpix_heading, tableNum, friendlyHostname);
		interfacePIXConfigPointer = configPIX->interface;
		while (interfacePIXConfigPointer != 0)
		{
			fprintf(reportFile, "%s%s%s", tablerow_start, interfacePIXConfigPointer->interface, tablerow_mid);
			outputFriendly(interfacePIXConfigPointer->name, tempString, maxSize, reportFormat);
			fprintf(reportFile, "%s%s", tempString, tablerow_mid);
			if (interfacePIXConfigPointer->shutdown == true)
				fprintf(reportFile, "Yes");
			else
				fprintf(reportFile, "No");
			fprintf(reportFile, "%s%s%s", tablerow_mid, interfacePIXConfigPointer->ipAddress, tablerow_mid);
			fprintf(reportFile, "%s%s", interfacePIXConfigPointer->netMask, tablerow_mid);
			fprintf(reportFile, "%d%s", interfacePIXConfigPointer->security, tablerow_mid);
			switch (interfacePIXConfigPointer->netSpeed)
			{
				case int_netspeed_auto:
					fprintf(reportFile, "Auto");
					break;
				case int_netspeed_100full:
					fprintf(reportFile, "100Mb Full Duplex");
					break;
			}
			fprintf(reportFile, "%s%d%s", tablerow_mid, interfacePIXConfigPointer->mtu, tablerow_mid);
			if (interfacePIXConfigPointer->in[0] == 0)
				fprintf(reportFile, " %s", tablerow_mid);
			else
			{
				outputFriendly(interfacePIXConfigPointer->in, tempString, maxSize, reportFormat);
				fprintf(reportFile, "%s%s", tempString, tablerow_mid);
			}
			if (interfacePIXConfigPointer->out[0] == 0)
				fprintf(reportFile, " ");
			else
			{
				outputFriendly(interfacePIXConfigPointer->out, tempString, maxSize, reportFormat);
				fprintf(reportFile, "%s", tempString);
			}
			fprintf(reportFile, "%s", tablerow_end);
			interfacePIXConfigPointer = interfacePIXConfigPointer->next;
		}
		output_table(reportFile, reportFormat, false, passwords, settings_inter_table, &settings_interpix_heading, tableNum, friendlyHostname);
		tableNum++;
		output_endsection(reportFile, reportFormat, subsection_end);
		addAbbreviation("ACL", false);
		addAbbreviation("MTU", false);
		addAbbreviation("IP", false);
	}

	// NAT / PAT
	if (configPIX->staticNAT !=0)
	{
		output_parseTitle(settings_natpat_title, reportFile, reportFormat, friendlyHostname, rate_none, section_config);
		output_parseText(settings_natpat_desc, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, 0, 0);
		output_table(reportFile, reportFormat, true, passwords, settings_nat_static_table, &settings_nat_static_heading, tableNum, friendlyHostname);
		staticPIXPointer = configPIX->staticNAT;
		while (staticPIXPointer != 0)
		{
			outputFriendly(staticPIXPointer->interfaceReal, tempString, maxSize, reportFormat);
			fprintf(reportFile, "%s%s%s", tablerow_start, tempString, tablerow_mid);
			if (staticPIXPointer->interface == true)
				fprintf(reportFile, "Interface");
			else if (staticPIXPointer->accessList[0] != 0)
			{
				outputFriendly(staticPIXPointer->accessList, tempString, maxSize, reportFormat);
				fprintf(reportFile, "ACL: %s", tempString);
				addAbbreviation("ACL", false);
			}
			else if (staticPIXPointer->ipNetMaskReal[0] != 0)
				fprintf(reportFile, "%s %s", staticPIXPointer->ipReal, staticPIXPointer->ipNetMaskReal);
			else
				fprintf(reportFile, "%s", staticPIXPointer->ipReal);
			outputFriendly(staticPIXPointer->interfaceMapped, tempString, maxSize, reportFormat);
			fprintf(reportFile, "%s%s%s", tablerow_mid, tempString, tablerow_mid);
			outputFriendly(staticPIXPointer->ipMapped, tempString, maxSize, reportFormat);
			fprintf(reportFile, "%s%s", tempString, tablerow_mid);
			if (staticPIXPointer->dns == true)
				fprintf(reportFile, "Yes%s", tablerow_mid);
			else
				fprintf(reportFile, "No%s", tablerow_mid);
			if (staticPIXPointer->randomseq == true)
				fprintf(reportFile, "Yes%s", tablerow_mid);
			else
				fprintf(reportFile, "No%s", tablerow_mid);
			if (staticPIXPointer->nailed == true)
				fprintf(reportFile, "Yes%s", tablerow_mid);
			else
				fprintf(reportFile, "No%s", tablerow_mid);
			if (staticPIXPointer->tcpMaxConnections == 0)
				fprintf(reportFile, "Unlimited%s", tablerow_mid);
			else
				fprintf(reportFile, "%d%s", staticPIXPointer->tcpMaxConnections, tablerow_mid);
			if (staticPIXPointer->embrionicConnections == 0)
				fprintf(reportFile, "Unlimited%s", tablerow_mid);
			else
				fprintf(reportFile, "%d%s", staticPIXPointer->embrionicConnections, tablerow_mid);
			if (staticPIXPointer->udpMaxConnections == 0)
				fprintf(reportFile, "Unlimited%s", tablerow_end);
			else
				fprintf(reportFile, "%d%s", staticPIXPointer->udpMaxConnections, tablerow_end);

			staticPIXPointer = staticPIXPointer->next;
		}
		output_table(reportFile, reportFormat, false, passwords, settings_nat_static_table, &settings_nat_static_heading, tableNum, friendlyHostname);
		tableNum++;
		output_endsection(reportFile, reportFormat, subsection_end);
	}

	// Routing
	if (configPIX->route != 0)
	{
		output_parseTitle(settings_routing_title, reportFile, reportFormat, friendlyHostname, rate_none, section_config);
		output_parseText(settings_routing_desc, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, 0, 0);
		output_table(reportFile, reportFormat, true, passwords, settings_static_route_table, &settings_route_heading, tableNum, friendlyHostname);
		routePIXPointer = configPIX->route;
		while (routePIXPointer != 0)
		{
			outputFriendly(routePIXPointer->interface, tempString, maxSize, reportFormat);
			fprintf(reportFile,"%s%s%s", tablerow_start, tempString, tablerow_mid);
			outputFriendly(routePIXPointer->ipAddress, tempString, maxSize, reportFormat);
			fprintf(reportFile, "%s%s", tempString, tablerow_mid);
			outputFriendly(routePIXPointer->netMask, tempString, maxSize, reportFormat);
			fprintf(reportFile, "%s%s", tempString, tablerow_mid);
			outputFriendly(routePIXPointer->gateway, tempString, maxSize, reportFormat);
			fprintf(reportFile, "%s%s", tempString, tablerow_mid);
			fprintf(reportFile, "%d%s", routePIXPointer->metric, tablerow_mid);
			if (routePIXPointer->track == 0)
				fprintf(reportFile, "-%s", tablerow_mid);
			else
				fprintf(reportFile, "%d%s", routePIXPointer->track, tablerow_mid);
			if (routePIXPointer->tunneled == true)
				fprintf(reportFile, "Yes%s", tablerow_end);
			else
				fprintf(reportFile, "No%s", tablerow_end);
			routePIXPointer = routePIXPointer->next;
		}
		output_table(reportFile, reportFormat, false, passwords, settings_static_route_table, &settings_route_heading, tableNum, friendlyHostname);
		tableNum++;
		output_endsection(reportFile, reportFormat, subsection_end);
	}

	// Access Lists
	if (configPIX->access != 0)
	{
		output_parseTitle(settings_accesslists_title, reportFile, reportFormat, friendlyHostname, rate_none, section_config);
		// Intro text
		output_parseText(settings_accesslists1, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, 0, 0);
		output_newPara(reportFile, reportFormat);
		if (reportFormat == output_xml)
			fprintf(reportFile, "</text>\n");
		output_parseText(settings_accesslists3, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, 0, 0);
		// General ACL Settings
		output_table(reportFile, reportFormat, true, passwords, settings_accesslist_table, &settings_general_heading, tableNum, friendlyHostname);
		fprintf(reportFile, "%sAlert Interval%s%d seconds%s", tablerow_start, tablerow_mid, configPIX->accessListAlertInterval, tablerow_end);
		fprintf(reportFile, "%sMaximum Deny Flows%s%d%s", tablerow_start, tablerow_mid, configPIX->accessListDenyFlowMax, tablerow_end);
		output_table(reportFile, reportFormat, false, passwords, settings_accesslist_table, &settings_general_heading, tableNum, friendlyHostname);
		tableNum++;
		// Old Access Lists
		if (configPIX->oldAccessList > 0)
		{
			accessPointer = configPIX->access;
			while (accessPointer != 0)
			{
				if (accessPointer->type == access_old)
				{
					sprintf(tempString, "%s %s", settings_accesslists_table, accessPointer->name);
					outputFriendly(tempString, tempString2, maxSize, reportFormat);
					output_table(reportFile, reportFormat, true, passwords, tempString2, &settings_accesspix_heading, tableNum, friendlyHostname);
					permissionPointer = accessPointer->permission;
					tempInt = 1;
					while (permissionPointer != 0)
					{
						outputPIXOldACL(configPIX, reportFile, reportFormat, tempInt, expandACL, maxSize, permissionPointer->permission, permissionPointer->protocol, permissionPointer->source, permissionPointer->sourceMask, permissionPointer->sourceType, permissionPointer->sourceService, permissionPointer->sourceServiceOp, permissionPointer->dest, permissionPointer->destMask, permissionPointer->destType, permissionPointer->destService, permissionPointer->destServiceOp);
						permissionPointer = permissionPointer->next;
						tempInt++;
					}
					output_table(reportFile, reportFormat, false, passwords, tempString2, &settings_accesspix_heading, tableNum, friendlyHostname);
					tableNum++;
				}
				accessPointer = accessPointer->next;
			}
		}
		// Standard Access Lists
		if (configPIX->standardAccessList > 0)
		{
			accessPointer = configPIX->access;
			while (accessPointer != 0)
			{
				if (accessPointer->type == access_std)
				{
					if (accessPointer->remark[0] == 0)
						sprintf(tempString, "%s %s", settings_accessstd_table, accessPointer->name);
					else
						sprintf(tempString, "%s %s (%s)", settings_accessstd_table, accessPointer->name, accessPointer->remark);
					outputFriendly(tempString, tempString2, maxSize, reportFormat);
					output_table(reportFile, reportFormat, true, passwords, tempString2, &settings_accessstdpix_heading, tableNum, friendlyHostname);
					permissionPointer = accessPointer->permission;
					tempInt = 1;
					while (permissionPointer != 0)
					{
						fprintf(reportFile, "%s%d%s", tablerow_start, tempInt, tablerow_mid);
						tempInt++;
						if (permissionPointer->permission == perm_permit)
							fprintf(reportFile, "Permit%s", tablerow_mid);
						else
							fprintf(reportFile, "Deny%s", tablerow_mid);
						outputFriendly(permissionPointer->source, tempString, maxSize, reportFormat);
						fprintf(reportFile, "%s%s", tempString, tablerow_end);
						permissionPointer = permissionPointer->next;
					}
					output_table(reportFile, reportFormat, false, passwords, tempString2, &settings_accessstdpix_heading, tableNum, friendlyHostname);
					tableNum++;
				}
				accessPointer = accessPointer->next;
			}
		}
		// Extended Access Lists
		if (configPIX->extendedAccessList > 0)
		{
			accessPointer = configPIX->access;
			while (accessPointer != 0)
			{
				if (accessPointer->type == access_ext)
				{
					if (accessPointer->remark[0] == 0)
						sprintf(tempString, "%s %s", settings_accessext_table, accessPointer->name);
					else
						sprintf(tempString, "%s %s (%s)", settings_accessext_table, accessPointer->name, accessPointer->remark);
					outputFriendly(tempString, tempString2, maxSize, reportFormat);
					output_table(reportFile, reportFormat, true, passwords, tempString2, &settings_accessextpix_heading, tableNum, friendlyHostname);
					permissionPointer = accessPointer->permission;
					tempInt = 1;
					while (permissionPointer != 0)
					{
						outputPIXExtendedACL(configPIX, reportFile, reportFormat, tempInt, expandACL, maxSize, permissionPointer->permission, permissionPointer->protocol, permissionPointer->source, permissionPointer->sourceMask, permissionPointer->sourceType, permissionPointer->sourceService, permissionPointer->sourceServiceOp, permissionPointer->dest, permissionPointer->destMask, permissionPointer->destType, permissionPointer->destService, permissionPointer->destServiceOp, permissionPointer->timeRange, permissionPointer->log, permissionPointer->inactive);
						permissionPointer = permissionPointer->next;
						tempInt++;
					}
					output_table(reportFile, reportFormat, false, passwords, tempString2, &settings_accessextpix_heading, tableNum, friendlyHostname);
					tableNum++;
				}
				accessPointer = accessPointer->next;
			}
		}
		output_endsection(reportFile, reportFormat, subsection_end);
	}

	// ICMP Access
	if (configPIX->icmp != 0)
	{
		output_parseTitle(settings_icmpaccess_title, reportFile, reportFormat, friendlyHostname, rate_none, section_config);
		// Intro
		output_parseText(settings_icmpaccess_desc, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, 0, 0);
		output_table(reportFile, reportFormat, true, passwords, settings_icmpaccess_table, &settings_icmpaccess_heading, tableNum, friendlyHostname);
		icmpPIXPointer = configPIX->icmp;
		while (icmpPIXPointer != 0)
		{
			fprintf(reportFile, "%s%s%s", tablerow_start, icmpPIXPointer->source, tablerow_mid);
			outputFriendly(icmpPIXPointer->interface, tempString, maxSize, reportFormat);
			fprintf(reportFile, "%s", tempString);
			if (icmpPIXPointer->icmpType[0] != 0)
				fprintf(reportFile, "%s%s%s", tablerow_mid, icmpPIXPointer->icmpType, tablerow_end);
			else
				fprintf(reportFile, "%sany%s", tablerow_mid, tablerow_end);
			icmpPIXPointer = icmpPIXPointer->next;
		}
		output_table(reportFile, reportFormat, false, passwords, settings_icmpaccess_table, &settings_icmpaccess_heading, tableNum, friendlyHostname);
		tableNum++;
		output_endsection(reportFile, reportFormat, subsection_end);
	}

	// Protocol Analysis
	if (configPIX->fixup != 0)
	{
		output_parseTitle(settings_protoinspect_title, reportFile, reportFormat, friendlyHostname, rate_none, section_config);
		// Intro
		output_parseText(settings_protoinspect_desc, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, 0, 0);
		if (configPIX->fixup != 0)
		{
			output_table(reportFile, reportFormat, true, passwords, settings_protoinspect_table, &settings_protoinspect_heading, tableNum, friendlyHostname);
			fixupPIXPointer = configPIX->fixup;
			while (fixupPIXPointer != 0)
			{
				fprintf(reportFile, "%s%s%s", tablerow_start, fixupPIXPointer->protocol, tablerow_mid);
				addAbbreviation(fixupPIXPointer->protocol, false);
				if (fixupPIXPointer->enabled == true)
					fprintf(reportFile, "Yes");
				else
					fprintf(reportFile, "No");
				fprintf(reportFile, "%s%s%s", tablerow_mid, fixupPIXPointer->options, tablerow_end);
				fixupPIXPointer = fixupPIXPointer->next;
			}
			output_table(reportFile, reportFormat, false, passwords, settings_protoinspect_table, &settings_protoinspect_heading, tableNum, friendlyHostname);
			tableNum++;
		}
		output_endsection(reportFile, reportFormat, subsection_end);
	}

	// Group Objects
	if (configPIX->objectGroup != 0)
	{
		output_parseTitle(settings_objects_title, reportFile, reportFormat, friendlyHostname, rate_none, section_config);
		// Intro
		output_parseText(settings_objects_desc, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, 0, 0);
		output_list(reportFile, reportFormat, &settings_objects_heading);
		objectGroupPointer = configPIX->objectGroup;
		while (objectGroupPointer != 0)
		{
			if (objectGroupPointer->type == object_type_network)
			{
				if (objectGroupPointer->description[0] == 0)
					sprintf(tempString, "Network %s %s", settings_objects_table, objectGroupPointer->name);
				else
					sprintf(tempString, "Network %s %s (%s)", settings_objects_table, objectGroupPointer->name, objectGroupPointer->description);
			}
			else if (objectGroupPointer->type == object_type_protocol)
			{
				if (objectGroupPointer->description[0] == 0)
					sprintf(tempString, "Protocol %s %s", settings_objects_table, objectGroupPointer->name);
				else
					sprintf(tempString, "Protocol %s %s (%s)", settings_objects_table, objectGroupPointer->name, objectGroupPointer->description);
			}
			else if (objectGroupPointer->type == object_type_service)
			{
				if (objectGroupPointer->description[0] == 0)
					sprintf(tempString, "Service %s %s", settings_objects_table, objectGroupPointer->name);
				else
					sprintf(tempString, "Service %s %s (%s)", settings_objects_table, objectGroupPointer->name, objectGroupPointer->description);
			}
			else if (objectGroupPointer->type == object_type_icmp)
			{
				if (objectGroupPointer->description[0] == 0)
					sprintf(tempString, "ICMP %s %s", settings_objects_table, objectGroupPointer->name);
				else
					sprintf(tempString, "ICMP %s %s (%s)", settings_objects_table, objectGroupPointer->name, objectGroupPointer->description);
			}
			outputFriendly(tempString, tempString2, maxSize, reportFormat);
			output_table(reportFile, reportFormat, true, passwords, tempString2, &settings_object_heading, tableNum, friendlyHostname);
			objectPointer = objectGroupPointer->object;
			while (objectPointer != 0)
			{
				switch (objectPointer->type)
				{
					case object_type_group:
						outputFriendly(objectPointer->object, tempString, maxSize, reportFormat);
						fprintf(reportFile, "%sObject Group%s%s%s", tablerow_start, tablerow_mid, tempString, tablerow_end);
						break;
					case object_type_port:
						fprintf(reportFile, "%sPort%s%s%s", tablerow_start, tablerow_mid, objectPointer->object, tablerow_end);
						addAbbreviation(objectPointer->object, false);
						break;
					case object_type_service:
						fprintf(reportFile, "%sService%s%s%s", tablerow_start, tablerow_mid, objectPointer->object, tablerow_end);
						addAbbreviation(objectPointer->object, false);
						break;
					case object_type_network:
						fprintf(reportFile, "%sNetwork%s%s %s%s", tablerow_start, tablerow_mid, objectPointer->object, objectPointer->netMask, tablerow_end);
						break;
					case object_type_host:
						fprintf(reportFile, "%sHost%s%s%s", tablerow_start, tablerow_mid, objectPointer->object, tablerow_end);
						break;
					case object_type_protocol:
						fprintf(reportFile, "%sProtocol%s%s%s", tablerow_start, tablerow_mid, objectPointer->object, tablerow_end);
						break;
					case object_type_icmp:
						fprintf(reportFile, "%sICMP%s%s%s", tablerow_start, tablerow_mid, objectPointer->object, tablerow_end);
						addAbbreviation("ICMP", false);
						break;
				}
				objectPointer = objectPointer->next;
			}
			output_table(reportFile, reportFormat, false, passwords, tempString2, &settings_object_heading, tableNum, friendlyHostname);
			tableNum++;
			objectGroupPointer = objectGroupPointer->next;
		}
		output_endsection(reportFile, reportFormat, subsection_end);
	}

	// IP Address - Name Mappings
	if (configPIX->nameList != 0)
	{
		output_parseTitle(settings_namemap_title, reportFile, reportFormat, friendlyHostname, rate_none, section_config);
		output_table(reportFile, reportFormat, true, passwords, settings_namemap_table, &settings_namemap_heading, tableNum, friendlyHostname);
		namePIXPointer = configPIX->nameList;
		while (namePIXPointer != 0)
		{
			fprintf(reportFile, "%s%s%s%s%s", tablerow_start, namePIXPointer->address, tablerow_mid, namePIXPointer->name, tablerow_end);
			namePIXPointer = namePIXPointer->next;
		}
		output_table(reportFile, reportFormat, false, passwords, settings_namemap_table, &settings_namemap_heading, tableNum, friendlyHostname);
		tableNum++;
		output_endsection(reportFile, reportFormat, subsection_end);
	}

	// End of Configuration Section
	output_endsection(reportFile, reportFormat, section_end);

	// Appendix
	if (appendix == true)
		generateAppendix(reportFile, reportFormat, friendlyHostname, tableNum, abs, commonPorts, glossary);

	// Output end of file stuff
	output_fileend(reportFormat, reportFile);
}
