/***************************************************************************
 *   Copyright (C) 2006 - 2007 by Ian Ventura-Whiting (Fizz)               *
 *   Fizz@Titania.co.uk                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

// Defines...
#define false 0
#define true 1
#define unconfigured 2

#define output_text 0
#define output_html 1
#define output_latex 2
// 3 is reserved!
#define output_xml 4

#define type_router 0			// Cisco IOS
#define type_switch 1			// Cisco IOS
#define type_firewall 2			// Cisco PIX
#define type_catalyst 3			// Cisco NMP
#define type_screenos 4			// Juniper ScreenOS

#define firewall_type_pix 0
#define firewall_type_asa 1
#define firewall_type_fwsm 2

#define auth_none 0
#define auth_clear 1
#define auth_md5 2

#define encrypt_type7 0
#define encrypt_md5 1
#define encrypt_des56 2
#define encrypt_none 3
#define encrypt_unknown 4
#define encrypt_pix 5
#define encrypt_mschap 6
#define encrypt_ntenc 7

#define perm_permit 0
#define perm_deny 1

#define perm_op_none 0
#define perm_op_lt 1
#define perm_op_gt 2
#define perm_op_eq 3
#define perm_op_neq 4
#define perm_op_range 5


// Structs...
struct host							// Host
{
	char server[32];
	struct host *next;
};


struct netInterface					// Interfaces
{
	char interface[32];
	struct netInterface *next;
};


struct latexConfig					// Latex Configuration
{
	char paperSize[32];
	char documentClass[32];
};


struct ciscoIOSConfig					// Cisco IOS configuration
{
	// Device versions...
	char hostname[128];
	char version[6];
	int versionMajor;
	int versionMinor;

	// Program options...
	struct johnPassword *john;

	// Counters...
	int simplePasswords;
	int passwordStrengths;

	// Configuration settings...
	int sourceRouting;					// true or false
	int tcpSmallServers;				// true, false, unconfigured
	int udpSmallServers;				// true, false, unconfigured
	int finger;							// true, false, unconfigured
	int cdp;							// true or false
	int tftp;							// true, false, unconfigured
	int telnet;							// true, false, unconfigured
	int bootp;							// true or false
	int servicePasswordEncryption;		// true or false
	int bootNetwork;					// true or false
	int serviceConfig;					// true or false
	int pad;							// true or false
	int domainLookup;					// true or false
	struct ntpConfig *ntp;				// NTP
	struct enablePassword *enable;		// Enable Passwords
	struct accessList *access;			// Access Lists
	struct snmpConfig *snmp;			// SNMP
	struct ftpConfig *ftp;				// FTP
	struct usernameList *username;		// Users
	struct privilageLevels *privilage;	// Privilege Levels
	struct loggingConfig *logging;		// Logging
	struct tacacsConfig *tacacs;		// TACACS
	struct aaaConfig *aaa;				// AAA
	struct httpConfig *http;			// HTTP
	struct sshConfig *ssh;				// SSH
	struct interfaceConfig *interface;	// Interfaces
	struct ripConfig *rip;				// Routing - RIP
	struct ospfConfig *ospf;			// Routing - OSPF
	struct eigrpConfig *eigrp;			// Routing - EIGRP
	struct isisConfig *isis;			// Routing - ISIS
	struct bgpConfig *bgp;				// Routing - BGP
	struct keyChain *chain;				// Key Chain
	struct lineConfig *line;			// Local / Remote Administration Lines
	struct bannerConfig *banner;		// Logon Banner
	struct staticRoute *route;			// Static Routing
};


struct ciscoPIXConfig		// Cisco PIX configuration
{
	// Device versions...
	char hostname[128];
	char version[10];
	int versionMajor;
	int versionMinor;
	int versionRevision;
	int type;				// firewall_type_pix...

	// Program options...
	struct johnPassword *john;

	// Counters
	int oldAccessList;
	int standardAccessList;
	int extendedAccessList;
	int interfaceAccessList;

	// Configuration settings...
	char domainName[40];
	char password[32];
	int transparent;						// Transparent Firewall (true or false)
	struct enablePIXPassword *enable;		// Enable Passwords
	struct interfacePIXConfig *interface;	// Interfaces
	struct accessListPIX *access;			// Access Lists
	int accessListAlertInterval;
	int accessListDenyFlowMax;
	struct sshPIXConfig *ssh;				// SSH Configuration
	struct objectGroupList *objectGroup;	// Object Groups
	struct namePIXList *nameList;			// Name mappings
	struct usernamePIX *username;			// Users
	struct fixupList *fixup;				// Protocol Analysis
	struct icmpAccess *icmp;				// ICMP to interfaces
	struct staticPIX *staticNAT;			// Static NAT / PAT
	struct routePIX *route;					// Static Routes
};


struct ciscoNMPConfig			// Cisco NMP Configuration
{
	// Device versions...
	char hostname[128];
	char version[10];
	int versionMajor;
	int versionMinor;
	int versionRevision;

	// Program options...
	struct johnPassword *john;

	// Counters...
	int simplePasswords;
	int passwordStrengths;

	// Configuration settings...
	char location[64];
	char contact[64];
	int timeout;
	char coreFile[32];
	char syslogFile[32];
	char password[32];
	char enablePassword[32];
	int cdp;
	int cdpInterval;
	int cdpHoldtime;
	int cdpVersion;
	int portSecurityAuto;
	struct switchNMPConfig *switchConfig;
	struct snmpNMP *snmp;
	struct moduleConfig *module;
	struct permitConfig *permit;
	struct vlanConfig *vlan;
};


struct hostNMP				// NMP Host Struct
{
	char host[32];
	char netMask[16];
	struct hostNMP *next;
};


struct screenOSConfig					// Juniper ScreenOS Configuration
{
	char hostname[128];
	struct adminSOS *admin;				// Admin settings
	struct authServerSOS *authServer;	// Authentication settings
	struct interfaceSOS *interface;		// Interfaces
	struct policySOS *policies;			// Zone Policy Settings
	struct addressSOS *nameList;		// Address name mappings

	// SSH Settings
	int sshVersion;						// SSH Server Protocol Version
	int sshEnabled;						// SSH Server Enabled/Disabled true or false
	int sshKeyGenTime;					// Mins for Key Gen - SSH V1 Only
	char sshPublicKey[128];				// Public Key - SSH v2 Only

	// Program options...
	struct johnPassword *john;
};


struct listSOS				// NetScreenOS List
{
	char name[32];
	struct listSOS *next;
};


// Used to break each line up
struct ciscoCommand
{
	int parts;
	char part[40][128];
};


// Encrypted passwords for john
struct johnPassword
{
	char username[128];
	char password[128];
	struct johnPassword *next;
};


// SNMP Traps
struct snmpTrap
{
	char trap[128];
	int enabled;			// true, false (Only used by NMP)
	struct snmpTrap *next;
};


#define object_type_network 0
#define object_type_protocol 1
#define object_type_icmp 2
#define object_type_service 3
#define object_type_group 4
#define object_type_port 5
#define object_type_host 6

#define object_service_none 0
#define object_service_tcp 1
#define object_service_udp 2
#define object_service_both 3


struct objectGroupList
{
	int type;					// object_type_network...
	int serviceType;			// object_service_tcp...
	char name[64];
	char description[64];
	struct objectList *object;
	struct objectGroupList *next;
};


struct objectList
{
	char object[64];
	char netMask[16];
	int type;
	struct objectList *next;
};
