/***************************************************************************
 *   Copyright (C) 2006-2007 by Ian Ventura-Whiting (Fizz)                 *
 *   Fizz@Titania.co.uk                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#define section_none 0
#define section_obs 1
#define section_imp 2
#define section_ease 3
#define section_rec 4
#define subsection_end 5
#define section_end 6

#define rate_none 0
#define rate_low 1
#define rate_lowmed 2
#define rate_med 3
#define rate_medhigh 4
#define rate_high 5
#define rate_noease 6
#define rate_trivial 7
#define rate_moderate 8
#define rate_difficult 9


// Make text format friendly
// -------------------------
// This routing will convert the text into a version that
// is friendly to whatever format the report is to be
// output in.
void outputFriendly(char *line, char *output, int maxSize, int outputFormat)
{
	// Variables...
	int loop;

	switch (outputFormat)
	{
		case output_xml:
		case output_html:
			htmlFriendly(line, output, maxSize);
			break;

		case output_latex:
			latexFriendly(line, output, maxSize);
			break;

		default:
			for (loop = 0; loop < maxSize - 1; loop++)
			{
				output[loop] = line[loop];
			}
			output[loop] = 0;
			break;
	}
}


// Insert Bullet List
void output_list(FILE *reportFile, int outputFormat, struct tableHeading *listitems)
{
	// Variables
	struct tableHeading *listPointer;

	// Draw list start
	switch (outputFormat)
	{
		case output_html:
			fprintf(reportFile, "\n<ul>\n");
			break;
		case output_latex:
			fprintf(reportFile, "\n\\begin{itemize}\n");
			break;
		case output_text:
			fprintf(reportFile, "\n\n");
		case output_xml:
			fprintf(reportFile, "\n   <list>\n");
			break;
	}

	// Draw list items
	listPointer = listitems;
	while (listPointer != 0)
	{
		switch (outputFormat)
		{
			case output_html:
				fprintf(reportFile, "<li>%s</li>\n", listPointer->heading);
				break;
			case output_latex:
				fprintf(reportFile, "\\item %s\n", listPointer->heading);
				break;
			case output_text:
				fprintf(reportFile, "  * %s\n", listPointer->heading);
				break;
			case output_xml:
				fprintf(reportFile, "    <listitem>%s</listitem>\n", listPointer->heading);
				break;
		}
		listPointer = listPointer->next;
	}

	// Draw list end
	switch (outputFormat)
	{
		case output_html:
			fprintf(reportFile, "</ul>\n");
			break;
		case output_latex:
			fprintf(reportFile, "\\end{itemize}\n");
			break;
		case output_text:
			fprintf(reportFile, "\n");
			break;
		case output_xml:
			fprintf(reportFile, "\n   </list>\n");
			break;
	}
}


// Insert Start / End of Table
// ---------------------------
// This code inserts the start of end of a table into the
// report output.
void output_table(FILE *reportFile, int outputFormat, int tableStart, int passwords, const char *tableTitle, struct tableHeading *heading, int tableNum, const char *deviceName)
{
	// Variables
	struct tableHeading *headingPointer;
	int tempInt;
	int total;
	int remainder;

	// Table start...
	if (tableStart == true)
	{
		switch (outputFormat)
		{
			case output_html:
				fprintf(reportFile, "<table width=\"100%%\">\n <caption>Table %d: %s.</caption>\n <thead>\n  <tr>", tableNum, tableTitle);
				headingPointer = heading;
				while (headingPointer != 0)
				{
					if ((passwords == true) || (headingPointer->password == false))
						fprintf(reportFile, "  <th>%s</th>", headingPointer->heading);
					headingPointer = headingPointer->next;
				}
				fprintf(reportFile, "</tr>\n</thead>\n <tbody>\n");
				break;

			case output_xml:
				fprintf(reportFile, "    <table number=\"%d\" ref=\"TABLEREF%d\">\n", tableNum, tableNum);
				fprintf(reportFile, "     <caption>%s</caption>\n     <tableheadings>\n", tableTitle);
				headingPointer = heading;
				while (headingPointer != 0)
				{
					if ((passwords == true) || (headingPointer->password == false))
						fprintf(reportFile, "      <tableheading>%s</tableheading>\n", headingPointer->heading);
					headingPointer = headingPointer->next;
				}
				fprintf(reportFile, "     </tableheadings>\n     <tablebody>\n");
				break;

			case output_latex:
				// Print start of table
				fprintf(reportFile, "\\begin{table}[h!]\n");		//, deviceName, tableNum);
				fprintf(reportFile, "\\begin{tabular}{|");
				headingPointer = heading;
				while (headingPointer != 0)
				{
					if ((passwords == true) || (headingPointer->password == false))
						fprintf(reportFile, " c |");
					headingPointer = headingPointer->next;
				}
				fprintf(reportFile, "}\n\\hline\n");
				headingPointer = heading;
				while (headingPointer != 0)
				{
					if ((passwords == true) || (headingPointer->password == false))
					{
						if (headingPointer->next != 0)
							fprintf(reportFile, "%s&", headingPointer->heading);
						else
							fprintf(reportFile, "%s\\\\\n", headingPointer->heading);
					}
					headingPointer = headingPointer->next;
				}
				fprintf(reportFile, "\\hline\n");
				break;

			case output_text:
				total = 0;
				tempInt = 0;
				// Count headings length
				headingPointer = heading;
				while (headingPointer != 0)
				{
					if ((passwords == true) || (headingPointer->password == false))
					{
						total += strlen(headingPointer->heading);
						tempInt++;
					}
					headingPointer = headingPointer->next;
				}
				if (tableNum < 10)
					remainder = 1;
				else if (tableNum < 100)
					remainder = 2;
				else if (tableNum < 1000)
					remainder = 3;
				else
					remainder = 4;
				total += (tempInt * 2) - 2;
				// Is table title or headings longest
				if ((strlen(tableTitle) + 8 + remainder) > total)
					total = strlen(tableTitle) + 8 + remainder;
				// Print start of table
				for (tempInt = 0; tempInt < total; tempInt++)
				{
					fprintf(reportFile, "-");
				}
				fprintf(reportFile, "\n");
				headingPointer = heading;
				while (headingPointer != 0)
				{
					if ((passwords == true) || (headingPointer->password == false))
					{
						if (headingPointer->next != 0)
							fprintf(reportFile, "%s, ", headingPointer->heading);
						else
							fprintf(reportFile, "%s", headingPointer->heading);
					}
					headingPointer = headingPointer->next;
				}
				fprintf(reportFile, "\n");
				for (tempInt = 0; tempInt < total; tempInt++)
				{
					fprintf(reportFile, "-");
				}
				fprintf(reportFile, "\n");
				break;
		}
	}

	// Table end...
	else
	{
		switch (outputFormat)
		{
			case output_html:
				fprintf(reportFile, " </tbody>\n</table>\n&nbsp;<br>\n\n");
				break;

			case output_xml:
				fprintf(reportFile, "     </tablebody>\n    </table>\n");
				break;

			case output_latex:
				fprintf(reportFile, "\\end{tabular}\n");
				fprintf(reportFile, "\\caption{%s}\n", tableTitle);
				fprintf(reportFile, "\\end{table}\n\n");
				break;

			case output_text:
				total = 0;
				tempInt = 0;
				// Count headings length
				headingPointer = heading;
				while (headingPointer != 0)
				{
					if ((passwords == true) || (headingPointer->password == false))
					{
						total += strlen(headingPointer->heading);
						tempInt++;
					}
					headingPointer = headingPointer->next;
				}
				if (tableNum < 10)
					remainder = 1;
				else if (tableNum < 100)
					remainder = 2;
				else if (tableNum < 1000)
					remainder = 3;
				else
					remainder = 4;
				total += (tempInt * 2) - 2;
				// Is table title or headings longest
				if ((strlen(tableTitle) + 8 + remainder) > total)
					total = strlen(tableTitle) + 8 + remainder;
				for (tempInt = 0; tempInt < total; tempInt++)
				{
					fprintf(reportFile, "-");
				}
				fprintf(reportFile, "\nTable %d: %s\n\n", tableNum, tableTitle);
				break;
		}
	}
}


// Insert New Paragrapgh
// ---------------------
// This code simply inserts a new paragraph.
void output_newPara(FILE *reportFile, int outputFormat)
{
	switch (outputFormat)
	{
		case output_html:
			fprintf(reportFile, "<br>\n&nbsp;<br>\n");
			break;
		case output_latex:
		case output_text:
			fprintf(reportFile, "\n\n");
			break;
	}
}

#define section_main_sec 0
#define section_main_con 1
#define section_main_app 2
#define section_security 3
#define section_config 4
#define section_appendix 5

// Report Index Parsing and Output
// -------------------------------
// This code will output the index text and create hyperlinks (where required) for
// the different output formats.
void output_parseIndex(const char *theText, FILE *reportFile, int outputFormat, char *deviceName, int section)
{
	// Variables...
	int textLoop;

	switch (outputFormat)
	{
		case output_html:
			if (section == section_security)
				fprintf(reportFile, "<li><a href=\"#sec_");
			else
				fprintf(reportFile, "<li><a href=\"#con_");
			for (textLoop = 0; textLoop < strlen(theText); textLoop++)
			{
				if (theText[textLoop] != ' ')
					fprintf(reportFile, "%c", theText[textLoop]);
			}
			fprintf(reportFile, "\">%s</a></li>\n", theText);
			break;

		case output_text:
			fprintf(reportFile, "  * %s\n", theText);
			break;
	}
}

// Report Title Parsing and Output
// -------------------------------
// This code will output the section title and create a link destination (where
// required) for the different output formats.
void output_parseTitle(const char *theText, FILE *reportFile, int outputFormat, char *deviceName, int rating, int section)
{
	// Variables...
	int textLoop;
	char tempChar;

	switch (outputFormat)
	{
		case output_latex:
			if ((section == section_main_sec) || (section == section_main_con) || (section == section_main_app))
				fprintf(reportFile, "\\section[");
			else
				fprintf(reportFile, "\\subsection[");
			for (textLoop = 0; textLoop < strlen(theText); textLoop++)
			{
				if (theText[textLoop] != ' ')
					fprintf(reportFile, "%c", theText[textLoop]);
			}
			fprintf(reportFile, "%s]\n  {%s}\n\n", deviceName, theText);
			break;

		case output_html:
			if ((section == section_main_sec) || (section == section_main_con) || (section == section_main_app))
				fprintf(reportFile, "<hr><h2><a name=\"");
			else if (section == section_security)
				fprintf(reportFile, "<h3><a name=\"sec_");
			else if (section == section_config)
				fprintf(reportFile, "<h3><a name=\"con_");
			else
				fprintf(reportFile, "<h3><a name=\"app_");
			for (textLoop = 0; textLoop < strlen(theText); textLoop++)
			{
				if (theText[textLoop] != ' ')
					fprintf(reportFile, "%c", theText[textLoop]);
			}
			if ((section == section_main_sec) || (section == section_main_con) || (section == section_main_app))
				fprintf(reportFile, "\">%s</a></h2>\n", theText);
			else
				fprintf(reportFile, "\">%s</a></h3>\n", theText);
			break;

		case output_xml:
			if (section == section_main_sec)
				fprintf(reportFile, " <section type=\"security\"");
			else if (section == section_main_con)
				fprintf(reportFile, " <section type=\"configuration\"");
			else if (section == section_main_app)
				fprintf(reportFile, " <section type=\"appendix\"");
			else if (section == section_security)
			{
				fprintf(reportFile, "  <subsection type=\"security\" rate=\"");
				switch (rating)
				{
					case rate_low:
						fprintf(reportFile, "low");
						break;
					case rate_lowmed:
						fprintf(reportFile, "lowmed");
						break;
					case rate_med:
						fprintf(reportFile, "med");
						break;
					case rate_medhigh:
						fprintf(reportFile, "medhigh");
						break;
					case rate_high:
						fprintf(reportFile, "high");
						break;
				}
				fprintf(reportFile, "\"");
			}
			else if (section == section_config)
				fprintf(reportFile, "  <subsection type=\"configuration\"");
			else
				fprintf(reportFile, "  <subsection type=\"appendix\"");

			fprintf(reportFile, " title=\"%s\" ref=\"", theText);
			for (textLoop = 0; textLoop < strlen(theText); textLoop++)
			{
				if (theText[textLoop] != ' ')
					fprintf(reportFile, "%c", theText[textLoop]);
			}
			fprintf(reportFile, "%d%s\">\n", section + 48, deviceName);
			break;

		case output_text:
			if ((section == section_main_sec) || (section == section_main_con) || (section == section_main_app))
				tempChar = '~';
			else
				tempChar = '-';
			fprintf(reportFile, "%s\n", theText);
			for (textLoop = 0; textLoop < strlen(theText); textLoop++)
			{
				fprintf(reportFile, "%c", tempChar);
			}
			fprintf(reportFile, "\n\n");
			break;
	}
}


// Report End of Section
// ---------------------
// This routine outputs an end of section.
void output_endsection(FILE *reportFile, int outputFormat, int reportSection)
{
	// Format...
	switch (outputFormat)
	{
		case output_xml:
			switch (reportSection)
			{
				case section_obs:
				case section_imp:
				case section_ease:
					fprintf(reportFile, "   </textsection>\n");
					break;

				case section_rec:
					fprintf(reportFile, "   </textsection>\n");
					fprintf(reportFile, "  </subsection>\n");
					break;

				case subsection_end:
					fprintf(reportFile, "  </subsection>\n");
					break;

				case section_end:
					fprintf(reportFile, " </section>\n");
					break;
			}
			break;
	}
}


// Report Text Parsing and Output
// ------------------------------
// This code outputs text to the report. The text could be a security report
// paragraph or just plain text. It will use the embeded codes to format the
// text for each output type. The current codes include:
//     COMPANY           - Company name
//     DEVICENAME        - The device name
//     ABBREV ^          - Abbreviation
//     COMMAND ^         - A command
//     CMDOPTION ^       - A command option
//     NEWLINE           - A newline
//     SINGULA ^         - One item
//     PLAURAL ^         - More than one item
//     NEGATIVE ^        - Supplied item is negative
//     TABLEREF          - Insert Table Reference
//     NUMBER            - Insert Number (provided by data)
//     NONEWPARA         - No new paragraph at the end
//     NEWPARA           - Insert New Paragraph
// Each code is encolsed in * and codes that have a start and end [^} use the
// same code with a - prepending it.
void output_parseText(const char *theText, FILE *reportFile, int outputFormat, const char *companyName, char *deviceName, int reportSection, int rating, int data, int reference)
{
	// Variables...
	int offset;
	int textLength;
	int abbreviationStatus;
	char *charPointer;
	char tempString[12];
	const char *abbreviation;

	// Section...
	switch (reportSection)
	{
		case section_obs:
			switch (outputFormat)
			{
				case output_xml:
					fprintf(reportFile, "   <textsection type=\"observation\">\n    <text>");
					break;
				case output_html:
					fprintf(reportFile, "<b>Observation:</b> ");
					break;
				case output_latex:
				case output_text:
					fprintf(reportFile, "OBSERVATION: ");
					break;
			}
			break;

		case section_imp:
			switch (outputFormat)
			{
				case output_xml:
					fprintf(reportFile, "   <textsection type=\"impact\" rate=\"");
					switch (rating)
					{
						case rate_low:
							fprintf(reportFile, "low");
							break;
						case rate_lowmed:
							fprintf(reportFile, "lowmed");
							break;
						case rate_med:
							fprintf(reportFile, "med");
							break;
						case rate_medhigh:
							fprintf(reportFile, "medhigh");
							break;
						case rate_high:
							fprintf(reportFile, "high");
							break;
					}
					fprintf(reportFile, "\">\n    <text>");
					break;
				case output_html:
					fprintf(reportFile, "<b>Impact:</b> ");
					break;
				case output_latex:
				case output_text:
					fprintf(reportFile, "IMPACT: ");
					break;
			}
			break;

		case section_ease:
			switch (outputFormat)
			{
				case output_xml:
					fprintf(reportFile, "   <textsection type=\"ease\" rate=\"");
					switch (rating)
					{
						case rate_noease:
							fprintf(reportFile, "noease ");
							break;
						case rate_trivial:
							fprintf(reportFile, "trivial");
							break;
						case rate_moderate:
							fprintf(reportFile, "moderate");
							break;
						case rate_difficult:
							fprintf(reportFile, "difficult");
							break;
					}
					fprintf(reportFile, "\">\n    <text>");
					break;
				case output_html:
					if (rating == rate_noease)
						fprintf(reportFile, "<b>Ease:</b> N/A");
					else
						fprintf(reportFile, "<b>Ease:</b> ");
					break;
				case output_latex:
				case output_text:
					if (rating == rate_noease)
						fprintf(reportFile, "EASE: N/A");
					else
						fprintf(reportFile, "EASE: ");
					break;
			}
			break;

		case section_rec:
			switch (outputFormat)
			{
				case output_html:
					fprintf(reportFile, "<b>Recommendation:</b> ");
					break;
				case output_xml:
					fprintf(reportFile, "   <textsection type=\"recommendation\">\n    <text>");
					break;
				case output_latex:
				case output_text:
					fprintf(reportFile, "RECOMMENDATION: ");
					break;
			}
			break;

		default:
			switch (outputFormat)
			{
				case output_xml:
					if ((theText != report_timeout_obs2) && (theText != report_timeout_rec2) && (theText != report_http_rec2) && (theText != settings_accesslists2) && (theText != report_http_impact2) && (theText != report_http_impact3) && (theText != report_http_rec2))
						fprintf(reportFile, "    <text>");
					break;
			}
			break;
	}

	// Parse the text
	textLength = strlen(theText);
	for (offset = 0; offset < textLength; offset++)
	{
		// Init...
		memset(tempString, 0, sizeof(tempString));

		// Company name...
		if (strncmp("*COMPANY*", theText + offset, 9) == 0)
		{
			switch (outputFormat)
			{
				default:
					fprintf(reportFile, "%s", companyName);
					break;
			}
			offset += 8;
		}

		// Device name...
		else if (strncmp("*DEVICENAME*", theText + offset, 12) == 0)
		{
			switch (outputFormat)
			{
				default:
					fprintf(reportFile, "%s", deviceName);
					break;
			}
			offset += 11;
		}

		// Newline...
		else if (strncmp("*NEWLINE*", theText + offset, 9) == 0)
		{
			switch (outputFormat)
			{
				case output_html:
					fprintf(reportFile, "&nbsp;<br>\n");
					break;
				default:
					fprintf(reportFile, "\n");
					break;
			}
			offset += 8;
		}

		// New paragraph...
		else if (strncmp("*NEWPARA*", theText + offset, 9) == 0)
		{
			switch (outputFormat)
			{
				case output_html:
					fprintf(reportFile, "<br>\n&nbsp;<br>\n");
					break;
				case output_xml:
					fprintf(reportFile, "</text>\n    <text>");
					break;
				default:
					fprintf(reportFile, "\n\n");
					break;
			}
			offset += 8;
		}

		// Singula...
		else if (strncmp("*SINGULA*", theText + offset, 9) == 0)
		{
			// If not singula, skip the entire section
			if (data != 1)
			{
				charPointer = strstr(theText + offset, "*-SINGULA*");
				if (charPointer != 0)
					offset = charPointer - theText + 9;
			}
			else
				offset += 8;
		}
		else if (strncmp("*-SINGULA*", theText + offset, 10) == 0)
			offset += 9;

		// Plaural...
		else if (strncmp("*PLAURAL*", theText + offset, 9) == 0)
		{
			// If not plaural, skip the entire section
			if (data < 2)
			{
				charPointer = strstr(theText + offset, "*-PLAURAL*");
				if (charPointer != 0)
					offset = charPointer - theText + 9;
			}
			else
				offset += 8;
		}
		else if (strncmp("*-PLAURAL*", theText + offset, 10) == 0)
			offset += 9;

		// Negative...
		else if (strncmp("*NEGATIVE*", theText + offset, 10) == 0)
		{
			// If not negative, skip the entire section
			if (data > 0)
			{
				charPointer = strstr(theText + offset, "*-NEGATIVE*");
				if (charPointer != 0)
					offset = charPointer - theText + 10;
			}
			else
				offset += 9;
		}
		else if (strncmp("*-NEGATIVE*", theText + offset, 11) == 0)
			offset += 10;

		// Command...
		else if (strncmp("*COMMAND*", theText + offset, 9) == 0)
		{
			switch (outputFormat)
			{
				case output_html:
					fprintf(reportFile, "<quote>");
					break;
				case output_latex:
					fprintf(reportFile, "\\texttt{");
					break;
				case output_xml:
					fprintf(reportFile, "<command>");
					break;
				default:
					fprintf(reportFile, "\"");
					break;
			}
			offset += 8;
		}
		else if (strncmp("*-COMMAND*", theText + offset, 10) == 0)
		{
			switch (outputFormat)
			{
				case output_html:
					fprintf(reportFile, "</quote>");
					break;
				case output_latex:
					fprintf(reportFile, "}");
					break;
				case output_xml:
					fprintf(reportFile, "</command>");
					break;
				default:
					fprintf(reportFile, "\"");
					break;
			}
			offset += 9;
		}

		// Command Option...
		else if (strncmp("*CMDOPTION*", theText + offset, 11) == 0)
		{
			switch (outputFormat)
			{
				case output_xml:
				case output_html:
					fprintf(reportFile, "&lt;");
					break;
				default:
					fprintf(reportFile, "<");
					break;
			}
			offset += 10;
		}
		else if (strncmp("*-CMDOPTION*", theText + offset, 12) == 0)
		{
			switch (outputFormat)
			{
				case output_xml:
				case output_html:
					fprintf(reportFile, "&gt;");
					break;
				default:
					fprintf(reportFile, ">");
					break;
			}
			offset += 11;
		}

		// Table Reference...
		else if (strncmp("*TABLEREF*", theText + offset, 10) == 0)
		{
			switch (outputFormat)
			{
				case output_xml:
					fprintf(reportFile, "<reference name=\"TABLEREF%d\"/>", reference);
					break;
				default:
					fprintf(reportFile, "%d", reference);
					break;
			}
			offset += 9;
		}

		// Number...
		else if (strncmp("*NUMBER*", theText + offset, 8) == 0)
		{
			numToWord(reportFile, data);
			offset += 7;
		}

		// Abbreviation...
		else if (strncmp("*ABBREV*", theText + offset, 8) == 0)
		{
			// Get abbreviation...
			offset += 8;
			charPointer = strstr(theText + offset, "*-ABBREV*");
			if (charPointer != 0)
			{
				strncpy(tempString, theText + offset, charPointer - theText - offset);
				offset = charPointer - theText + 8;
				abbreviationStatus = addAbbreviation(&tempString[0], true);

				// Process
				switch (outputFormat)
				{
					case output_xml:
						abbreviation = getAbbreviation(&tempString[0]);
						if (abbreviationStatus == abbrev_expand)
							fprintf(reportFile, "<abbrev expand=\"true\" long=\"%s\" short=\"%s\" />", abbreviation, tempString);
						else
							fprintf(reportFile, "<abbrev expand=\"false\" long=\"%s\" short=\"%s\" />", abbreviation, tempString);
						break;
					default:
						switch (abbreviationStatus)
						{
							case abbrev_expand:
								abbreviation = getAbbreviation(&tempString[0]);
								// If ACLs or ACEs
								if (strcmp(tempString, "ACLs") == 0)
									sprintf(tempString, "ACL");
								else if (strcmp(tempString, "ACEs") == 0)
									sprintf(tempString, "ACE");
								if (abbreviation != 0)
									fprintf(reportFile, "%s (%s)", abbreviation, tempString);
								else
									fprintf(reportFile, "%s", tempString);
								break;
							default:
								fprintf(reportFile, "%s", tempString);
								break;
						}
						break;
				}
			}
		}
		else if (strncmp("*-ABBREV*", theText + offset, 9) == 0)
		{
			offset += 8;
		}

		// No new paragraph at end
		else if (strncmp("*NONEWPARA*", theText + offset, 11) == 0)
			return;

		// Output text
		else
		{
			fprintf(reportFile, "%c", theText[offset]);
		}
	}

	// End text
	switch (outputFormat)
	{
		case output_xml:
			fprintf(reportFile, "</text>\n");
			break;
	}
	output_newPara(reportFile, outputFormat);
}


// Insert File Start
// -----------------
// This code inserts the start of the file stuff
void output_filestart(FILE *reportFile, int reportFormat, int deviceType, char *friendlyHostname, int fullReport, FILE *cssFile, struct latexConfig *latex, const char *companyName, int maxSize)
{
	// Variables
	char tempString[maxSize];

	switch (reportFormat)
	{
		case output_html:
			fprintf(reportFile, "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\">\n<html>\n <head>\n  <title>%s %s ", companyName, friendlyHostname);
			if (fullReport == true)
				fprintf(reportFile, "Security Report");
			else
				fprintf(reportFile, "Device Configuration");
			fprintf(reportFile, "</title>\n  <style type='text/css'>\n  <!--\n  ");
			if (cssFile == 0)
				fprintf(reportFile, "%s", html_css);
			else
			{
				// Read line
				readLine(cssFile, tempString, maxSize);
				while (feof(cssFile) == 0)
				{
					fprintf(reportFile, "%s\n", tempString);
					// Read line, then write
					readLine(cssFile, tempString, maxSize);
				}
			}
			fprintf(reportFile, "\n  \n  --></style>\n </head>\n <body>\n<h1>%s ", friendlyHostname);
			if (fullReport == true)
				fprintf(reportFile, "Security Audit</h1>\n");
			else
				fprintf(reportFile, "Device Configuration</h1>\n");
			break;

		case output_xml:
			fprintf(reportFile, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
			if (fullReport == true)
				fprintf(reportFile, "<document title=\"Security Audit\" device=\"%s\">\n", friendlyHostname);
			else
				fprintf(reportFile, "<document title=\"Device Configuration\" device=\"%s\">\n", friendlyHostname);
			break;

		case output_text:
			if (fullReport == true)
				fprintf(reportFile, "Security Audit\n==============\n\n");
			else
				fprintf(reportFile, "Device Configuration\n=============\n\n");
			break;

		case output_latex:
			fprintf(reportFile, "\\documentclass[%s]{%s}\n\n\\begin{document}\n\n", latex->paperSize, latex->documentClass);
			break;
	}

	// Output Document Introduction (if required)
	switch (reportFormat)
	{
		case output_latex:
		case output_html:
		case output_text:
		case output_xml:
			if (fullReport == true)
			{
				switch (deviceType)
				{
					case type_router:
					case type_switch:
					case type_firewall:
					case type_catalyst:
						output_parseText(report_cisco_intro, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, 0, 0);
						break;
					case type_screenos:
						output_parseText(report_screenos_intro, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, 0, 0);
						break;
				}
			}
			else
			{
				switch (deviceType)
				{
					case type_router:
					case type_switch:
					case type_firewall:
					case type_catalyst:
						output_parseText(report_cisco_intro_configonly, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, 0, 0);
						break;
					case type_screenos:
						output_parseText(report_screenos_intro_configonly, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, 0, 0);
						break;
				}
			}
			break;
	}
}


// Insert File End
// ---------------
// This code inserts the end of the file stuff
void output_fileend(int reportFormat, FILE *reportFile)
{
	// Output file end
	if (reportFormat == output_html)
		fprintf(reportFile, "</body>\n</html>\n");
	else if (reportFormat == output_latex)
		fprintf(reportFile, "\\end{document}\n");
	else if (reportFormat == output_xml)
		fprintf(reportFile, "</document>\n");

	// Close file
	//if (outputFile[0] != 0)
	fclose(reportFile);
}


// Generate Appendix
// -----------------
// This code generates the appendix section.
void generateAppendix(FILE *reportFile, int reportFormat, char *friendlyHostname, int tableNum, int showAbbrev, int showPorts, int showGlos)
{
	// Variables
	int tempInt;
	struct abbrev *abbrevPointer;
	struct ports *portPointer;

	// Table Pointers
	const char *tablerow_start;
	const char *tablerow_mid;
	const char *tablerow_end;
	switch (reportFormat)
	{
		case output_html:
			tablerow_start = table_html_rowstart;
			tablerow_mid = table_html_rowmid;
			tablerow_end = table_html_rowend;
			break;
		case output_xml:
			tablerow_start = table_xml_rowstart;
			tablerow_mid = table_xml_rowmid;
			tablerow_end = table_xml_rowend;
			break;
		case output_text:
			tablerow_start = table_text_rowstart;
			tablerow_mid = table_text_rowmid;
			tablerow_end = table_text_rowend;
			break;
		case output_latex:
			tablerow_start = table_latex_rowstart;
			tablerow_mid = table_latex_rowmid;
			tablerow_end = table_latex_rowend;
			break;
	}

	// Check that there is something to show
	if ((showAbbrev == true) || (showPorts == true) || (showGlos == true))
	{

		// Display Section Start
		output_parseTitle("Appendix", reportFile, reportFormat, friendlyHostname, rate_none, section_main_app);

		// Abbreviations
		abbrevPointer = &ab_append;
		while ((abbrevPointer->next != 0) && (abbrevPointer->add == false))
			abbrevPointer = abbrevPointer->next;
		if ((showAbbrev == true) && (abbrevPointer->add == true))
		{
			output_parseTitle(appendix_abbrev_title, reportFile, reportFormat, friendlyHostname, rate_none, section_appendix);

			// Display Start of abbreviations
			switch (reportFormat)
			{
				case output_html:
					fprintf(reportFile, "<table cellspacing=\"2\" cellpadding=\"2\" border=\"0\" bgcolor=\"white\">\n<tbody>\n");
					break;
				case output_xml:
					fprintf(reportFile, "   <abbreviations>\n");
					break;
				case output_text:
					fprintf(reportFile, "-------------\nAbbreviations\n-------------\n");
					break;
				case output_latex:
					fprintf(reportFile, "\\begin{quote}\n\\begin{tabular}{ll}\n");
					break;
			}

			// Display Abbreviations
			while (abbrevPointer != 0)
			{
				if (abbrevPointer->add == true)
				{
					switch (reportFormat)
					{
						case output_html:
							fprintf(reportFile, "<tr><td><b>%s</b></td><td>%s</td></tr>\n", abbrevPointer->ab, abbrevPointer->desc);
							break;
						case output_xml:
							fprintf(reportFile, "    <abbreviation short=\"%s\">%s</abbreviation>\n", abbrevPointer->ab, abbrevPointer->desc);
							break;
						case output_text:
							fprintf(reportFile, "  %s", abbrevPointer->ab);
							tempInt = 10 - strlen(abbrevPointer->ab);
							while (tempInt != 0)
							{
								fprintf(reportFile, " ");
								tempInt--;
							}
							fprintf(reportFile, ": %s\n", abbrevPointer->desc);
							break;
						case output_latex:
							fprintf(reportFile, "%s&%s\\\\\n", abbrevPointer->ab, abbrevPointer->desc);
							break;
					}
				}
				abbrevPointer = abbrevPointer->next;
			}

			// Display End of Abbreviations
			switch (reportFormat)
			{
				case output_html:
					fprintf(reportFile, "</tbody>\n</table>\n&nbsp;<br>\n\n");
					break;
				case output_xml:
					fprintf(reportFile, "   </abbreviations>\n");
					break;
				case output_text:
					fprintf(reportFile, "-------------\n\n");
					break;
				case output_latex:
					fprintf(reportFile, "\\end{tabular}\n\\end{quote}\n\n");
					break;
			}

			// Output the end of the abbreviations section
			output_endsection(reportFile, reportFormat, subsection_end);
		}

		// Common Ports
		portPointer = &ap_port;
		while ((portPointer->next != 0) && (portPointer->add == false))
			portPointer = portPointer->next;
		if ((showPorts == true) && (portPointer->add == true))
		{
			output_parseTitle(appendix_ports_title, reportFile, reportFormat, friendlyHostname, rate_none, section_appendix);

			// Display Ports
			output_table(reportFile, reportFormat, true, 0, appendix_ports_table, &appendix_ports_heading, tableNum, friendlyHostname);
			while (portPointer != 0)
			{
				if (portPointer->add == true)
					fprintf(reportFile, "%s%s%s%d%s", tablerow_start, portPointer->text, tablerow_mid, portPointer->port, tablerow_end);
				portPointer = portPointer->next;
			}
			output_table(reportFile, reportFormat, false, 0, appendix_ports_table, &appendix_ports_heading, tableNum, friendlyHostname);

			// Output the end of the ports section
			output_endsection(reportFile, reportFormat, subsection_end);
		}

		// Output section end
		output_endsection(reportFile, reportFormat, section_end);
	}
}
